@@ -1,8 +1,29 @@
+/***************************************************************************
+                                d2asc.cpp
+                             -------------------
+    begin                : Tue Aug 22 13:46:13 CST 2000
+    copyright            : (C) 2000 by Barth Netterfield
+    email                :
+ ***************************************************************************/
+
+/***************************************************************************
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ ***************************************************************************/
+
 #include <stdlib.h> // atoi
+#include <kconfig.h>
 #include <kinstance.h>
 
+// hack to make main() a friend of kstdatasource
+#define protected public
 #include "kstrvector.h"
 #include "kstdatacollection.h"
+#undef protected
 
 void Usage() {
   fprintf(stderr, "usage: d2asc filename [-f <first frame>]\n");
@@ -11,11 +32,22 @@ void Usage() {
   fprintf(stderr, "   -x specifies that the field should be printed in hex\n");
 }
 
+
+static void exitHelper() {
+  KST::vectorList.clear();
+  KST::scalarList.clear();
+  KST::dataObjectList.clear();
+}
+
 int main(int argc, char *argv[]) {
+  atexit(exitHelper);
   KInstance inst("d2asc");
   KstDataSourcePtr file;
   int i;
 
+  KConfig *kConfigObject = new KConfig("kstdatarc", false, false);
+  KstDataSource::setupOnStartup(kConfigObject);
+
   char field_list[40][120], filename[180];
   bool do_hex[40];
   int n_field=0;
@@ -70,7 +102,6 @@ int main(int argc, char *argv[]) {
   }
   /** make vectors and fill the list **/
   QPtrList<KstRVector> vlist;
-  vlist.setAutoDelete(true);
 
   for (i=0; i<n_field; i++) {
 
@@ -90,8 +121,8 @@ int main(int argc, char *argv[]) {
     while (vlist.at(i)->update(-1) != KstObject::NO_CHANGE)
       ; // read vector
 
-    if (vlist.at(i)->sampleCount() > NS)
-      NS = vlist.at(i)->sampleCount();
+    if (vlist.at(i)->length() > NS)
+      NS = vlist.at(i)->length();
   }
 
   for (i_S = 0; i_S < NS; i_S++) {
