@@ -23,36 +23,55 @@
 
 class AsciiSource : public KstDataSource {
   public:
-    AsciiSource(KConfig *cfg, const QString& filename, const QString& type);
+    AsciiSource(KConfig *cfg, const QString& filename, const QString& type, const QDomElement& e = QDomElement());
 
-    virtual ~AsciiSource();
+    ~AsciiSource();
 
-    virtual bool initFile();
+    bool initRowIndex();
 
-    virtual KstObject::UpdateType update(int = -1);
+    KstObject::UpdateType update(int = -1);
 
-    virtual int readField(double *v, const QString &field, int s, int n);
+    int readField(double *v, const QString &field, int s, int n);
 
-    virtual bool isValidField(const QString &field) const;
+    bool isValidField(const QString &field) const;
 
-    virtual int samplesPerFrame(const QString &field);
+    int samplesPerFrame(const QString &field);
 
-    virtual int frameCount(const QString& field = QString::null) const;
+    int frameCount(const QString& field = QString::null) const;
 
-    virtual QString fileType() const;
+    QString fileType() const;
 
-    virtual QStringList fieldList() const;
+    QStringList fieldList() const;
 
-    virtual void save(QTextStream &ts, const QString& indent = QString::null);
+    void save(QTextStream &ts, const QString& indent = QString::null);
 
-    virtual bool isEmpty() const;
+    bool isEmpty() const;
+
+    bool supportsTimeConversions() const;
+
+    int sampleForTime(int seconds, bool *ok);
+
+    int sampleForTime(const QDateTime& time, bool *ok);
+
+    bool fieldListIsComplete() const;
+
+    bool reset();
+
+    class Config;
+    static QStringList fieldListFor(const QString& filename, Config *cfg);
 
   private:
     int *_rowIndex;
     int _numLinesAlloc;
     int _numFrames;
     int _byteLength;
-    QString _delimiters;
+    mutable QStringList _fields;
+    friend class ConfigWidgetAscii;
+    mutable Config *_config;
+    char *_tmpBuf;
+    uint _tmpBufSize;
+    bool _haveHeader;
+    mutable bool _fieldListComplete;
 };
 
 
