@@ -17,7 +17,9 @@
 
 #include "indirect.h"
 
+#include <qdir.h>
 #include <qfile.h>
+#include <qfileinfo.h>
  
 
 IndirectSource::IndirectSource(KConfig *cfg, const QString& filename, KstDataSourcePtr child)
@@ -36,11 +38,19 @@ IndirectSource::~IndirectSource() {
 
 
 KstObject::UpdateType IndirectSource::update(int u) {
+  if (KstObject::checkUpdateCounter(u)) {
+    return lastUpdateResult();
+  }
+
   // recheck the indirect file for a changed filename
   QFile f(_filename);
   if (f.open(IO_ReadOnly)) {
     QString ifn;
     if (0 < f.readLine(ifn, 1000)) {
+      if (QFileInfo(ifn).isRelative()) {
+        ifn = QFileInfo(_filename).dirPath(true) + QDir::separator() + ifn;
+      }
+
       if (!_child || ifn.stripWhiteSpace() != _child->fileName()) {
         _child = 0L; // release
         KstDataSourcePtr p = KstDataSource::loadSource(ifn.stripWhiteSpace());
@@ -55,7 +65,7 @@ KstObject::UpdateType IndirectSource::update(int u) {
     }
   }
 
-  return _child ? _child->update(u) : KstObject::NO_CHANGE;
+  return setLastUpdateResult(_child ? _child->update(u) : KstObject::NO_CHANGE);
 }
 
 
@@ -69,6 +79,11 @@ bool IndirectSource::isValidField(const QString& field) const {
 }
 
 
+bool IndirectSource::reset() {
+  return _child ? _child->reset() : KstDataSource::reset();
+}
+
+
 int IndirectSource::samplesPerFrame(const QString &field) {
   return _child ? _child->samplesPerFrame(field) : 0;
 }
@@ -115,6 +130,10 @@ KstDataSource *create_indirect(KConfig *cfg, const QString& filename, const QStr
     return 0L;
   }
 
+  if (QFileInfo(ifn).isRelative()) {
+    ifn = QFileInfo(filename).dirPath(true) + QDir::separator() + ifn;
+  }
+
   KstDataSourcePtr p = KstDataSource::loadSource(ifn.stripWhiteSpace());
   f.close();
 
@@ -130,8 +149,10 @@ QStringList provides_indirect() {
 
 
 int understands_indirect(KConfig*, const QString& filename) {
-  if (!filename.endsWith(".cur")) { // Do we really have to do this?
-    return 0;
+  int percent = 50;
+
+  if (filename.endsWith(".cur")) { // Do we really have to do this?
+    percent = 100;
   }
 
   QFile f(filename);
@@ -144,7 +165,11 @@ int understands_indirect(KConfig*, const QString& filename) {
     return 0;
   }
 
-  return QFile::exists(ifn.stripWhiteSpace()) ? 100 : 0;
+  if (QFileInfo(ifn).isRelative()) {
+    ifn = QFileInfo(filename).dirPath(true) + QDir::separator() + ifn;
+  }
+
+  return QFile::exists(ifn.stripWhiteSpace()) ? percent : 0;
 }
 
 
@@ -163,6 +188,10 @@ QStringList fieldList_indirect(KConfig *cfg, const QString& filename, const QStr
     return QStringList();
   }
 
+  if (QFileInfo(ifn).isRelative()) {
+    ifn = QFileInfo(filename).dirPath(true) + QDir::separator() + ifn;
+  }
+
   return KstDataSource::fieldListForSource(ifn.stripWhiteSpace(), type, typeSuggestion, complete);
 }
 
