@@ -121,11 +121,12 @@ int PlanckSource::readField(double *v, const QString& field, int s, int n) {
     return 0;
   }
 
+  assert(n != 0);
   return _planckObject->readObject(field, v, start + s, start + s + n - 1);
 }
 
 
-int PlanckSource::readField(double *v, const QString &field, int s, int n, int skip, int *lastFrameRead) {
+int PlanckSource::readField(double *v, const QString& field, int s, int n, int skip, int *lastFrameRead) {
   if (*lastFrameRead) {
     *lastFrameRead = -1;
   }
@@ -159,12 +160,18 @@ int PlanckSource::readField(double *v, const QString &field, int s, int n, int s
     return 0;
   }
 
+  if (skip > n || skip > count || skip > end) { // what are they thinking?
+    return 0;
+  }
+
   if (n < 0) { // reading less than 0 -> read only one sample!
     n = 1;
   }
 
-  if (s + (n + 1)*skip > count) { // trying to read past the end
-    n = -1 + (count - s) / skip;
+  if (s + (n-1)*skip > count) { // trying to read past the end
+    kdDebug() << "TRYING TO READ PAST END.  n=" << n << endl;
+    n = (count - s) / skip + 1;
+    kdDebug() << "N IS NOW=" << n << endl;
   }
 
   if (s + start > end) {
@@ -176,10 +183,9 @@ int PlanckSource::readField(double *v, const QString &field, int s, int n, int s
   // later.  For now, we read into a temp buffer, then extract what we need.
   double *tmp = new double[(n + 1) * skip];
   int rc = _planckObject->readObject(field, tmp, start + s, start + s + (n + 1)*skip - 1);
+  //kdDebug() << "readObject rc=" << rc << " from=" << start+s << " to=" << start + s + (n+1)*skip -1 << endl;
   int i = 0;
-  int rctmp = rc;
-  while (rctmp >= skip) {
-    rctmp -= skip;
+  while (i < n && i*skip < rc) {
     v[i] = tmp[i * skip];
     ++i;
   }
