@@ -6,14 +6,15 @@
 ** init() function in place of a constructor, and a destroy() function in
 ** place of a destructor.
 *****************************************************************************/
-static const QString& defaultTag = KGlobal::staticQString("<Auto Name>");
+const QString& DataWizard::defaultTag = KGlobal::staticQString("<Auto Name>");
 
 void DataWizard::init()
 {
     _configWidget = 0L;
+    _inTest = false;
     KST::vectorDefaults.sync();
     QString default_source = KST::vectorDefaults.dataSource();
-    _url->setMode(KFile::File | KFile::Directory | KFile::ExistingOnly | KFile::LocalOnly);
+    _url->setMode(KFile::File | KFile::Directory | KFile::ExistingOnly);
     setAppropriate(_pageFilters, false);
     setIcon(BarIcon("kst_datawizard"));
 
@@ -29,6 +30,8 @@ void DataWizard::init()
 
     disconnect(finishButton(), SIGNAL(clicked()), (QDialog*)this, SLOT(accept()));
     connect(finishButton(), SIGNAL(clicked()), this, SLOT(finished()));
+    _vectors->addColumn(i18n("Position"));
+    _vectors->setSorting(1);
 
     // No help button
     setHelpEnabled(_pageDataSource, false);
@@ -43,6 +46,11 @@ void DataWizard::init()
 }
 
 
+void DataWizard::plotColsChanged() {
+  _reGrid->setChecked(true);
+}
+
+
 bool DataWizard::xVectorOk()
 {
     QString txt = _xVector->currentText();
@@ -72,7 +80,7 @@ bool DataWizard::yVectorsOk()
 }
 
 
-void DataWizard::xChanged( const QString & txt )
+void DataWizard::xChanged(const QString& txt)
 {
     bool found = false;
 
@@ -87,44 +95,70 @@ void DataWizard::xChanged( const QString & txt )
 }
 
 
-void DataWizard::sourceChanged( const QString &txt )
+void DataWizard::testURL()
 {
-    delete _configWidget; // FIXME: very inefficient
+    _inTest = true;
+    sourceChanged(_url->url());
+    _inTest = false;
+}
+
+
+void DataWizard::sourceChanged(const QString& text)
+{
+    delete _configWidget;
     _configWidget = 0L;
     _configureSource->setEnabled(false);
     _file = QString::null;
-    if (!txt.isEmpty()) {
+    if (!text.isEmpty() && text != "stdin" && text != "-") {
 	KURL url;
+	QString txt = _url->completionObject()->replacedPath(text);
 	if (QFile::exists(txt) && QFileInfo(txt).isRelative()) {
 	    url.setPath(txt);
 	} else {
 	    url = KURL::fromPathOrURL(txt);
 	}
 
-	if (!url.isValid() || (!url.protocol().isEmpty() && url.protocol() != "file")) {
+	if (!_inTest && !url.isLocalFile() && url.protocol() != "file" && !url.protocol().isEmpty()) {
+	    setNextEnabled(_pageDataSource, false);
+	    _fileType->setText(QString::null);
+	    _testURL->show();
+	    return;
+	}
+
+	if (!_inTest) {
+	    _testURL->hide();
+	}
+
+	if (!url.isValid()) {
 	    setNextEnabled(_pageDataSource, false);
 	    _fileType->setText(QString::null);
 	    return;
 	}
 
-	QString file = url.path();
+	QString file = txt;
 	
-	KstDataSourcePtr ds = *KST::dataSourceList.findFileName(file);
+	KstDataSourcePtr ds = *KST::dataSourceList.findReusableFileName(file);
 	QStringList fl;
 	bool complete = false;
 	QString fileType;
+	int index = 0;
+	int count;
 
 	if (ds) {
 	    fl = ds->fieldList();
 	    fileType = ds->fileType();
 	    complete = ds->fieldListIsComplete();
 	    ds = 0L;
-	} else { // FIXME: deal with stdin?
+	} else {
 	    fl = KstDataSource::fieldListForSource(file, QString::null, &fileType, &complete);
 	}
 
 	if (!fl.isEmpty() && !fileType.isEmpty()) {
-	    _configWidget = KstDataSource::configWidgetForSource(file, fileType);
+	    if (ds) {
+		_configWidget = ds->configWidget();
+	    } else {
+		_configWidget = KstDataSource::configWidgetForSource(file, fileType);
+	    }
 	}
 
 	_configureSource->setEnabled(_configWidget);
@@ -142,9 +176,22 @@ void DataWizard::sourceChanged( const QString &txt )
 	_xVector->insertStringList(fl);
 	_xVector->completionObject()->insertItems(fl);
 	_xVector->setEditable(!complete);
-	for (QStringList::ConstIterator it = fl.begin(); it != fl.end(); ++it) {
-	    new QCheckListItem(_vectors, *it, QCheckListItem::CheckBox);
+
+	count = fl.count();
+	if (count > 0) {
+	    count = 1+(int)log10((double)count);
+	} else {
+	    count = 1;
 	}
+	for (QStringList::ConstIterator it = fl.begin(); it != fl.end(); ++it) {
+	    QCheckListItem *pItem = new QCheckListItem(_vectors, *it, QCheckListItem::CheckBox);
+	    QString str;
+	    str.sprintf("%0*d", count, index);
+	    pItem->setText(1, str);
+	    index++;
+	}
+	_vectors->sort();
+
 	KST::vectorDefaults.sync();
 	QString defaultX = KST::vectorDefaults.wizardXVector();
 	if (defaultX == "INDEX" || fl.contains(defaultX)) {
@@ -158,6 +205,9 @@ void DataWizard::sourceChanged( const QString &txt )
     }
     setNextEnabled(_pageVectors, xVectorOk() && yVectorsOk());
     setNextEnabled(_pageDataSource, true);
+    if (_inTest) {
+	_testURL->hide();
+    }
 }
 
 
@@ -172,6 +222,11 @@ void DataWizard::fieldListChanged()
 void DataWizard::showPage( QWidget *page )
 {
     if (page == _pageVectors) {
+	KstDataSourcePtr ds = *KST::dataSourceList.findReusableFileName(_file);
+	if (!ds) {
+	    ds = KstDataSource::loadSource(_file);
+	}
+	_kstDataRange->setAllowTime(ds && ds->supportsTimeConversions());
 	_vectorReduction->setFocus();
     } else if (page == _pageFilters) {
 	QString save = _filterList->currentText();
@@ -193,7 +248,7 @@ void DataWizard::showPage( QWidget *page )
 
 	_psdAxisGroup->setEnabled(_radioButtonPlotDataPSD->isChecked() || _radioButtonPlotPSD->isChecked());
 
-	// set window and plot defaults based on some guesses....
+	// set window and plot defaults based on some guesses.
 	if (_radioButtonPlotDataPSD->isChecked()) { // plotting both psds and XY plots
 	    _newWindows->setEnabled(true);
 	    if (n_curves > 1) {
@@ -209,6 +264,7 @@ void DataWizard::showPage( QWidget *page )
 
 	updateWindowBox();
 	updatePlotBox();
+	updateColumns();
     }
 
     QWizard::showPage(page);
@@ -239,6 +295,29 @@ void DataWizard::updateWindowBox()
 }
 
 
+void DataWizard::updateColumns()
+{
+    if (_newWindow->isChecked() || _newWindows->isChecked()) {
+	_reGrid->setChecked(false);
+	return;
+    }
+
+    KstViewWindow *v;
+    if (_currentWindow->isChecked()) {
+	v = static_cast<KstViewWindow*>(KstApp::inst()->activeWindow());
+    } else {
+	v = static_cast<KstViewWindow*>(KstApp::inst()->findWindow(_windowName->currentText()));
+    }
+
+    if (v) {
+	if (v->view()->onGrid()) {
+	    _plotColumns->setValue(v->view()->columns());
+	    _reGrid->setChecked(true);
+	}
+    }
+}
+
+
 void DataWizard::updatePlotBox()
 {
     QString psave = _existingPlotName->currentText();
@@ -288,6 +367,7 @@ void DataWizard::updatePlotBox()
     if (_existingPlot->isEnabled() && _existingPlotName->listBox()->findItem(psave)) {
 	_existingPlotName->setCurrentText(psave);
     }
+    updateColumns();
 }
 
 
@@ -309,7 +389,7 @@ void DataWizard::vectorSubset(const QString& filter)
 {
     QListViewItem *after = 0L;
     _vectors->clearSelection();
-    _vectors->setSorting(2, true); // Qt 3.1 compat
+    _vectors->setSorting(3, true); // Qt 3.1 compat
     QRegExp re(filter, true /* case insensitive */, true /* wildcard */);
     QListViewItemIterator it(_vectors);
     while (it.current()) {
@@ -354,7 +434,6 @@ void DataWizard::checkSelected()
 
 void DataWizard::newFilter()
 {
-  // FIXME
 }
 
 
@@ -362,7 +441,7 @@ void DataWizard::finished()
 {
     KstApp *app = KstApp::inst();
     KstVectorList l;
-    QString name = QString("V%1-%2").arg(KST::vectorList.count()).arg(_xVector->currentText());
+    QString name = KST::suggestVectorName(_xVector->currentText());
     QValueList<QColor> colors;
     QColor color;
     uint n_curves = 0;
@@ -370,11 +449,11 @@ void DataWizard::finished()
     int ptype = 0;
     int prg = 0;
 
-    KstDataSourcePtr ds = *KST::dataSourceList.findFileName(_file);
+    KstDataSourcePtr ds = *KST::dataSourceList.findReusableFileName(_file);
     if (!ds) {
 	ds = KstDataSource::loadSource(_file);
 	if (!ds) {
-	    KMessageBox::sorry(this, i18n("Sorry, unable to load the data file."));
+	    KMessageBox::sorry(this, i18n("<qt>Sorry, unable to load the data file '<b>%1</b>'.").arg(_file));
 	    return;
 	}
 	KST::dataSourceList.lock().writeLock();
@@ -382,52 +461,63 @@ void DataWizard::finished()
 	KST::dataSourceList.lock().writeUnlock();
     }
 
-    //
-    // check for sufficient memory...
-    //
+    // check for sufficient memory
     unsigned long memoryRequested = 0, memoryAvailable = 1024*1024*1024; // 1GB
     unsigned long frames;
 #ifdef HAVE_LINUX
     meminfo();
     memoryAvailable = S(kb_main_free + kb_main_buffers + kb_main_cached);
 #endif
-    
-    if (_kstDataRange->ReadToEnd->isChecked()) {
-	frames = ds->frameCount(_xVector->currentText()) - _kstDataRange->F0->value();
+
+    int f0Value, nValue;
+    if (_kstDataRange->isTime()) {
+	f0Value = ds->sampleForTime(_kstDataRange->f0Value());
+	nValue = ds->sampleForTime(_kstDataRange->nValue());
     } else {
-	frames = _kstDataRange->N->value();
-	if (frames > ds->frameCount(_xVector->currentText())) {
+	f0Value = _kstDataRange->f0Value();
+	nValue = _kstDataRange->nValue();
+    }
+
+    if (_kstDataRange->ReadToEnd->isChecked()) {
+	frames = ds->frameCount(_xVector->currentText()) - f0Value;
+    } else {
+	frames = _kstDataRange->nValue();
+	if (frames > (unsigned long)ds->frameCount(_xVector->currentText())) {
 	    frames = ds->frameCount(_xVector->currentText());
 	}
     }
 
     if (_kstDataRange->DoSkip->isChecked()) {
-	memoryRequested += frames / _kstDataRange->Skip->value() * sizeof(double);;
+	memoryRequested += frames / _kstDataRange->Skip->value() * sizeof(double);
     } else {
 	memoryRequested += frames * ds->samplesPerFrame(_xVector->currentText())*sizeof(double);
     }
 
     {
 	QListViewItemIterator it(_vectors);
+	int fftLen = int(pow(2.0, double(_kstFFTOptions->FFTLen->text().toInt() - 1)));
 	while (it.current()) {
 	    QCheckListItem *i = static_cast<QCheckListItem*>(it.current());
 	    if (i->isOn()) {
 		QString field = it.current()->text(0);
 
 		if (_kstDataRange->ReadToEnd->isChecked()) {
-		    frames = ds->frameCount(field) - _kstDataRange->F0->value();
+		    frames = ds->frameCount(field) - f0Value;
 		} else {
-		    frames = _kstDataRange->N->value();
-		    if (frames > ds->frameCount(field)) {
+		    frames = nValue;
+		    if (frames > (unsigned long)ds->frameCount(field)) {
 			frames = ds->frameCount();
 		    }
 		}
 
 		if (_kstDataRange->DoSkip->isChecked()) {
-		    memoryRequested += frames / _kstDataRange->Skip->value()*sizeof(double);;
+		    memoryRequested += frames / _kstDataRange->Skip->value()*sizeof(double);
 		} else {
 		    memoryRequested += frames * ds->samplesPerFrame(field)*sizeof(double);
-		}                
+		}
+		if (_radioButtonPlotPSD->isChecked() || _radioButtonPlotDataPSD->isChecked()) {
+		    memoryRequested += fftLen * 6;
+		}
 	    }
 	    ++it;
 	}
@@ -440,6 +530,12 @@ void DataWizard::finished()
 
     accept();
 
+    // Pause updates to avoid event storms
+    bool wasPaused = app->paused();
+    if (!wasPaused) {
+	app->setPaused(true);
+    }
+
     {
 	QListViewItemIterator it(_vectors);
 	while (it.current()) {
@@ -454,33 +550,34 @@ void DataWizard::finished()
 	}
     }
 
-    n_steps += 1; // for the creation of the x-vector...
+    n_steps += 1; // for the creation of the x-vector
     prg = 0;
     app->slotUpdateProgress(n_steps, prg, i18n("Creating vectors..."));
 
-    //
-    // create the x-vector...
-    //
+    // create the x-vector
     KstVectorPtr xv = new KstRVector(ds, _xVector->currentText(),
-	    name, _kstDataRange->CountFromEnd->isChecked() ? -1 : _kstDataRange->F0->value(),
-	    _kstDataRange->ReadToEnd->isChecked() ? -1 : _kstDataRange->N->value(),
+	    name, _kstDataRange->CountFromEnd->isChecked() ? -1 : f0Value,
+	    _kstDataRange->ReadToEnd->isChecked() ? -1 : nValue,
 	    _kstDataRange->DoSkip->isChecked() ? _kstDataRange->Skip->value() : 0, _kstDataRange->DoSkip->isChecked(),
 	    _kstDataRange->DoFilter->isChecked());
     KST::addVectorToList(xv);
     app->slotUpdateProgress(n_steps, ++prg, i18n("Creating vectors..."));
 
-    //
-    // create the y-vectors...
-    //
+    // create the y-vectors
     {
 	QListViewItemIterator it(_vectors);
+	int vectorCount;
+
 	while (it.current()) {
 	    QCheckListItem *i = static_cast<QCheckListItem*>(it.current());
 	    if (i->isOn()) {
-		name = QString("V%1-%2").arg(KST::vectorList.count()).arg(it.current()->text(0));
+		vectorCount = KST::vectorList.count();
+
+                name = KST::suggestVectorName(it.current()->text(0));
+                
 		KstVectorPtr v = new KstRVector(ds, it.current()->text(0), name,
-			_kstDataRange->CountFromEnd->isChecked() ? -1 : _kstDataRange->F0->value(),
-			_kstDataRange->ReadToEnd->isChecked() ? -1 : _kstDataRange->N->value(),
+			_kstDataRange->CountFromEnd->isChecked() ? -1 : f0Value,
+			_kstDataRange->ReadToEnd->isChecked() ? -1 : nValue,
 			_kstDataRange->DoSkip->isChecked() ? _kstDataRange->Skip->value() : 0, _kstDataRange->DoSkip->isChecked(),
 			_kstDataRange->DoFilter->isChecked());
 		KST::addVectorToList(v);
@@ -492,9 +589,7 @@ void DataWizard::finished()
 	}
     }
 
-    //
-    // get a pointer to the first window...
-    //
+    // get a pointer to the first window
     QString newName;
     KstViewWindow *w;
     if (_newWindow->isChecked() || _newWindows->isChecked()) {
@@ -510,17 +605,18 @@ void DataWizard::finished()
     }
 
     if (!w) {
+	if (!wasPaused) {
+	    app->setPaused(false);
+	}
 	return;
     }
 
-    //
-    // create the necessary plots...
-    //
+    // create the necessary plots
     app->slotUpdateProgress(n_steps, prg, i18n("Creating plots..."));
     KstViewObjectList plots;
     bool relayout = true;
     if (_onePlot->isChecked()) {
-	Kst2DPlotPtr p = kst_cast<Kst2DPlot>(w->view()->findChild(w->createPlot<Kst2DPlot>(KST::suggestPlotName())));
+	Kst2DPlotPtr p = kst_cast<Kst2DPlot>(w->view()->findChild(w->createPlot<Kst2DPlot>(KST::suggestPlotName(), false)));
 	plots.append(p.data());
 	if (_radioButtonPlotDataPSD->isChecked()) {
 	    if (_newWindows->isChecked()) {
@@ -528,25 +624,30 @@ void DataWizard::finished()
 		QString n = app->newWindow(newName);
 		w = static_cast<KstViewWindow*>(app->findWindow(n));
 	    }
-	    Kst2DPlotPtr p = kst_cast<Kst2DPlot>(w->view()->findChild(w->createPlot<Kst2DPlot>(KST::suggestPlotName())));
+	    Kst2DPlotPtr p = kst_cast<Kst2DPlot>(w->view()->findChild(w->createPlot<Kst2DPlot>(KST::suggestPlotName(), false)));
 	    plots.append(p.data());
+	    p->setXAxisInterpretation(false, KstAxisInterpretation(), KstAxisDisplay(), true, 0.0);
+	    p->setYAxisInterpretation(false, KstAxisInterpretation(), KstAxisDisplay(), true, 0.0);
 	}
     } else if (_multiplePlots->isChecked()) {
 	Kst2DPlotPtr p;
+	for (uint i = 0; i < l.count(); ++i) {
+	    p = kst_cast<Kst2DPlot>(w->view()->findChild(w->createPlot<Kst2DPlot>(KST::suggestPlotName(), false)));
+	    plots.append(p.data());
+	}
 	if (_radioButtonPlotDataPSD->isChecked()) {
-	    for (uint i = 0; i < l.count(); ++i) {
-		p = kst_cast<Kst2DPlot>(w->view()->findChild(w->createPlot<Kst2DPlot>(KST::suggestPlotName())));
-		plots.append(p.data());
-	    }
 	    if (_newWindows->isChecked()) {
+                w->view()->cleanup(signed(sqrt(plots.count())));
 		newName += "-PSD";
 		QString n = app->newWindow(newName);
 		w = static_cast<KstViewWindow*>(app->findWindow(n));
 	    }
-	}
-	for (uint i = 0; i < l.count(); ++i) {
-	    p = kst_cast<Kst2DPlot>(w->view()->findChild(w->createPlot<Kst2DPlot>(KST::suggestPlotName())));
-	    plots.append(p.data());
+	    for (uint i = 0; i < l.count(); ++i) {
+		p = kst_cast<Kst2DPlot>(w->view()->findChild(w->createPlot<Kst2DPlot>(KST::suggestPlotName(), false)));
+		plots.append(p.data());
+		p->setXAxisInterpretation(false, KstAxisInterpretation(), KstAxisDisplay(), true, 0.0);
+		p->setYAxisInterpretation(false, KstAxisInterpretation(), KstAxisDisplay(), true, 0.0);
+	    }
 	}
     } else if (_existingPlot->isChecked()) {
 	Kst2DPlotPtr p = kst_cast<Kst2DPlot>(w->view()->findChild(_existingPlotName->currentText()));
@@ -560,30 +661,31 @@ void DataWizard::finished()
 	relayout = false;
     } else { /* cycle */
 	Kst2DPlotPtr p;
+	for (int i = 0; i < _plotNumber->value(); ++i) {
+	    p = kst_cast<Kst2DPlot>(w->view()->findChild(w->createPlot<Kst2DPlot>(KST::suggestPlotName(), false)));
+	    plots.append(p.data());
+	}
 	if (_radioButtonPlotDataPSD->isChecked()) {
-	    for (int i = 0; i < _plotNumber->value(); ++i) {
-		p = kst_cast<Kst2DPlot>(w->view()->findChild(w->createPlot<Kst2DPlot>(KST::suggestPlotName())));
-		plots.append(p.data());
-	    }
 	    if (_newWindows->isChecked()) {
+                w->view()->cleanup(signed(sqrt(plots.count())));
 		QString n = app->newWindow(newName + "-PSD");
 		w = static_cast<KstViewWindow*>(app->findWindow(n));
 	    }
-	}
-	for (int i = 0; i < _plotNumber->value(); ++i) {
-	    p = kst_cast<Kst2DPlot>(w->view()->findChild(w->createPlot<Kst2DPlot>(KST::suggestPlotName())));
-	    plots.append(p.data());
+	    for (int i = 0; i < _plotNumber->value(); ++i) {
+		p = kst_cast<Kst2DPlot>(w->view()->findChild(w->createPlot<Kst2DPlot>(KST::suggestPlotName(), false)));
+		plots.append(p.data());
+		p->setXAxisInterpretation(false, KstAxisInterpretation(), KstAxisDisplay(), true, 0.0);
+		p->setYAxisInterpretation(false, KstAxisInterpretation(), KstAxisDisplay(), true, 0.0);
+	    }
 	}
     }
 
-    //
-    // create the data curves...
-    //
+    // create the data curves
     app->slotUpdateProgress(n_steps, prg, i18n("Creating curves..."));
     KstViewObjectList::Iterator pit = plots.begin();
     for (KstVectorList::Iterator it = l.begin(); it != l.end(); ++it) {
 	if (_radioButtonPlotData->isChecked() || _radioButtonPlotDataPSD->isChecked()) {
-	    name = KST::suggestCurveName((*it)->tagName());
+          name = KST::suggestCurveName((*it)->tagName(), false);
 	    Kst2DPlotPtr plot = kst_cast<Kst2DPlot>(*pit);
 	    if (plot) {
 		color = KstColorSequence::next(plot->Curves, plot->backgroundColor());
@@ -591,7 +693,7 @@ void DataWizard::finished()
 		color = KstColorSequence::next();
 	    }
 	    colors.append(color);
-	    KstBaseCurvePtr c = new KstVCurve(name, xv, *it, 0L, 0L, color);
+	    KstBaseCurvePtr c = new KstVCurve(name, xv, *it, 0L, 0L, 0L, 0L, color);
 	    if (_drawBoth->isChecked()) {
 		c->setHasPoints(true);
 		c->Point.setType(ptype++ % KSTPOINT_MAXTYPE);
@@ -610,7 +712,7 @@ void DataWizard::finished()
 	    if (plot) {
 		plot->addCurve(c);
 	    }
-	    if (!_onePlot->isChecked()) { // change plots if we are not onePlot...
+	    if (!_onePlot->isChecked()) { // change plots if we are not onePlot
 		if (_radioButtonPlotDataPSD->isChecked()) { // if xy and psd
 		    ++pit;
 		    if (plots.findIndex(*pit) >= (int)plots.count()/2) {
@@ -633,9 +735,7 @@ void DataWizard::finished()
 	pit = plots.at(plots.count()/2);
     }
 
-    //
-    // create the PSDs...
-    //
+    // create the PSDs
     if (_radioButtonPlotPSD->isChecked() || _radioButtonPlotDataPSD->isChecked()) {
 	KstVCurvePtr c;
 	int indexColor = 0;
@@ -668,10 +768,10 @@ void DataWizard::finished()
 			_kstFFTOptions->RemoveMean->isChecked(),
 			_kstFFTOptions->VectorUnits->text(),
 			_kstFFTOptions->RateUnits->text());
-		if (_radioButtonPlotPSD->isChecked() || colors.count() <= indexColor) {
-		    c = new KstVCurve(name + "-curve", p->vX(), p->vY(), 0L, 0L, KstColorSequence::next(plot->Curves, plot->backgroundColor()));
+		if (_radioButtonPlotPSD->isChecked() || colors.count() <= (unsigned long)indexColor) {
+                  c = new KstVCurve(KST::suggestCurveName(name, true), p->vX(), p->vY(), 0L, 0L, 0L, 0L, KstColorSequence::next(plot->Curves, plot->backgroundColor()));
 		} else {
-		    c = new KstVCurve(name + "-curve", p->vX(), p->vY(), 0L, 0L, colors[indexColor]);
+                  c = new KstVCurve(KST::suggestCurveName(name, true), p->vX(), p->vY(), 0L, 0L, 0L, 0L, colors[indexColor]);
 		    indexColor++;
 		}
 		if (_drawBoth->isChecked()) {
@@ -692,7 +792,7 @@ void DataWizard::finished()
 		KST::dataObjectList.lock().writeUnlock();
 		plot->addCurve(c.data());
 	        plot->setLog(_psdLogX->isChecked(),_psdLogY->isChecked());
-		if (!_onePlot->isChecked()) { // change plots if we are not onePlot...
+		if (!_onePlot->isChecked()) { // change plots if we are not onePlot
 		    if (++pit == plots.end()) {
                       if (_radioButtonPlotDataPSD->isChecked()) { // if xy and psd
                         pit = plots.at(plots.count()/2);
@@ -706,9 +806,7 @@ void DataWizard::finished()
 	app->slotUpdateProgress(n_steps, ++prg, i18n("Creating PSDs..."));
     }
 
-    //
-    // legends and labels...
-    //
+    // legends and labels
     bool xl = _xAxisLabels->isChecked();
     bool yl = _yAxisLabels->isChecked();
     bool tl = _plotTitles->isChecked();
@@ -745,11 +843,22 @@ void DataWizard::finished()
 	++pit;
     }
 
-    if (relayout) {
-	w->view()->cleanup(QMAX(w->view()->columns(), signed(sqrt(plots.count()))));
+    if (_reGrid->isChecked()) {
+	w->view()->cleanup(_plotColumns->value());
+    } else if (relayout) {
+	if (_radioButtonPlotDataPSD->isChecked()) {
+	    w->view()->cleanup(signed(sqrt(plots.count()/2)));
+	} else {
+	    w->view()->cleanup(signed(sqrt(plots.count())));
+	}
+    } else if (w->view()->onGrid()) {
+	w->view()->cleanup(-1);
     }
-    w->view()->paint(P_DATA);
+    w->view()->paint(P_PAINT);
     app->slotUpdateProgress(0, 0, QString::null);
+    if (!wasPaused) {
+	app->setPaused(false);
+    }
 }
 
 
@@ -812,18 +921,50 @@ void DataWizard::_enableWindowEntries()
 }
 
 
-void DataWizard::configureSource()
+void DataWizard::markSourceAndSave()
 {
+    assert(_configWidget);
+    KstDataSourcePtr src = static_cast<KstDataSourceConfigWidget*>((QWidget*)_configWidget)->instance();
+    if (src) {
+	src->disableReuse();
+    }
+    static_cast<KstDataSourceConfigWidget*>((QWidget*)_configWidget)->save();
+}
+
+
+void DataWizard::configureSource()
+{
+    bool isNew = false;
+    KST::dataSourceList.lock().readLock();
+    KstDataSourcePtr ds = *KST::dataSourceList.findReusableFileName(_file);
+    KST::dataSourceList.lock().readUnlock();
+    if (!ds) {
+	isNew = true;
+	ds = KstDataSource::loadSource(_file);
+	if (!ds || !ds->isValid()) {
+	    _configureSource->setEnabled(false);
+	    return;
+	}
+    }
+
     assert(_configWidget);
     KDialogBase *dlg = new KDialogBase(this, "Data Config Dialog", true, i18n("Configure Data Source"));
-    connect(dlg, SIGNAL(okClicked()), _configWidget, SLOT(save()));
-    connect(dlg, SIGNAL(applyClicked()), _configWidget, SLOT(save()));
+    if (isNew) {
+	connect(dlg, SIGNAL(okClicked()), _configWidget, SLOT(save()));
+	connect(dlg, SIGNAL(applyClicked()), _configWidget, SLOT(save()));
+    } else {
+	connect(dlg, SIGNAL(okClicked()), this, SLOT(markSourceAndSave()));
+	connect(dlg, SIGNAL(applyClicked()), this, SLOT(markSourceAndSave()));
+    }
     _configWidget->reparent(dlg, QPoint(0, 0));
     dlg->setMainWidget(_configWidget);
+    static_cast<KstDataSourceConfigWidget*>((QWidget*)_configWidget)->setInstance(ds);
+    static_cast<KstDataSourceConfigWidget*>((QWidget*)_configWidget)->load();
     dlg->exec();
     _configWidget->reparent(0L, QPoint(0, 0));
     dlg->setMainWidget(0L);
     delete dlg;
+    sourceChanged(_url->url());
 }
 
 // vim: ts=8 sw=4 noet
