@@ -48,8 +48,13 @@ extern "C" {
   void *NewArgumentList();
   void *NewNumber(double n);
   void *NewNot(void *n);
+  void *NewNegation(void *n);
   void AppendArgument(void *list, void *arg);
 
+  void DeleteNode(void *n); /* WARNING: this does evil things.  will cause a
+                               crash if we ever do MI in the tree */
+  void ParenthesizeNode(void *n);
+
 #ifdef __cplusplus
 }
 #endif
