@@ -27,7 +27,7 @@
 
 // include files for KDE
 #include <kaccel.h>
-#include <kdebug.h>
+#include "ksdebug.h"
 #include <kfiledialog.h>
 #include <kkeydialog.h>
 #include <kmessagebox.h>
@@ -35,6 +35,7 @@
 #include <kprogress.h>
 #include <kstandarddirs.h>
 #include <kstatusbar.h>
+#include <ktabwidget.h>
 #include <qdatetime.h>
 
 // application specific includes
@@ -150,10 +151,13 @@ KstApp::KstApp(QWidget *parent, const char *name)
   connect(KstEventMonitorI::globalInstance(), SIGNAL(modified()), doc, SLOT(wasModified()));
   connect(KstImageDialogI::globalInstance(), SIGNAL(modified()), doc, SLOT(wasModified()));
   connect(KstMatrixDialogI::globalInstance(), SIGNAL(modified()), doc, SLOT(wasModified()));
+  connect(this, SIGNAL(mdiModeHasBeenChangedTo(KMdi::MdiMode)), SLOT(fixKMdi()));
 
   initActions();
   readOptions();
 
+  fixKMdi();
+
   _updateThread = new UpdateThread(doc);
   _updateThread->setUpdateTime(KstSettings::globalSettings()->plotUpdateTimer);
   _updateThread->start();
@@ -229,9 +233,11 @@ KstApp::KstApp(QWidget *parent, const char *name)
 
 
 KstApp::~KstApp() {
-  _updateThread->setFinished(true);
-  if (!_updateThread->wait(3000)) { // 3s
-    _updateThread->terminate();
+  if (_updateThread) {
+    _updateThread->setFinished(true);
+    if (!_updateThread->wait(3000)) { // 3s
+      _updateThread->terminate();
+    }
   }
 
   KstDataSource::cleanupForExit(); // must be before deletions
@@ -1096,9 +1102,11 @@ void KstApp::immediatePrintActiveWindowToPng(const QString& filename, const QStr
 }
 
 void KstApp::slotFilePrint() {
+  KstViewWindow *currentWin = dynamic_cast<KstViewWindow*>(activeWindow());
   KstViewWindow *win;
+  int currentPage = 0;  
   int pages = 0;
-
+  
   KMdiIterator<KMdiChildView*> *it = createIterator();
   if (it) {
     while (it->currentItem()) {
@@ -1106,6 +1114,9 @@ void KstApp::slotFilePrint() {
       if (win && !win->view()->children().isEmpty()) {
         pages++;
       }
+      if (win == currentWin) {
+        currentPage = pages;
+      }
       it->next();
     }
     deleteIterator(it);
@@ -1114,7 +1125,7 @@ void KstApp::slotFilePrint() {
   if (pages > 0) {
     KPrinter printer;
     KstSettings *ks = KstSettings::globalSettings();
-
+        
     printer.setOption("kde-pagesize", ks->printing.pageSize);
     printer.setOption("kde-orientation", ks->printing.orientation);
     printer.setOption("kst-plot-datetime-footer", ks->printing.plotDateTimeFooter);
@@ -1123,6 +1134,8 @@ void KstApp::slotFilePrint() {
     printer.setOption("kst-plot-monochrome", ks->printing.monochrome);
     printer.setFromTo(0, 0);
     printer.setMinMax(1, pages);
+    printer.setCurrentPage(currentPage);
+    printer.setPageSelection(KPrinter::ApplicationSide);
 
     pages = 0;
     printer.addDialogPage(new KstPrintOptionsPage);
@@ -1147,6 +1160,17 @@ void KstApp::slotFilePrint() {
       ks->printing.monochrome = printer.option("kst-plot-monochrome");
       ks->save();
 
+#if KDE_VERSION < KDE_MAKE_VERSION(3,3,0)
+      int iFromPage = printer.fromPage();
+      int iToPage = printer.toPage();
+      
+      if (iFromPage == 0 && iToPage == 0) {
+        printer.setPageSelection(KPrinter::SystemSide);
+      }
+#else
+      QValueList<int> pageList = printer.pageList();
+#endif
+      
       it = createIterator();
       if (it) {
         bool firstPage = true;
@@ -1154,12 +1178,15 @@ void KstApp::slotFilePrint() {
           win = dynamic_cast<KstViewWindow*>(it->currentItem());
           if (win && !win->view()->children().isEmpty()) {
             pages++;
+#if KDE_VERSION < KDE_MAKE_VERSION(3,3,0)
             if ((iFromPage == 0 && iToPage == 0) ||
                 (iFromPage <= pages && iToPage >= pages)) {
+#else
+            if (pageList.contains(pages)) {                   
+#endif
               if (!firstPage && !printer.newPage()) {
                 break;
               }
-
               KstTopLevelViewPtr tlv = kst_cast<KstTopLevelView>(win->view());
               if (tlv) {
                 if (lineAdjust != 0) {
@@ -1438,7 +1465,7 @@ void KstApp::paintAll(KstPaintType pt) {
   }
 #ifdef PAINTTIMER
   int x = t.elapsed();
-  kdDebug() << "paintAll with painttype " << (int)pt << " - " << x << "ms" << endl;
+  kstdDebug() << "paintAll with painttype " << (int)pt << " - " << x << "ms" << endl;
 #endif
 }
 
@@ -1585,7 +1612,7 @@ void KstApp::updateDialogs(bool onlyVisible) {
     if (!onlyVisible || dataManager->isShown()) {
       dataManager->update();
     }
-    kdDebug() << "Dialogs updated in " << t.elapsed() << "ms" << endl;
+    kstdDebug() << "Dialogs updated in " << t.elapsed() << "ms" << endl;
   }
 }
 
@@ -1633,7 +1660,7 @@ void KstApp::showDataWizard() {
 }
 
 void KstApp::registerDocChange() {
-  kdDebug() << "register doc changed" << endl;
+  kstdDebug() << "register doc changed" << endl;
   forceUpdate();
   updateVisibleDialogs();
   doc->setModified();
@@ -1891,10 +1918,12 @@ void KstApp::updateMemoryStatus() {
 #endif
 }
 
+
 const QStringList KstApp::recentFiles() const {
   return recent->items();
 }
 
+
 void KstApp::showQuickStartDialog() {
 #ifdef KST_QUICKSTART_DLG
   if (KstSettings::globalSettings()->showQuickStart) {
@@ -1903,5 +1932,13 @@ void KstApp::showQuickStartDialog() {
 #endif
 }
 
+
+void KstApp::fixKMdi() {
+  KTabWidget *tw = tabWidget();
+  if (tw) {
+    tw->setHoverCloseButton(false);
+  }
+}
+
 #include "kst.moc"
 // vim: ts=2 sw=2 et
