@@ -18,8 +18,9 @@
 
 #include "kst2dplot.h"
 #include "kstbasecurve.h"
+#include "kstdatacollection.h"
 
-KstBaseCurve::KstBaseCurve(QDomElement& e) : KstDataObject(e) {
+KstBaseCurve::KstBaseCurve(const QDomElement& e) : KstDataObject(e) {
   commonConstructor();
 }
 
@@ -41,6 +42,7 @@ void KstBaseCurve::commonConstructor() {
   MinPosX = MinPosY = 0;
   Color = in_color;
   NS = 0;
+  _ignoreAutoScale = false;
 }
 
 
@@ -115,7 +117,7 @@ int KstBaseCurve::getIndexNearXY(double x, double dx_per_pix, double y) {
       i0--;
       point(i0, xi, yi);
     }
-    
+
     point(iN, xi, yi);
     while (iN < sc-1 && x+dx_per_pix > xi) {
       iN++;
@@ -154,28 +156,77 @@ int KstBaseCurve::getIndexNearXY(double x, double dx_per_pix, double y) {
 double KstBaseCurve::maxX() const {
   if (hasBars() && sampleCount() > 0) {
     return MaxX + (MaxX - MinX)/(2*(sampleCount()-1));
-  } else {
-    return MaxX;
   }
+  return MaxX;
 }
 
 
 double KstBaseCurve::minX() const {
   if (hasBars() && sampleCount() > 0) {
     return MinX - (MaxX - MinX)/(2*(sampleCount()-1));
-  } else {
-    return MinX;
   }
+  return MinX;
 }
 
 
 bool KstBaseCurve::deleteDependents() {
   bool rc = KstDataObject::deleteDependents();
-  Kst2DPlotList pl = Kst2DPlot::globalPlotList();
-  for (Kst2DPlotList::Iterator i = pl.begin(); i != pl.end(); ++i) {
-    (*i)->Curves.remove(this);
-  }
+  KST::removeCurveFromPlots(this);
   return rc;
 }
 
+
+void KstBaseCurve::setHasPoints(bool in_HasPoints) {
+  HasPoints = in_HasPoints;
+  setDirty();
+}
+
+
+void KstBaseCurve::setHasLines(bool in_HasLines) {
+  HasLines = in_HasLines;
+  setDirty();
+}
+
+
+void KstBaseCurve::setHasBars(bool in_HasBars) {
+  HasBars = in_HasBars;
+  setDirty();
+}
+
+
+void KstBaseCurve::setLineWidth(int in_LineWidth) {
+  LineWidth = in_LineWidth;
+  setDirty();
+}
+
+
+void KstBaseCurve::setLineStyle(int in_LineStyle) {
+  LineStyle = in_LineStyle;
+  setDirty();
+}
+
+
+void KstBaseCurve::setBarStyle(int in_BarStyle) {
+  BarStyle = in_BarStyle;
+  setDirty();
+}
+
+
+void KstBaseCurve::setPointDensity(int in_PointDensity) {
+  PointDensity = in_PointDensity;
+  setDirty();
+}
+
+
+void KstBaseCurve::setColor(const QColor& new_c) {
+  setDirty();
+  Color = new_c;
+}
+
+
+void KstBaseCurve::setIgnoreAutoScale(bool ignoreAutoScale) {
+  setDirty();
+  _ignoreAutoScale = ignoreAutoScale;
+}
+
 // vim: ts=2 sw=2 et
