@@ -62,12 +62,12 @@ class KstBorderedViewObject : public KstViewObject {
   protected:
     KstBorderedViewObject(const QString& type);
   public:
-    KstBorderedViewObject(QDomElement& e);
+    KstBorderedViewObject(const QDomElement& e);
+    KstBorderedViewObject(const KstBorderedViewObject& borderedViewObject);
     virtual ~KstBorderedViewObject();
 
-    virtual UpdateType update();
-    virtual void save(QTextStream& ts);
-    virtual void saveTag(QTextStream& ts);
+    virtual void save(QTextStream& ts, const QString& indent = QString::null);
+    virtual void saveTag(QTextStream& ts, const QString& indent = QString::null);
 
     void setBorderColor(const QColor& c);
     const QColor& borderColor() const;
