@@ -16,12 +16,11 @@
  *                                                                         *
  ***************************************************************************/
 #include <qcheckbox.h>
-#include <qlabel.h>
 #include <qlistbox.h>
 #include <qpushbutton.h>
 #include <qspinbox.h>
-#include <qstring.h>
-#include <qwidget.h>
+
+#include <knuminput.h>
 
 #include "datarangewidget.h"
 #include "kstchangenptsdialog_i.h"
@@ -50,22 +49,34 @@ void KstChangeNptsDialogI::selectAll() {
   CurveList->selectAll(true);
 }
 
-void KstChangeNptsDialogI::updateChangeNptsDialog() {
-  CurveList->clear();
+bool KstChangeNptsDialogI::updateChangeNptsDialog() {
+  QStringList qsl;
 
+  for (uint i_vector = 0; i_vector < CurveList->count(); i_vector++) {
+    if (CurveList->isSelected(i_vector)) {
+      qsl.append(CurveList->text(i_vector));
+    }
+  }
+  CurveList->clear();
   KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
   /* insert vectors into ChangeNptsCurveList */
   for (uint i = 0; i < rvl.count(); i++) {
     rvl[i]->readLock();
     CurveList->insertItem(rvl[i]->tagName(), -1);
+    if (qsl.contains(rvl[i]->tagName())) {
+      CurveList->setSelected(i, true);
+    }
     rvl[i]->readUnlock();
   }
+  return !qsl.isEmpty();
 }
 
 void KstChangeNptsDialogI::showChangeNptsDialog() {
-  updateChangeNptsDialog();
+  bool some_slected = updateChangeNptsDialog();
   updateDefaults(0);
-  CurveList->selectAll(true);
+  if (!some_slected) {
+    CurveList->selectAll(true);
+  }
   show();
   raise();
 }
@@ -75,10 +86,22 @@ void KstChangeNptsDialogI::applyNptsChange() {
   for (uint i_vector = 0; i_vector < CurveList->count(); i_vector++) {
     if (CurveList->isSelected(i_vector)) {
       KstRVectorPtr vector = rvl[i_vector];
+      vector->readLock();
+      int f0, n;
+      if (_kstDataRange->isTime() && vector->dataSource()) {
+        vector->dataSource()->readLock();
+        f0 = vector->dataSource()->sampleForTime(_kstDataRange->f0Value());
+        n = vector->dataSource()->sampleForTime(_kstDataRange->nValue());
+        vector->dataSource()->readUnlock();
+      } else {
+        f0 = _kstDataRange->f0Value();
+        n = _kstDataRange->nValue();
+      }
+      vector->readUnlock();
       vector->writeLock();
       vector->changeFrames(
-        (_kstDataRange->CountFromEnd->isChecked() ? -1 : _kstDataRange->F0->value()),
-        (_kstDataRange->ReadToEnd->isChecked() ? -1 : _kstDataRange->N->value()),
+        (_kstDataRange->CountFromEnd->isChecked() ? -1 : f0),
+        (_kstDataRange->ReadToEnd->isChecked() ? -1 : n),
         _kstDataRange->Skip->value(),
         _kstDataRange->DoSkip->isChecked(),
         _kstDataRange->DoFilter->isChecked());
@@ -104,7 +127,7 @@ void KstChangeNptsDialogI::updateDefaults(int index) {
   } else {
     _kstDataRange->CountFromEnd->setChecked(false);
   }
-  _kstDataRange->F0->setValue(vector->reqStartFrame());
+  _kstDataRange->setF0Value(vector->reqStartFrame());
 
   /* fill number of frames entries */
   if (vector->readToEOF()) {
@@ -112,7 +135,7 @@ void KstChangeNptsDialogI::updateDefaults(int index) {
   } else {
     _kstDataRange->ReadToEnd->setChecked(false);
   }
-  _kstDataRange->N->setValue(vector->reqNumFrames());
+  _kstDataRange->setNValue(vector->reqNumFrames());
 
   /* fill in frames to skip box */
   _kstDataRange->Skip->setValue(vector->skip());
