@@ -39,7 +39,7 @@ public slots:
   static KstCurveDialogI *globalInstance();
 
 private:
-  static KstCurveDialogI* _inst;
+  static QGuardedPtr<KstCurveDialogI> _inst;
   KstVCurvePtr _getPtr(const QString &tagin);
 
   bool _newDialog;
@@ -54,7 +54,10 @@ public slots:
   void Init();
   void close();
   void reject();
+  void toggledXErrorSame(bool on);
+  void toggledYErrorSame(bool on);
 private:
+  static const QString& defaultTag;
   void _fillFieldsForEdit();
   void _fillFieldsForNew();
 
