@@ -22,6 +22,7 @@ class KstDoc;
 
 #include "datamanager.h"
 #include "kstrvector.h"
+#include "kstsvector.h"
 
 class KstDataManagerI: public KstDataManager {
   Q_OBJECT
@@ -49,6 +50,7 @@ class KstDataManagerI: public KstDataManager {
 
   signals:
     void editDataVector(const QString &);
+    void editStaticVector(const QString &);
     void docChanged();
 };
 
@@ -56,13 +58,14 @@ class KstObjectItem : public QObject, public QListViewItem {
   Q_OBJECT
   public:
     KstObjectItem(QListView *parent, KstRVectorPtr x, KstDataManagerI *dm, int localUseCount = 0);
+    KstObjectItem(QListView *parent, KstSVectorPtr x, KstDataManagerI *dm, int localUseCount = 0);
     KstObjectItem(QListViewItem *parent, KstVectorPtr x, KstDataManagerI *dm, int localUseCount = 0);
     KstObjectItem(QListView *parent, KstDataObjectPtr x, KstDataManagerI *dm, int localUseCount = 0);
     virtual ~KstObjectItem();
 
     void update(bool recursive = true, int localUseCount = 0);
     virtual int rtti() const { return _rtti; }
-    
+
     const QString& tagName() const { return _name; }
     KstDataObjectPtr dataObject() { return *KST::dataObjectList.findTag(_name); }
     bool removable() const { return _removable; }
@@ -76,6 +79,8 @@ class KstObjectItem : public QObject, public QListViewItem {
     void removeFromPlot(int);
     void removeImageFromPlot(int);
     void makeCurve();
+    void makeHistogram();
+    void makePSD();
     void makeMatrix();
     void makeImage();
     void reload();
