@@ -23,19 +23,19 @@
 #include <qdeepcopy.h>
 #include <qtimer.h>
 
-#include <kdebug.h>
+#include "ksdebug.h"
 #include <klocale.h>
 
 //#define LOCKTRACE
 
 KstDataObject::KstDataObject() : KstObject() {
-  //kdDebug() << "+++ CREATING DATA OBJECT: " << (void*)this << endl;
+  //kstdDebug() << "+++ CREATING DATA OBJECT: " << (void*)this << endl;
   _curveHints = new KstCurveHintList;
 }
 
-KstDataObject::KstDataObject(QDomElement& e) : KstObject() {
+KstDataObject::KstDataObject(const QDomElement& e) : KstObject() {
   Q_UNUSED(e)
-  //kdDebug() << "+++ CREATING DATA OBJECT: " << (void*)this << endl;
+  //kstdDebug() << "+++ CREATING DATA OBJECT: " << (void*)this << endl;
   _curveHints = new KstCurveHintList;
 }
 
@@ -68,8 +68,8 @@ KstDataObject::~KstDataObject() {
   delete _curveHints;
 }
 
-double *KstDataObject::vectorRealloced(KstVectorPtr v, double *memptr, int newSize, bool bNew) const {
-  if (v.data() == 0L) {
+double *KstDataObject::vectorRealloced(KstVectorPtr v, double *memptr, int newSize) const {
+  if (!v) {
     return 0L;
   }
 
@@ -77,9 +77,7 @@ double *KstDataObject::vectorRealloced(KstVectorPtr v, double *memptr, int newSi
   // in complicated object hierarchies such as filtered vectors.  Therefore if
   // you call vectorRealloced() and v is not locked by you already, you'd
   // better lock it!
-  double *rc = v->realloced(memptr, newSize, bNew);
-
-  return rc;
+  return v->realloced(memptr, newSize);
 }
 
 void KstDataObject::save(QTextStream& ts, const QString& indent) {
@@ -90,6 +88,7 @@ void KstDataObject::save(QTextStream& ts, const QString& indent) {
 bool KstDataObject::loadInputs() {
   bool rc = true;
   QValueList<QPair<QString,QString> >::Iterator i;
+  
   KST::vectorList.lock().readLock();
   for (i = _inputVectorLoadQueue.begin(); i != _inputVectorLoadQueue.end(); ++i) {
     KstVectorList::Iterator it = KST::vectorList.findTag((*i).second);
@@ -129,7 +128,9 @@ bool KstDataObject::loadInputs() {
   _inputVectorLoadQueue.clear();
   _inputScalarLoadQueue.clear();
   _inputStringLoadQueue.clear();
-  update();
+  
+  setDirty();
+
   return rc;
 }
 
@@ -148,7 +149,7 @@ int KstDataObject::getUsage() const {
     }
   }
 
-  for (KstScalarMap::ConstIterator i = _outputScalars.begin(); i != _outputScalars.end(); ++i) {
+  for (KstStringMap::ConstIterator i = _outputStrings.begin(); i != _outputStrings.end(); ++i) {
     if (i.data().data()) {
       rc += i.data()->getUsage() - 1;
     }
@@ -166,7 +167,7 @@ void KstDataObject::showDialog() {
 void KstDataObject::readLock() const {
   KstObject::readLock();
   #ifdef LOCKTRACE
-  kdDebug() << "Read lock by tid=" << (int)QThread::currentThread() << endl;
+  kstdDebug() << "Read lock by tid=" << (int)QThread::currentThread() << endl;
   #endif
   for (KstStringMap::ConstIterator i = _inputStrings.begin(); i != _inputStrings.end(); ++i) {
     (*i)->readLock();
@@ -191,7 +192,7 @@ void KstDataObject::readLock() const {
 
 void KstDataObject::readUnlock() const {
   #ifdef LOCKTRACE
-  kdDebug() << "Read unlock by tid=" << (int)QThread::currentThread() << endl;
+  kstdDebug() << "Read unlock by tid=" << (int)QThread::currentThread() << endl;
   #endif
   for (KstVectorMap::ConstIterator i = _outputVectors.begin(); i != _outputVectors.end(); ++i) {
     (*i)->readUnlock();
@@ -220,7 +221,7 @@ void KstDataObject::readUnlock() const {
 // Might want to guard these with another lock yet.
 void KstDataObject::writeLock() const {
   #ifdef LOCKTRACE
-  kdDebug() << (void*) this << " Write lock by tid=" << (int)QThread::currentThread() << endl;
+  kstdDebug() << (void*) this << " Write lock by tid=" << (int)QThread::currentThread() << endl;
   #endif
   KstRWLock::writeLock();
   for (KstStringMap::ConstIterator i = _inputStrings.begin(); i != _inputStrings.end(); ++i) {
@@ -246,7 +247,7 @@ void KstDataObject::writeLock() const {
 
 void KstDataObject::writeUnlock() const {
   #ifdef LOCKTRACE
-  kdDebug() << (void*)this << "Write unlock by tid=" << (int)QThread::currentThread() << endl;
+  kstdDebug() << (void*)this << "Write unlock by tid=" << (int)QThread::currentThread() << endl;
   #endif
   for (KstVectorMap::ConstIterator i = _outputVectors.begin(); i != _outputVectors.end(); ++i) {
     (*i)->writeUnlock();
@@ -287,10 +288,15 @@ bool KstDataObject::deleteDependents() {
   for (KstDataObjectList::Iterator i = dol.begin(); i != dol.end(); ++i) {
     bool user = (*i)->uses(this);
     if (!user) {
-      // FIXME: should we care about scalars and strings? Needs policy decision
       for (KstVectorMap::Iterator j = _outputVectors.begin(); !user && j != _outputVectors.end(); ++j) {
         user = (*i)->uses(j.data().data());
       }
+      for (KstScalarMap::Iterator j = _outputScalars.begin(); !user && j != _outputScalars.end(); ++j) {
+        user = (*i)->uses(j.data().data());
+      }
+      for (KstStringMap::Iterator j = _outputStrings.begin(); !user && j != _outputStrings.end(); ++j) {
+        user = (*i)->uses(j.data().data());
+      }
     }
     if (user) {
       KstDataObjectPtr dop = *i;
