@@ -24,15 +24,20 @@
 #include <assert.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <math.h>
 
 // include files for Qt
 #include <qdeepcopy.h>
+#include <qeventloop.h>
 #include <qstylesheet.h>
 
 // include files for KDE
+#include <dcopclient.h>
 #include <kdebug.h>
+#include <kdeversion.h>
 #include <kfiledialog.h>
 #include <kmessagebox.h>
+#include <kmdimainfrm.h>
 #include <kprogress.h>
 #include <ksavefile.h>
 
@@ -46,7 +51,10 @@
 #include "kstrvector.h"
 #include "kstvcurve.h"
 #include "kstvectordefaults.h"
+#include "kstsvector.h"
+#include "kstavector.h"
 #include "kstviewwindow.h"
+#include "logevents.h"
 #include "threadevents.h"
 
 static bool backupFile(const QString& qFilename, const QString& backupDir = QString::null, const QString& backupExtension = "~");
@@ -56,7 +64,7 @@ KstDoc::KstDoc(QWidget *parent, const char *name)
   _lock = 0;
   _updating = false;
   _stopping = false;
-  _updateDelay = 5;
+  _nextEventPaint = false;
 }
 
 KstDoc::~KstDoc() {
@@ -87,39 +95,41 @@ bool KstDoc::saveModified(bool doDelete) {
   bool completed = true;
 
   if (_modified) {
-    KstApp *win = static_cast<KstApp*>(parent());
-    int want_save = KMessageBox::warningYesNoCancel( win, i18n("The current plot definition has been modified. Do you want to save it?"), i18n("Question"));
-    switch (want_save) {
-      case KMessageBox::Yes:
-        if (_title == i18n("Untitled")) {
-          if (!win->slotFileSaveAs()) {
-            return false;
+    KstApp *win = KstApp::inst();
+    if (win->activeWindow()) {
+      int want_save = KMessageBox::warningYesNoCancel( win, i18n("The current plot definition has been modified. Do you want to save it?"), i18n("Question"));
+      switch (want_save) {
+        case KMessageBox::Yes:
+          if (_title == i18n("Untitled")) {
+            if (!win->slotFileSaveAs()) {
+              return false;
+            }
+          } else {
+            saveDocument(absFilePath());
           }
-        } else {
-          saveDocument(absFilePath());
-        }
 
-        if (doDelete) {
-          deleteContents();
-        }
-        completed = true;
-        break;
+          if (doDelete) {
+            deleteContents();
+          }
+          completed = true;
+          break;
 
-      case KMessageBox::No:
-        if (doDelete) {
-          setModified(false);
-          deleteContents();
-        }
-        completed = true;
-        break;
+        case KMessageBox::No:
+          if (doDelete) {
+            setModified(false);
+            deleteContents();
+          }
+          completed = true;
+          break;
 
-      case KMessageBox::Cancel:
-        completed = false;
-        break;
+        case KMessageBox::Cancel:
+          completed = false;
+          break;
 
-      default:
-        completed = false;
-        break;
+        default:
+          completed = false;
+          break;
+      }
     }
   }
 
@@ -146,18 +156,24 @@ bool KstDoc::newDocument() {
 
 bool KstDoc::openDocument(const KURL& url, const QString& o_file,
                           int o_n, int o_f, int o_s, bool o_ave) {
-  KstRVectorPtr vector;
-  KstApp *app = KstApp::inst();
-  QString readingDocument = i18n("Reading document");
-  QString openingDocument = i18n("Opening document");
+  static bool opening = false;
+
+  if (opening) {
+    return false;
+  }
+
+  opening = true;
 
   deleteContents();
-  _updating = true; // block update thread from sending events 'till we're done
+  KstApp::inst()->setPaused(true);
+  _updating = true; // block update thread from sending events until we're done
 
   QFile f(url.path());
   if (!f.exists()) {
     KMessageBox::sorry(0L, i18n("%1: There is no file with that name to open.").arg(url.path()));
+    opening = false;
     _updating = false;
+    KstApp::inst()->setPaused(false);
     return false;
   }
 
@@ -170,14 +186,18 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
 
   if (!f.open(IO_ReadOnly)) {
     KMessageBox::sorry(0L, i18n("%1: File exists, but kst could not open it.").arg(url.path()));
+    opening = false;
     _updating = false;
+    KstApp::inst()->setPaused(false);
     return false;
   }
 
   if (!doc.setContent(&f)) {
     KMessageBox::sorry(0L, i18n("%1: Not a valid kst plot specification file.").arg(url.path()));
     f.close();
+    opening = false;
     _updating = false;
+    KstApp::inst()->setPaused(false);
     return false;
   }
 
@@ -188,29 +208,52 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
   int count;
   int handled = 0;
   bool warnOldKstFile = false;
+  QString activeWindow;
+  KstRVectorPtr vector;
+  KstApp *app = KstApp::inst();
+  QString readingDocument = i18n("Reading Kst file");
+
+  if (docElem.tagName() != "kstdoc") {
+    f.close();
+    opening = false;
+    _updating = false;
+    KstApp::inst()->setPaused(false);
+    return false;
+  }
+
+  if (!docElem.attribute("version").isEmpty()) {
+    f.close();
+    opening = false;
+    _updating = false;
+    KstApp::inst()->setPaused(false);
+    return false;
+  }
 
   QDomNode n = docElem.firstChild();
   count = docElem.childNodes().length();
   app->slotUpdateProgress(count, handled, readingDocument);
-  
+
+  kapp->dcopClient()->setAcceptCalls(false);
   while (!n.isNull()) {
     QDomElement e = n.toElement(); // try to convert the node to an element.
     if (!e.isNull()) { // the node was really an element.
       if (e.tagName() == "windowsize") {
         QDomNode nn = e.firstChild();
-        QSize size = static_cast<KstApp*>(parent())->size();
+        QSize size = app->size();
         while (!nn.isNull()) {
           QDomElement ee = nn.toElement(); // convert child to element
           if (ee.tagName() == "width") {
             size.setWidth(ee.text().toInt());
           } else if (ee.tagName() == "height") {
             size.setHeight(ee.text().toInt());
+          } else if (ee.tagName() == "active") {
+            activeWindow = ee.attribute("name");
           }
           nn = nn.nextSibling();
         }
-        static_cast<KstApp*>(parent())->resize(size);
+        app->resize(size);
       } else if (e.tagName() == "graphicsautosave") {
-        KstGraphFileDialogI *gdlg = KstApp::inst()->graphFileDlg();
+        KstGraphFileDialogI *gdlg = app->graphFileDlg();
         int time = e.attribute("time").toInt();
         bool en = e.attribute("enabled") == "true";
         QString fmt = e.attribute("format");
@@ -218,10 +261,10 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
         bool sq = e.attribute("square") == "true";
         int sizeX = e.attribute("xsize").toInt();
         int sizeY = e.attribute("ysize").toInt();
+        int display = e.attribute("display").toInt();
 
         if (sizeX > 0 && sizeY > 0) {
           gdlg->setImageSize(sizeX, sizeY);
-          gdlg->setSquare(sq);
         }
         if (!url.isEmpty()) {
           gdlg->setURL(url);
@@ -232,12 +275,16 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
         if (time >= 0) {
           gdlg->setAutoSaveTimer(time);
         }
+        if (sq) {
+          gdlg->setDisplay(1);
+        } else {
+          gdlg->setDisplay(display);
+        }
         gdlg->setAutoSave(en);
+        gdlg->updateDialog();
+        gdlg->applyAutosave();
       } else if (e.tagName() == "plotcols") {
         columns = e.text().toInt();
-      } else if (e.tagName() == "updateDelay") {
-// FIXME XXXX
-        setDelay(e.text().toInt());
       } else if (e.tagName() == "kstfile") {
         KstWriteLocker dswl(&KST::dataSourceList.lock());
         KstDataSourcePtr file;
@@ -260,6 +307,13 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
         vector = new KstRVector(e, o_file, o_n, o_f, o_s, o_ave);
         KST::addVectorToList(KstVectorPtr(vector));
         // Vectors are automatically added to the global list.
+      } else if (e.tagName() == "svector") {
+        KstSVectorPtr svector = new KstSVector(e);
+        KST::addVectorToList(KstVectorPtr(svector));
+      } else if (e.tagName() == "avector") {
+        KstAVectorPtr avector = new KstAVector(e);
+        avector = new KstAVector(e);
+        KST::addVectorToList(KstVectorPtr(avector));
       } else if (e.tagName() == "plugin") {
         KstDataObjectPtr p = new KstPlugin(e);
         KstWriteLocker dowl(&KST::dataObjectList.lock());
@@ -297,7 +351,7 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
       } else if (e.tagName() == "plot") {
         KstBaseCurveList l = kstObjectSubList<KstDataObject,KstBaseCurve>(KST::dataObjectList);
         Kst2DPlotPtr plot = new Kst2DPlot(e);
-        KstApp::inst()->plotHolderWhileOpeningDocument().insert(plot->tagName(), plot);
+        app->plotHolderWhileOpeningDocument().insert(plot->tagName(), plot);
       } else if (e.tagName() == "matrix") {
         KstDataObjectPtr p = new KstMatrix(e);
         KstWriteLocker dowl(&KST::dataObjectList.lock());
@@ -307,13 +361,13 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
         KstWriteLocker dowl(&KST::dataObjectList.lock());
         KST::dataObjectList.append(p);
       } else if (e.tagName() == "window") {
-        KstViewWindow *p = dynamic_cast<KstViewWindow*>(KstApp::inst()->activeWindow());
+        KstViewWindow *p = dynamic_cast<KstViewWindow*>(app->activeWindow());
         KstViewWindow *toDelete = 0L;
 
         if (!p) {
-          KMdiIterator<KMdiChildView*> *it = KstApp::inst()->createIterator();
+          KMdiIterator<KMdiChildView*> *it = app->createIterator();
           p = dynamic_cast<KstViewWindow*>(it->currentItem());
-          KstApp::inst()->deleteIterator(it);
+          app->deleteIterator(it);
         }
 
         if (p && p->view()->children().count() == 0) {
@@ -321,10 +375,10 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
         }
 
         p = new KstViewWindow(e);
-        KstApp::inst()->addWindow(p);
+        app->addWindow(p, KMdi::StandardAdd | KMdi::UseKMdiSizeHint);
         p->activate();
         if (toDelete) {
-          KstApp::inst()->closeWindow(toDelete);
+          app->closeWindow(toDelete);
         }
       } else {
         KstDebug::self()->log(i18n("Unsupported element '%1' in file %2.").arg(e.tagName()).arg(o_file), KstDebug::Warning);
@@ -333,18 +387,17 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
     handled++;
     app->slotUpdateProgress(count, handled, readingDocument);
     n = n.nextSibling();
+    kapp->eventLoop()->processEvents(QEventLoop::ExcludeSocketNotifiers, 10);
   }
-  app->slotUpdateProgress(0, 0, openingDocument);
 
   if (warnOldKstFile) {
     KMessageBox::sorry(0L, i18n("You tried to load an old Kst file.  Curves created by equations or PSDs will not be loaded."));
   }
 
-  //
+  app->slotUpdateProgress(0, 0, i18n("Creating plots"));
   // if we have anything left in plotHolderWhileOpeningDocument then
   //  we are most likely reading an old style document, so we create
   //  a default view and fill it with whatever is left...
-  //
   if (!app->plotHolderWhileOpeningDocument().isEmpty() && !app->activeWindow()) {
 
     QString winName = app->newWindow(QString::null);
@@ -376,11 +429,11 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
     }
   }
 
-  KstApp::inst()->plotHolderWhileOpeningDocument().clear();
+  app->plotHolderWhileOpeningDocument().clear();
+
+  app->slotUpdateProgress(0, 0, i18n("Loading data"));
 
-  //
   // lazy load data
-  //
   KstDataObjectList bitBucket;
   {
     KST::dataObjectList.lock().readLock();
@@ -393,11 +446,10 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
       bool rc = (*i)->loadInputs();
       (*i)->KstRWLock::writeUnlock();
       if (!rc) {
-        //
         // schedule for removal
-        //
         bitBucket.append(*i);
       }
+      kapp->eventLoop()->processEvents(QEventLoop::ExcludeSocketNotifiers, 10);
     }
   }
 
@@ -408,35 +460,43 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
   KST::dataObjectList.lock().writeUnlock();
 
   if (!bitBucket.isEmpty()) {
-    KMessageBox::sorry(0L, i18n("The Kst file could not be loaded in its entirety due to missing objects or data."));
+    QStringList names;
+    for (KstDataObjectList::Iterator i = bitBucket.begin(); i != bitBucket.end(); ++i) {
+      names += (*i)->tagName();
+    }
+
+#if KDE_VERSION < KDE_MAKE_VERSION(3,3,90)
+    KMessageBox::informationList(0L, i18n("The Kst file could not be loaded in its entirety due to missing objects or data."), names);
+#else
+    KMessageBox::errorList(0L, i18n("The Kst file could not be loaded in its entirety due to missing objects or data."), names);
+#endif
   }
 
   createScalars();
 
-  //
-  // update plots now that lazy loading is done
-  //
-  KMdiIterator<KMdiChildView*> *it = KstApp::inst()->createIterator();
-  while (it->currentItem()) {
-    KstViewWindow *c = dynamic_cast<KstViewWindow*>(it->currentItem());
-    if (c) {
-      Kst2DPlotList allplots = c->view()->findChildrenType<Kst2DPlot>(true);
-      for (Kst2DPlotList::Iterator i = allplots.begin(); i != allplots.end(); ++i) {
-        (*i)->draw();
-      }
-    }
-    it->next();
-  }
-  KstApp::inst()->deleteIterator(it);
+  app->slotUpdateProgress(0, 0, i18n("Synchronizing data"));
+  kapp->dcopClient()->setAcceptCalls(true);
+
+  _nextEventPaint = true;
 
   emit updateDialogs();
 
+  if (!activeWindow.isEmpty()) {
+    KMdiChildView *c = app->findWindow(activeWindow);
+    if (c) {
+      c->activate();
+    }
+  }
+
   app->slotUpdateProgress(0, 0, QString::null);
 
   _modified = false;
-  
+
   QTimer::singleShot(0, this, SLOT(enableUpdates()));
-  
+
+  opening = false;
+
+  KstApp::inst()->setPaused(false);
   return true;
 }
 
@@ -446,32 +506,36 @@ void KstDoc::saveDocument(QTextStream& ts, bool saveAbsoluteVectorPositions) {
   ts << "<kstdoc>" << endl;
   // save window geometry for this kst file
   ts << "  <windowsize>" << endl;
-  ts << "    <width>" << ((KstApp *)parent())->width() << "</width>" << endl;
-  ts << "    <height>"<< ((KstApp *)parent())->height()<< "</height>" << endl;
+  ts << "    <width>" << app->width() << "</width>" << endl;
+  ts << "    <height>"<< app->height()<< "</height>" << endl;
+  KMdiChildView *c = app->activeWindow();
+  if (c) {
+    ts << "    <active name=\"" << QStyleSheet::escape(c->caption()) << "\"/>" << endl;
+  }
   ts << "  </windowsize>" << endl;
 
   // save graphics autosave settings
   ts << "  <graphicsautosave time=\""
-    << KstApp::inst()->graphFileDlg()->autoSaveTimer()
+    << app->graphFileDlg()->autoSaveTimer()
     << "\" enabled=\""
-    << (KstApp::inst()->graphFileDlg()->autoSaving() ? "true" : "false")
+    << (app->graphFileDlg()->autoSaving() ? "true" : "false")
     << "\" format=\""
-    << QStyleSheet::escape(KstApp::inst()->graphFileDlg()->format())
+    << QStyleSheet::escape(app->graphFileDlg()->format())
     << "\" xsize=\""
-    << KstApp::inst()->graphFileDlg()->imageXSize()
+    << app->graphFileDlg()->imageXSize()
     << "\" ysize=\""
-    << KstApp::inst()->graphFileDlg()->imageYSize()
+    << app->graphFileDlg()->imageYSize()
+    << "\" display=\""
+    << app->graphFileDlg()->display()
+    // #### Kst2 Remove me - compatibility
     << "\" square=\""
-    << (KstApp::inst()->graphFileDlg()->isSquare() ? "true" : "false");
-  if (KstApp::inst()->graphFileDlg()->autoSaving()) {
-    ts << "\" url=\"" << QStyleSheet::escape(KstApp::inst()->graphFileDlg()->url().url());
+    << (app->graphFileDlg()->display() == 1 ? "true" : "false");
+
+  if (app->graphFileDlg()->autoSaving()) {
+    ts << "\" url=\"" << QStyleSheet::escape(app->graphFileDlg()->url().url());
   }
   ts << "\" />" << endl;
 
-  // save update delay
-//FIXME XXXX
-  ts << " <updateDelay>" << delay() << "</updateDelay>" << endl;
-
   KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
 
   // save files
@@ -508,10 +572,17 @@ void KstDoc::saveDocument(QTextStream& ts, bool saveAbsoluteVectorPositions) {
   }
 
   // save vectors
-  for (KstRVectorList::Iterator it = rvl.begin(); it != rvl.end(); ++it) {
-    ts << "  <vector>" << endl;
-    (*it)->save(ts, "    ", saveAbsoluteVectorPositions);
-    ts << "  </vector>" << endl;
+//   for (KstRVectorList::Iterator it = rvl.begin(); it != rvl.end(); ++it) {
+//     ts << "  <vector>" << endl;
+//     (*it)->save(ts, "    ", saveAbsoluteVectorPositions);
+//     ts << "  </vector>" << endl;
+//   }
+  for (KstVectorList::Iterator it = KST::vectorList.begin(); it != KST::vectorList.end(); ++it) {
+    if ((*it)->saveable()) {
+      //      ts << "  <vector>" << endl;
+      (*it)->save(ts, "  ", saveAbsoluteVectorPositions);
+      //ts << "  </vector>" << endl;
+    }
   }
 
   // save data objects
@@ -532,7 +603,6 @@ void KstDoc::saveDocument(QTextStream& ts, bool saveAbsoluteVectorPositions) {
           ts << "  <plot>" << endl;
           plot->save(ts, "    ");
           ts << "  </plot>" << endl;
-        } else {
         }
       }
     }
@@ -576,6 +646,7 @@ bool KstDoc::saveDocument(const QString &filename, bool saveAbsoluteVectorPositi
   }
 
   QTextStream ts(&f);
+  ts.setEncoding(QTextStream::UnicodeUTF8);
 
   saveDocument(ts, saveAbsoluteVectorPositions);
 
@@ -585,8 +656,8 @@ bool KstDoc::saveDocument(const QString &filename, bool saveAbsoluteVectorPositi
   return true;
 }
 
-void KstDoc::deleteContents() {
 
+void KstDoc::deleteContents() {
   KST::vectorDefaults.sync();
 
   KstApp *app = KstApp::inst();
@@ -634,6 +705,7 @@ void KstDoc::samplesUp() {
   KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
   for (int i = 0; i < (int)rvl.count(); i++) {
     V = rvl[i];
+    V->writeLock();
     f0 = V->reqStartFrame();
     n = V->reqNumFrames();
     skip = V->skip();
@@ -641,12 +713,13 @@ void KstDoc::samplesUp() {
     doAve =  V->doAve();
     fileN = V->fileLength();
 
-    if (f0 + n > fileN) {
+    if (f0 + 2 * n > fileN) {
       f0 = fileN - n;
     } else {
       f0 += n;
     }
     V->changeFrames(f0, n, skip, doSkip, doAve);
+    V->writeUnlock();
   }
 
   setModified();
@@ -655,6 +728,7 @@ void KstDoc::samplesUp() {
   emit dataChanged();
 }
 
+
 void KstDoc::samplesDown() {
   KstRVectorPtr V;
   int f0, n, skip;
@@ -664,6 +738,7 @@ void KstDoc::samplesDown() {
   KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
   for (int i = 0; i < (int)rvl.count(); i++) {
     V = rvl[i];
+    V->writeLock();
     f0 = V->reqStartFrame();
     if (f0 == -1)
       f0 = V->startFrame();
@@ -680,6 +755,7 @@ void KstDoc::samplesDown() {
     }
 
     V->changeFrames(f0, n, skip, doSkip, doAve);
+    V->writeUnlock();
   }
 
   setModified();
@@ -688,23 +764,16 @@ void KstDoc::samplesDown() {
   emit dataChanged();
 }
 
+
 void KstDoc::samplesEnd() {
   KstRVectorPtr V;
-  int f0, n, skip;
-  bool doSkip, doAve;
-  int fileN;
 
   KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
   for (int i = 0; i < (int)rvl.count(); i++) {
     V = rvl[i];
-    f0 = -1;
-    n = V->numFrames();
-    skip = V->skip();
-    doSkip = V->doSkip();
-    doAve =  V->doAve();
-    fileN = V->fileLength();
-
-    V->changeFrames(f0, n, skip, doSkip, doAve);
+    V->writeLock();
+    V->setFromEnd();
+    V->writeUnlock();
   }
 
   setModified();
@@ -713,12 +782,14 @@ void KstDoc::samplesEnd() {
   emit dataChanged();
 }
 
+
 void KstDoc::wasModified() {
   _modified = true;
   forceUpdate();
-  KstApp::inst()->paintAll();
+  QTimer::singleShot(0, this, SIGNAL(updateDialogs()));
 }
 
+
 RemoveStatus KstDoc::removeDataObject(const QString& tag) {
   KST::dataObjectList.lock().writeLock();
   KST::dataObjectList.removeTag(tag);
@@ -734,16 +805,14 @@ void KstDoc::purge() {
   bool modified = false;
   KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
   KstApp *app = KstApp::inst();
-  
+
   KST::dataObjectList.lock().writeLock();
   int cnt = rvl.count() + KST::dataObjectList.count();
   int prg = 0;
 
   app->slotUpdateProgress(cnt, prg, purging);
 
-  //
   // ASSUMPTION: this only gets called from the data manager!
-  //
   for (KstDataObjectList::Iterator it = KST::dataObjectList.begin(); it != KST::dataObjectList.end(); ++it) {
     //kdDebug() << "OBJECT: " << (*it)->tagName() << " USAGE: " << (*it)->getUsage() << endl;
     if ((*it)->getUsage() == 0) {
@@ -780,7 +849,8 @@ void KstDoc::purge() {
 
 
 void KstDoc::forceUpdate() {
-  static_cast<KstApp*>(parent())->forceUpdate();
+  _nextEventPaint = false;
+  KstApp::inst()->forceUpdate();
 }
 
 
@@ -794,35 +864,96 @@ bool KstDoc::event(QEvent *e) {
     _updating = true; // block update thread from sending events 'till we're done
 
     ThreadEvent *te = static_cast<ThreadEvent*>(e);
-    if (te->_eventType == ThreadEvent::UpdateDataDialogs) {
-      kdDebug() << "Update data dialogs" << endl;
-      emit dataChanged();
-      KstApp::inst()->paintAll();
-    } else if (te->_eventType == ThreadEvent::UpdateAllDialogs) {
-      kdDebug() << "Update ALL dialogs" << endl;
+    switch (te->_eventType) {
+      case ThreadEvent::UpdateDataDialogs:
+        {
+          //kdDebug() << "Update data dialogs" << endl;
+          emit dataChanged();
+          // HACK: remove me later
+          KMdiIterator<KMdiChildView*> *it = KstApp::inst()->createIterator();
+          if (it) {
+            while (it->currentItem()) {
+              KstViewWindow *view = dynamic_cast<KstViewWindow*>(it->currentItem());
+              if (!view) {
+                it->next();
+                continue;
+              }
 
-      if (KstApp::inst()) {
-        KMdiIterator<KMdiChildView*>* it = KstApp::inst()->createIterator();
-        if (it) {
-          while (it->currentItem()) {
-            KstViewWindow *view = dynamic_cast<KstViewWindow*>(it->currentItem());
-            if (view) {
-              view->view()->update();
+              Kst2DPlotList pl = view->view()->findChildrenType<Kst2DPlot>(true);
+              for (Kst2DPlotList::Iterator i = pl.begin(); i != pl.end(); ++i) {
+                for (QValueList<KstBaseCurve*>::ConstIterator j = te->_curves.begin(); j != te->_curves.end(); ++j) {
+                  // race: if ((*i)->Curves.contains(*j)) {
+                  const KstBaseCurveList& cl = (*i)->Curves;
+                  bool doBreak = false;
+                  for (KstBaseCurveList::ConstIterator k = cl.begin(); k != cl.end(); ++k) {
+                    if (*j == *k) {
+                      (*i)->setDirty();
+                      doBreak = true;
+                      break;
+                    }
+                  }
+                  if (doBreak) {
+                    break;
+                  }
+                }
+              }
+              it->next();
             }
-            it->next();
+            KstApp::inst()->deleteIterator(it);
           }
-          KstApp::inst()->deleteIterator(it);
+          KstApp::inst()->paintAll();
         }
-      }
+        break;
+      case ThreadEvent::UpdateAllDialogs:
+        //kdDebug() << "Update ALL dialogs" << endl;
+        {
+          KMdiIterator<KMdiChildView*>* it = KstApp::inst()->createIterator();
+          if (it) {
+            while (it->currentItem()) {
+              KstViewWindow *view = dynamic_cast<KstViewWindow*>(it->currentItem());
+              if (view) {
+                view->view()->update();
+              }
+              it->next();
+            }
+            KstApp::inst()->deleteIterator(it);
+          }
 
-      emit updateDialogs();
-    } else if (te->_eventType == ThreadEvent::Repaint) {
-      KstApp::inst()->paintAll();
-    } else if (te->_eventType == ThreadEvent::Done) {
+          emit updateDialogs();
+        }
+        break;
+      case ThreadEvent::Done:
+        break;
+      case ThreadEvent::Repaint:
+        KstApp::inst()->paintAll();
+        break;
+      case ThreadEvent::NoUpdate:
+        if (_nextEventPaint) {
+          KstApp::inst()->paintAll();
+        }
+        break;
+      default:
+        break;
     }
 
+    _nextEventPaint = false;
     QTimer::singleShot(0, this, SLOT(enableUpdates()));
 
+    return true;
+  } else if (e->type() == KstEventTypeLog) {
+    LogEvent *le = dynamic_cast<LogEvent*>(e);
+    if (le) {
+      switch (le->_eventType) {
+        case LogEvent::LogAdded:
+          emit logAdded(le->_msg);
+          break;
+        case LogEvent::LogCleared:
+          emit logCleared();
+          break;
+        default:
+          break;
+      }
+    }
     return true;
   }
 
@@ -850,7 +981,10 @@ void KstDoc::createScalars() const {
   new KstScalar("CONST_MKSA_ELECTRON_CHARGE", 1.602176462e-19, false);
   new KstScalar("CONST_MKSA_MOLAR_GAS", 8.314472e0, false);
   new KstScalar("CONST_MKSA_STANDARD_GAS_VOLUME", 2.2710981e-2, false);
-  new KstScalar("CONST_PI", 3.141592654e0, false);
+  new KstScalar("CONST_PI", M_PI, false);
+  new KstScalar("C_PI", M_PI, false);
+  new KstScalar("C_R2D", 180.0/M_PI, false); // radians to degrees
+  new KstScalar("C_D2R", M_PI/180.0, false); // degrees to radians
 }
 
 
@@ -897,9 +1031,9 @@ static bool backupFile(const QString& qFilename, const QString& backupDir,
        QCString nameOnly;
        int slash = cFilename.findRev('/');
        if (slash < 0)
-	   nameOnly = cFilename;
+         nameOnly = cFilename;
        else
-	   nameOnly = cFilename.mid(slash + 1);
+         nameOnly = cFilename.mid(slash + 1);
        cBackup = QFile::encodeName(backupDir);
        if ( backupDir[backupDir.length()-1] != '/' ) {
            cBackup += '/';
