@@ -24,6 +24,7 @@
 
 #include <kapplication.h>
 
+#include "kstdebug.h"
 #include "kstlistenums.h"
 
 class KstDoc : public QObject {
@@ -60,9 +61,6 @@ public:
   /** delete a curve from the curvelist and from the plots */
   RemoveStatus removeDataObject(const QString& tag);
 
-  /** the update delay */
-  int delay() const { return _updateDelay; }
-
   virtual bool event(QEvent *e);
 
   bool updating() const { return _updating; }
@@ -89,8 +87,6 @@ public slots:
   /** cancel update */
   void cancelUpdate() { _stopping = true;}
 
-  void setDelay(int in_delay) { _updateDelay = in_delay; }
-
 private slots:
   void enableUpdates();
 
@@ -100,18 +96,19 @@ private:
   /** the modified flag of the current document */
   bool _modified;
   bool _stopping;
-
-  int _updateDelay;
+  bool _updating;
+  bool _nextEventPaint;
 
   QString _title;
   QString _absFilePath;
   int _lock;
-  bool _updating;
 
 signals:
   /** if something has changed the vectors */
   void dataChanged();
   void updateDialogs();
+  void logAdded(const KstDebug::LogMessage& msg);
+  void logCleared();
 };
 
 #endif // KSTDOC_H
