@@ -30,7 +30,7 @@ KstExtension::~KstExtension() {
 }
 
 
-void KstExtension::load(QDomElement& e) {
+void KstExtension::load(const QDomElement& e) {
   Q_UNUSED(e)
 }
 
