@@ -45,26 +45,14 @@ KstFitDialogI *KstFitDialogI::globalInstance() {
   if (!_inst) {
     _inst = new KstFitDialogI(KstApp::inst());
   }
-
   return _inst;
 }
 
 
-//const QString newFitPluginString = "<Auto Name>";
-const QString fit_defaultTag = "<Auto Name>";
-
 KstFitDialogI::KstFitDialogI(QWidget* parent, const char* name,
                              bool modal, WFlags fl)
-: KstFitDialog(parent, name, modal, fl) {
-  Init();
-
-  connect(_OK, SIGNAL(clicked()), this, SLOT(OK()));
-  connect(_cancel, SIGNAL(clicked()), this, SLOT(close()));
-  connect(PluginCombo, SIGNAL(activated(int)), this, SLOT(pluginChanged(int)));
-  connect(_pluginManager, SIGNAL(clicked()), this, SLOT(showPluginManager()));
-
-  QGridLayout *grid = new QGridLayout(_pluginFrame, 1, 1);
-  grid->addWidget(_frameWidget = new QWidget(_pluginFrame, "Frame Widget"), 0, 0);
+: KstPluginDialogI(parent, name, modal, fl) {
+  _curveAppearance->show();
 }
 
 
@@ -72,20 +60,10 @@ KstFitDialogI::~KstFitDialogI() {
 }
 
 
-void KstFitDialogI::Init() {
-}
-
-
-void KstFitDialogI::OK() {
-  if (new_I()) {
-    close();
-  }
-}
-
-
 void KstFitDialogI::show_setCurve(const QString& strCurve,
                                   const QString& strPlotName,
                                   const QString& strWindow) {
+                                    
   KstBaseCurvePtr curve;
   KstBaseCurveList curves = kstObjectSubList<KstDataObject, KstBaseCurve>(KST::dataObjectList);
   KstPluginList c = kstObjectSubList<KstDataObject, KstPlugin>(KST::dataObjectList);
@@ -97,22 +75,14 @@ void KstFitDialogI::show_setCurve(const QString& strCurve,
   curve = *curves.findTag(strCurve);
   if (curve) {
     KstReadLocker rl(curve);
-    _xvector = curve->xVTag();
     _yvector = curve->yVTag();
+    _xvector = curve->xVTag();
     _evector = curve->yETag();
   }
-
   if (_xvector == "<None>" || _yvector == "<None>") {
     return;
   }
-
-  _tagName->setText(fit_defaultTag);
-
-  updatePluginList();
-  pluginChanged(PluginCombo->currentItem());
-
-  show();
-  raise();
+  show_New();  
 }
 
 
@@ -130,9 +100,8 @@ void KstFitDialogI::updatePluginList() {
     if (it.data()._isFit) {
       if (!it.data()._isFitWeighted || _evector != "<None>") {
         _pluginList += it.data()._name;
-        PluginCombo->insertItem(i18n("%1 (v%2) - %3").arg(it.data()._readableName)
-                                .arg(it.data()._version)
-                                .arg(it.data()._description));
+        PluginCombo->insertItem(i18n("%1 (v%2)").arg(it.data()._readableName)
+                                .arg(it.data()._version));
         if (it.data()._name == previous) {
           newFocus = cnt;
         }
@@ -150,216 +119,6 @@ void KstFitDialogI::updatePluginList() {
 }
 
 
-void KstFitDialogI::update() {
-  KstSharedPtr<Plugin> pPtr = PluginCollection::self()->plugin(_pluginList[PluginCombo->currentItem()]);
-  if (pPtr) {
-    const QValueList<Plugin::Data::IOValue>& itable = pPtr->data()._inputs;
-    for (QValueList<Plugin::Data::IOValue>::ConstIterator it = itable.begin(); it != itable.end(); ++it) {
-      if ((*it)._type == Plugin::Data::IOValue::TableType) {
-        QObject *field = _frameWidget->child((*it)._name.latin1(), "VectorSelector");
-        if (field) {
-          VectorSelector *vs = static_cast<VectorSelector*>(field);
-          vs->update();
-        }
-      } else if ((*it)._type == Plugin::Data::IOValue::StringType) {
-        QObject *field = _frameWidget->child((*it)._name.latin1(), "StringSelector");
-        if (field) {
-          StringSelector *ss = static_cast<StringSelector*>(field);
-          ss->update();
-        }
-      } else if ((*it)._type == Plugin::Data::IOValue::PidType) {
-        // Nothing
-      } else {
-        QObject *field = _frameWidget->child((*it)._name.latin1(), "ScalarSelector");
-        if (field) {
-          ScalarSelector *ss = static_cast<ScalarSelector*>(field);
-          ss->update();
-        }
-      }
-    }
-  }
-}
-
-
-void KstFitDialogI::fixupLayout() {
-  _frameWidget->updateGeometry();
-  _pluginFrame->updateGeometry();
-  resize(sizeHint());
-  setMinimumSize(size());
-  updateGeometry();
-}
-
-
-bool KstFitDialogI::saveInputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p) {
-  int iInputVector = 0;
-
-  KST::vectorList.lock().readLock();
-  KST::scalarList.lock().readLock();
-  KST::stringList.lock().readLock();
-  const QValueList<Plugin::Data::IOValue>& itable = p->data()._inputs;
-  for (QValueList<Plugin::Data::IOValue>::ConstIterator it = itable.begin(); it != itable.end(); ++it) {
-    if ((*it)._type == Plugin::Data::IOValue::TableType) {
-      if (iInputVector < 2 || (plugin->plugin()->data()._isFitWeighted && iInputVector < 3) ) {
-        KstVectorPtr v;
-
-        switch (iInputVector) {
-          case 0:
-            v = *KST::vectorList.findTag(_xvector);
-            break;
-          case 1:
-            v = *KST::vectorList.findTag(_yvector);
-            break;
-          case 2:
-            v = *KST::vectorList.findTag(_evector);
-            break;
-        }
-        if (v) {
-          v->writeLock(); // to match with plugin->writeLock()
-        }
-        if (plugin->inputVectors().contains((*it)._name)) {
-          plugin->inputVectors()[(*it)._name]->writeUnlock();
-        }
-        plugin->inputVectors().insert((*it)._name, v);
-        iInputVector++;
-      } else {
-        QObject *field = _frameWidget->child((*it)._name.latin1(), "VectorSelector");
-        if (field) {
-          VectorSelector *vs = static_cast<VectorSelector*>(field);
-          KstVectorPtr v = *KST::vectorList.findTag(vs->selectedVector());
-          if (v) {
-            v->writeLock(); // to match with plugin->writeLock()
-          }
-          if (plugin->inputVectors().contains((*it)._name)) {
-            plugin->inputVectors()[(*it)._name]->writeUnlock();
-          }
-          plugin->inputVectors().insert((*it)._name, v);
-        }
-      }
-    } else if ((*it)._type == Plugin::Data::IOValue::StringType) {
-      QObject *field = _frameWidget->child((*it)._name.latin1(), "StringSelector");
-      if (field) {
-        StringSelector *ss = static_cast<StringSelector*>(field);
-        KstStringPtr s = *KST::stringList.findTag(ss->selectedString());
-        if (s == *KST::stringList.end()) {
-          QString val = ss->_string->currentText();
-
-          KstStringPtr newString = new KstString(ss->_string->currentText(), val, true, false);
-          newString->writeLock(); // to match with plugin->writeLock()
-          if (plugin->inputStrings().contains((*it)._name)) {
-            plugin->inputStrings()[(*it)._name]->writeUnlock();
-          }
-          plugin->inputStrings().insert((*it)._name, newString);
-        } else {
-          s->writeLock(); // to match with plugin->writeLock()
-          if (plugin->inputStrings().contains((*it)._name)) {
-            plugin->inputStrings()[(*it)._name]->writeUnlock();
-          }
-          plugin->inputStrings().insert((*it)._name, s);
-        }
-      }
-    } else if ((*it)._type == Plugin::Data::IOValue::PidType) {
-      // Nothing
-    } else {
-      QObject *field = _frameWidget->child((*it)._name.latin1(), "ScalarSelector");
-      if (field) {
-        ScalarSelector *ss = static_cast<ScalarSelector*>(field);
-        KstScalarPtr s = *KST::scalarList.findTag(ss->selectedScalar());
-        if (s == *KST::scalarList.end()) {
-          bool ok;
-          double val = ss->_scalar->currentText().toDouble(&ok);
-
-          if (ok) {
-            KstScalarPtr newScalar = new KstScalar(ss->_scalar->currentText(), val, true, false, false);
-            newScalar->writeLock(); // to match with plugin->writeLock()
-            if (plugin->inputScalars().contains((*it)._name)) {
-              plugin->inputScalars()[(*it)._name]->writeUnlock();
-            }
-            plugin->inputScalars().insert((*it)._name, newScalar);
-          } else {
-            s->writeLock(); // to match with plugin->writeLock()
-            if (plugin->inputScalars().contains((*it)._name)) {
-              plugin->inputScalars()[(*it)._name]->writeUnlock();
-            }
-            plugin->inputScalars().insert((*it)._name, s);
-          }
-        } else {
-          s->writeLock(); // to match with plugin->writeLock()
-          if (plugin->inputScalars().contains((*it)._name)) {
-            plugin->inputScalars()[(*it)._name]->writeUnlock();
-          }
-          plugin->inputScalars().insert((*it)._name, s);
-        }
-      }
-    }
-  }
-  KST::stringList.lock().readUnlock();
-  KST::scalarList.lock().readUnlock();
-  KST::vectorList.lock().readUnlock();
-
-  return true;
-}
-
-
-bool KstFitDialogI::saveOutputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p) {
-  const QValueList<Plugin::Data::IOValue>& otable = p->data()._outputs;
-
-  for (QValueList<Plugin::Data::IOValue>::ConstIterator it = otable.begin(); it != otable.end(); ++it) {
-    QObject *field = _frameWidget->child((*it)._name.latin1(), "QLineEdit");
-    if (!field) {
-      continue;
-    }
-
-    QLineEdit *li = static_cast<QLineEdit*>(field);
-
-    if (li->text().isEmpty()) {
-      QString tagName = _tagName->text();
-      if (tagName == fit_defaultTag) {
-        tagName = plugin->tagName();
-      }
-      li->setText(tagName+"-"+(*it)._name);
-    }
-
-    QString nt = li->text();
-    if ((*it)._type == Plugin::Data::IOValue::TableType) {
-      if (!KST::vectorTagNameNotUnique(nt, false)) {
-        KstVectorPtr v = plugin->outputVectors()[(*it)._name];
-        v->setTagName(nt);
-        v->setProvider(plugin.data());
-      } else if (plugin->outputVectors()[(*it)._name]->tagName() != nt) {
-        while (KST::vectorTagNameNotUnique(nt, false)) {
-          nt += "'";
-        }
-        KstVectorPtr v = plugin->outputVectors()[(*it)._name];
-        v->setTagName(nt);
-        v->setProvider(plugin.data());
-      }
-    } else if ((*it)._type == Plugin::Data::IOValue::StringType) {
-      if (!KST::vectorTagNameNotUnique(nt, false)) {
-        plugin->outputStrings()[(*it)._name]->setTagName(nt);
-      } else if (plugin->outputStrings()[(*it)._name]->tagName() != nt) {
-        while (KST::vectorTagNameNotUnique(nt, false)) {
-          nt += "'";
-        }
-        plugin->outputStrings()[(*it)._name]->setTagName(nt);
-      }
-    } else if ((*it)._type == Plugin::Data::IOValue::PidType) {
-      // Nothing
-    } else if ((*it)._type == Plugin::Data::IOValue::FloatType) {
-      if (!KST::vectorTagNameNotUnique(nt, false)) {
-        plugin->outputScalars()[(*it)._name]->setTagName(nt);
-      } else if (plugin->outputScalars()[(*it)._name]->tagName() != nt) {
-        while (KST::vectorTagNameNotUnique(nt, false)) {
-          nt += "'";
-        }
-        plugin->outputScalars()[(*it)._name]->setTagName(nt);
-      }
-    }
-  }
-
-  return true;
-}
-
-
 bool KstFitDialogI::createCurve(KstPluginPtr plugin) {
   KstVectorPtr xVector;
   KstVectorPtr yVector;
@@ -442,7 +201,7 @@ bool KstFitDialogI::createCurve(KstPluginPtr plugin) {
 bool KstFitDialogI::new_I() {
   QString tagName = _tagName->text();
 
-  if (tagName != fit_defaultTag) {
+  if (tagName != plugin_defaultTag) {
     if (KST::dataTagNameNotUnique(tagName, true, this)) {
       _tagName->setFocus();
       return false;
@@ -461,7 +220,7 @@ bool KstFitDialogI::new_I() {
       if (saveInputs(plugin, pPtr)) {
         plugin->setPlugin(pPtr);
 
-        if (tagName == fit_defaultTag) {
+        if (tagName == plugin_defaultTag) {
           tagName = KST::suggestPluginName(_pluginList[pitem], _strCurve);
         }
 
@@ -500,10 +259,15 @@ bool KstFitDialogI::new_I() {
 }
 
 
-void KstFitDialogI::generateEntries(bool isWeighted, bool input, int& cnt, QWidget *parent, QGridLayout *grid, const QValueList<Plugin::Data::IOValue>& table) {
+void KstFitDialogI::generateEntries(bool input, int& cnt, QWidget *parent, QGridLayout *grid, const QValueList<Plugin::Data::IOValue>& table) {
+  
+  const QString& pluginName = _pluginList[PluginCombo->currentItem()];
+  const Plugin::Data& pluginData = PluginCollection::self()->pluginList()[PluginCollection::self()->pluginNameList()[pluginName]];
+  bool isWeighted = pluginData._isFitWeighted;
+  
   QString scalarLabelTemplate, vectorLabelTemplate, stringLabelTemplate;
   int iInputVector = 0;
-
+  
   if (input) {
     stringLabelTemplate = i18n("Input String - %1:");
     scalarLabelTemplate = i18n("Input Scalar - %1:");
@@ -597,16 +361,23 @@ void KstFitDialogI::generateEntries(bool isWeighted, bool input, int& cnt, QWidg
         } else {
           widget = new VectorSelector(parent, (*it)._name.latin1());
           connect(widget, SIGNAL(newVectorCreated(const QString&)), this, SIGNAL(modified()));
-        }
+        } 
       }
     } else {
       widget = new QLineEdit(parent, (*it)._name.latin1());
     }
 
     grid->addWidget(label, cnt, 0);
+    _pluginWidgets.push_back(label);
+    label->show();
+    
     grid->addWidget(widget, cnt, 1);
-
+    _pluginWidgets.push_back(widget);
+    widget->show();
+    
     if (!(*it)._description.isEmpty()) {
+      QWhatsThis::remove(label);
+      QWhatsThis::remove(widget);
       QWhatsThis::add(label, (*it)._description);
       QWhatsThis::add(widget, (*it)._description);
     }
@@ -616,100 +387,95 @@ void KstFitDialogI::generateEntries(bool isWeighted, bool input, int& cnt, QWidg
 }
 
 
-void KstFitDialogI::pluginChanged(int idx) {
-  // Get rid of the old widget
-  delete _frameWidget;
-  _frameWidget = 0L;
+bool KstFitDialogI::saveInputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p) {
+  bool rc = true;
+  
+  KST::vectorList.lock().readLock();
+  KST::scalarList.lock().readLock();
+  KST::stringList.lock().readLock();
+  const QValueList<Plugin::Data::IOValue>& itable = p->data()._inputs;
+  for (QValueList<Plugin::Data::IOValue>::ConstIterator it = itable.begin(); it != itable.end(); ++it) {
+    if ((*it)._type == Plugin::Data::IOValue::TableType) {
+      QObject *field = _pluginInputOutputFrame->child((*it)._name.latin1(), "VectorSelector");
+      KstVectorPtr v;
+      if (!field) {
+        field = _pluginInputOutputFrame->child((*it)._name.latin1(), "QLabel");
+        assert(field);
+        v = *KST::vectorList.findTag(static_cast<QLabel*>(field)->text());
+      } else {
+        VectorSelector *vs = static_cast<VectorSelector*>(field);
+        v = *KST::vectorList.findTag(vs->selectedVector());
+      }
+      if (v) {
+        v->writeLock(); // to match with plugin->writeLock()
+        if (plugin->inputVectors().contains((*it)._name) && plugin->inputVectors()[(*it)._name] != v) {
+          plugin->inputVectors()[(*it)._name]->writeUnlock();
+        }
+        plugin->inputVectors().insert((*it)._name, v);
+      } else if (plugin->inputVectors().contains((*it)._name)) {
+        plugin->inputVectors().erase((*it)._name);
+        rc = false;
+      }
+    } else if ((*it)._type == Plugin::Data::IOValue::StringType) {
+      QObject *field = _pluginInputOutputFrame->child((*it)._name.latin1(), "StringSelector");
+      assert(field);
+      StringSelector *ss = static_cast<StringSelector*>(field);
+      KstStringPtr s = *KST::stringList.findTag(ss->selectedString());
+      if (s == *KST::stringList.end()) {
+        QString val = ss->_string->currentText();
+        KstStringPtr newString = new KstString(ss->_string->currentText(), val, true, false);
+        newString->writeLock(); // to match with plugin->writeLock()
+        if (plugin->inputStrings().contains((*it)._name) && plugin->inputStrings()[(*it)._name] != s) {
+          plugin->inputStrings()[(*it)._name]->writeUnlock();
+        }
+        plugin->inputStrings().insert((*it)._name, newString);
+      } else {
+        s->writeLock(); // to match with plugin->writeLock()
+        if (plugin->inputStrings().contains((*it)._name) && plugin->inputStrings()[(*it)._name] != s) {
+          plugin->inputStrings()[(*it)._name]->writeUnlock();
+        }
+        plugin->inputStrings().insert((*it)._name, s);
+      }
+    } else if ((*it)._type == Plugin::Data::IOValue::PidType) {
+      // Nothing
+    } else if ((*it)._type == Plugin::Data::IOValue::FloatType) {
+      QObject *field = _pluginInputOutputFrame->child((*it)._name.latin1(), "ScalarSelector");
+      assert(field);
+      ScalarSelector *ss = static_cast<ScalarSelector*>(field);
+      KstScalarPtr s = *KST::scalarList.findTag(ss->selectedScalar());
+      if (s == *KST::scalarList.end()) {
+        bool ok;
+        double val = ss->_scalar->currentText().toDouble(&ok);
 
-  // Create a new one
-  QGridLayout *topGrid = dynamic_cast<QGridLayout*>(_pluginFrame->layout());
-  if (topGrid) {
-    topGrid->addWidget(_frameWidget = new QWidget(_pluginFrame, "Frame Widget"), 0, 0);
-  } else {
-    kdError() << "Somehow we lost the grid!" << endl;
-    return;
-  }
-
-  // Refill it
-  if (idx >= 0 && PluginCombo->count() > 0) {
-    const QString& pluginName = _pluginList[idx];
-    const Plugin::Data& pluginData = PluginCollection::self()->pluginList()[PluginCollection::self()->pluginNameList()[pluginName]];
-    int cnt = 0;
-
-    // Setup the grid and the "static" entries
-    int variables = pluginData._inputs.count() + pluginData._outputs.count();
-    QGridLayout *grid = new QGridLayout(_frameWidget, 4+variables, 2);
-    grid->setMargin(6);
-    grid->setSpacing(5);
-    grid->addWidget(new QLabel(i18n("Plugin name:"), _frameWidget), cnt, 0);
-    grid->addWidget(new QLabel(pluginData._readableName, _frameWidget), cnt, 1);
-    grid->addWidget(new QLabel(i18n("Description:"), _frameWidget), ++cnt, 0);
-    QLabel *lab = new QLabel(pluginData._description, _frameWidget);
-    lab->setAlignment(lab->alignment() | Qt::WordBreak);
-    grid->addWidget(lab, cnt, 1);
-
-    // Add a separator
-    QFrame* line = new QFrame(_frameWidget);
-    line->setFrameShadow(QFrame::Sunken);
-    line->setFrameShape(QFrame::HLine);
-    ++cnt;
-    grid->addMultiCellWidget(line, cnt, cnt, 0, 1);
-
-    // Generate the input values
-    ++cnt;
-    generateEntries(pluginData._isFitWeighted, true, cnt, _frameWidget, grid, pluginData._inputs);
-
-    if (!pluginData._inputs.isEmpty() && !pluginData._outputs.isEmpty()) {
-      line = new QFrame(_frameWidget);
-      line->setFrameShape(QFrame::HLine);
-      line->setFrameShadow(QFrame::Sunken);
-      line->setFrameShape(QFrame::HLine);
-      ++cnt;
-      grid->addMultiCellWidget(line, cnt, cnt, 0, 1);
+        if (ok) {
+          KstScalarPtr newScalar = new KstScalar(ss->_scalar->currentText(), val, true, false, false);
+          newScalar->writeLock(); // to match with plugin->writeLock()
+          if (plugin->inputScalars().contains((*it)._name) && plugin->inputScalars()[(*it)._name] != s) {
+            plugin->inputScalars()[(*it)._name]->writeUnlock();
+          }
+          plugin->inputScalars().insert((*it)._name, newScalar);
+        } else {
+          s->writeLock(); // to match with plugin->writeLock()
+          if (plugin->inputScalars().contains((*it)._name) && plugin->inputScalars()[(*it)._name] != s) {
+            plugin->inputScalars()[(*it)._name]->writeUnlock();
+          }
+          plugin->inputScalars().insert((*it)._name, s);
+        }
+      } else {
+        s->writeLock(); // to match with plugin->writeLock()
+        if (plugin->inputScalars().contains((*it)._name) && plugin->inputScalars()[(*it)._name] != s) {
+          plugin->inputScalars()[(*it)._name]->writeUnlock();
+        }
+        plugin->inputScalars().insert((*it)._name, s);
+      }
+    } else {
     }
-
-    // Generate the output values
-    ++cnt;
-    generateEntries(pluginData._isFitWeighted, false, cnt, _frameWidget, grid, pluginData._outputs);
-  }
-
-  // show it
-  _frameWidget->show();
-
-  // resize everything
-  QTimer::singleShot(0, this, SLOT(fixupLayout()));
-}
-
-
-void KstFitDialogI::showPluginManager() {
-  PluginManager *pm = new PluginManager(this, "Plugin Manager");
-  pm->exec();
-  delete pm;
-  updatePluginList();
-}
-
-
-void KstFitDialogI::updateScalarTooltip(const QString& n) {
-  KstScalarPtr s = *KST::scalarList.findTag(n);
-  QWidget *w = const_cast<QWidget*>(static_cast<const QWidget*>(sender()));
-  if (s) {
-    QToolTip::remove(w);
-    QToolTip::add(w, QString::number(s->value()));
-  } else {
-    QToolTip::remove(w);
-  }
-}
-
-
-void KstFitDialogI::updateStringTooltip(const QString& n) {
-  KstStringPtr s = *KST::stringList.findTag(n);
-  QWidget *w = const_cast<QWidget*>(static_cast<const QWidget*>(sender()));
-  if (s) {
-    QToolTip::remove(w);
-    QToolTip::add(w, s->value());
-  } else {
-    QToolTip::remove(w);
   }
+  KST::stringList.lock().readUnlock();
+  KST::scalarList.lock().readUnlock();
+  KST::vectorList.lock().readUnlock();
+  
+  return rc;
 }
 
 
