@@ -41,7 +41,7 @@ class KstHsDialogI : public KstHsDialog {
     static KstHsDialogI *globalInstance();
 
   private:
-    static KstHsDialogI *_inst;
+    static QGuardedPtr<KstHsDialogI> _inst;
     KstHistogramPtr _getPtr(const QString &tagin);
 
     bool _newDialog;
@@ -60,6 +60,7 @@ class KstHsDialogI : public KstHsDialog {
     void close();
     void reject();
   private:
+    static const QString& defaultTag;
     void _fillFieldsForEdit();
     void _fillFieldsForNew();
 };
