@@ -20,8 +20,9 @@
 #include <dcopobject.h>
 #include <dcopref.h>
 
-#include <qstringlist.h>
 #include <qcolor.h>
+#include <qstringlist.h>  //Qt 3.1.2 KDE 3.1.4
+
 
 // Warning: If you change something here, you could break existing scripts.
 
@@ -30,6 +31,7 @@ class KstIface : virtual public DCOPObject {
 k_dcop:
 	virtual void showDataManager() = 0;
 
+  virtual QStringList stringList() = 0;
   virtual QStringList scalarList() = 0;
   virtual QStringList vectorList() = 0;
   virtual QStringList objectList() = 0;
@@ -50,19 +52,18 @@ k_dcop:
   virtual double scalar(const QString& name) = 0;
   virtual bool setScalar(const QString& name, double value) = 0;
 
+  virtual const QString& string(const QString& name) = 0;
+  virtual bool setString(const QString& name, const QString& value) = 0;
+
   virtual double vector(const QString& name, int index) = 0;
   virtual bool setVector(const QString& name, int index, double value) = 0;
   virtual bool resizeVector(const QString& name, int newSize) = 0;
   virtual bool clearVector(const QString& name) = 0;
   virtual int vectorSize(const QString& name) = 0;
 
-  // FIXME: might want to remove this.  it generates an orphan vector, but
-  //        it's useful for testing purposes
   virtual const QString& generateVector(const QString& name, double from, double to, int points) = 0;
-
   virtual const QString& generateScalar(const QString& name, double value) = 0;
 
-
   virtual bool plotEquation(const QString& xvector, const QString& equation, const QString& plotName, const QColor& color) = 0;
   virtual bool plotEquation(const QString& xvector, const QString& equation, const QString& plotName) = 0;
   virtual bool plotEquation(double start, double end, int numSamples, const QString& equation, const QString& plotName, const QColor& color) = 0;
@@ -102,8 +103,6 @@ k_dcop:
   virtual void newFile() = 0;
   virtual bool open(const QString& fileName) = 0;
 
-//-----------------Additional proposed functions--------------------------
-
   //Sets the window MDI mode
   //mode 1=Toplevel, 2=Childframe, 3=Tab Page, 4=IDEAl Mode
   virtual bool setMDIMode(int mode) = 0;
@@ -132,6 +131,8 @@ k_dcop:
 
   //zoom (maximize) plot
   virtual bool maximizePlot(const QString& plotName) = 0;
+  virtual bool unmaximizePlot(const QString& plotName) = 0;
+  virtual bool toggleMaximizePlot(const QString& plotName) = 0;
 
   //change plot axes
   virtual bool setPlotAxes(const QString& plotName,
@@ -218,6 +219,13 @@ k_dcop:
   //create a plot marker
   //returns false if plotName is not valid or markerValue already exists, else returns true
   virtual bool addPlotMarker(const QString &plotName, double markerValue) = 0;
+
+  virtual bool changeDataFile(const QString& fileName) = 0;
+
+  virtual bool changeDataFile(const QString& vector, const QString& fileName) = 0;
+
+  virtual bool changeDataFiles(const QStringList& vectors, const QString& fileName) = 0;
+
 };
 
 #endif
