@@ -33,6 +33,7 @@
 #include "kstplugin.h"
 #include "kstpsd.h"
 #include "kstrvector.h"
+#include "kstsvector.h"
 #include "kstvcurve.h"
 #include "kstviewwindow.h"
 #include "plugincollection.h"
@@ -183,7 +184,7 @@ bool KstIfaceImpl::plotEquation(double start, double end, int numSamples, const
     return false;
   }
 
-  etag = "Eq-" + QString(equation).replace(QRegExp("[\\[\\]\\s]"), "_");
+  etag = KST::suggestEQName(QString(equation).replace(QRegExp("[\\[\\]\\s]"), "_"));
   ptag = "P-" + plotName;
 
   if (!plotName.isEmpty()) {
@@ -227,17 +228,13 @@ bool KstIfaceImpl::plotEquation(double start, double end, int numSamples, const
     }
   }
 
-  while (KST::dataTagNameNotUnique(etag, false)) {
-    etag += '\'';
-  }
-
   KstEquationPtr eq = new KstEquation(etag, equation, start, end, numSamples);
 
   if (!eq->isValid()) {
     return false;
   }
 
-  KstVCurvePtr vc = new KstVCurve(etag + "-curve", eq->vX(), eq->vY(), 0L, 0L, color.isValid() ? color : KstColorSequence::next(plot->Curves,plot->backgroundColor()));
+  KstVCurvePtr vc = new KstVCurve(KST::suggestCurveName(etag, true), eq->vX(), eq->vY(), 0L, 0L, 0L, 0L, color.isValid() ? color : KstColorSequence::next(plot->Curves,plot->backgroundColor()));
   KST::dataObjectList.lock().writeLock();
   KST::dataObjectList.append(KstDataObjectPtr(eq));
   KST::dataObjectList.append(KstDataObjectPtr(vc));
@@ -277,7 +274,7 @@ bool KstIfaceImpl::plotEquation(const QString& xvector, const QString& equation,
 
   v = *it;
 
-  etag = "Eq-" + QString(equation).replace(QRegExp("[\\[\\]\\s]"), "_");
+  etag = KST::suggestEQName(QString(equation).replace(QRegExp("[\\[\\]\\s]"), "_"));
   ptag = "P-" + plotName;
 
   if (!plotName.isEmpty()) {
@@ -322,9 +319,6 @@ bool KstIfaceImpl::plotEquation(const QString& xvector, const QString& equation,
     }
   }
 
-  while (KST::dataTagNameNotUnique(etag, false)) {
-    etag += '\'';
-  }
 
   KstEquationPtr eq = new KstEquation(etag, equation, v, true);
 
@@ -332,14 +326,14 @@ bool KstIfaceImpl::plotEquation(const QString& xvector, const QString& equation,
     return false;
   }
 
-  KstVCurvePtr vc = new KstVCurve(etag + "-curve", eq->vX(), eq->vY(), 0L, 0L, color.isValid() ? color : KstColorSequence::next(plot->Curves,plot->backgroundColor()));
+  KstVCurvePtr vc = new KstVCurve(KST::suggestCurveName(etag, true), eq->vX(), eq->vY(), 0L, 0L, 0L, 0L, color.isValid() ? color : KstColorSequence::next(plot->Curves,plot->backgroundColor()));
   KST::dataObjectList.lock().writeLock();
   KST::dataObjectList.append(KstDataObjectPtr(eq));
   KST::dataObjectList.append(KstDataObjectPtr(vc));
   KST::dataObjectList.lock().writeUnlock();
 
   plot->Curves.append(KstBaseCurvePtr(vc));
-  
+
   _doc->forceUpdate();
   _doc->setModified();
 
@@ -350,12 +344,14 @@ const QString& KstIfaceImpl::generateScalar(const QString& name, double value) {
   KstScalarPtr s = new KstScalar(name, value);
   KstReadLocker rl(s);
   s->setOrphan(true);
+  s->setEditable(true);
   return s->tagName();
 }
 
 
 const QString& KstIfaceImpl::generateVector(const QString& name, double from, double to, int points) {
-  KstVectorPtr v = KstVector::generateVector(from, to, points, name);
+  KstVectorPtr v = new KstSVector(from, to, points, name);
+  KST::addVectorToList(v);
   KstReadLocker rl(v);
   return v->tagName();
 }
@@ -456,7 +452,7 @@ bool KstIfaceImpl::setScalar(const QString& name, double value) {
   KstReadLocker ml(&KST::scalarList.lock());
   KstScalarList::Iterator it = KST::scalarList.findTag(name);
 
-  if (it == KST::scalarList.end()) {
+  if (it == KST::scalarList.end() || !(*it)->editable()) {
     return false;
   }
 
@@ -600,7 +596,7 @@ QString KstIfaceImpl::createPlot(const QString& window, const QString& name) {
   KstApp*           pApp = KstApp::inst();
   Kst2DPlotPtr      p;
   QString           plotName;
-  
+
   pWindow = pApp->findWindow(window);
   if (pWindow) {
     pView = dynamic_cast<KstViewWindow*>(pWindow);
@@ -758,7 +754,7 @@ bool KstIfaceImpl::removeCurveFromPlot(const QString& window, const QString& plo
 bool KstIfaceImpl::removeCurveFromPlot(const QString& plot, const QString& curve) {
   KstApp *app = KstApp::inst();
   KMdiChildView *activewin = app->activeWindow();
-  
+
   return removeCurveFromPlot(activewin, plot, curve);
 }
 
@@ -782,7 +778,7 @@ const QString& KstIfaceImpl::createCurve(const QString& name, const QString& xVe
     n += "'";
   }
 
-  KstVCurvePtr c = new KstVCurve(n, vx, vy, ex, ey, color);
+  KstVCurvePtr c = new KstVCurve(n, vx, vy, ex, ey, ex, ey, color);
   KST::dataObjectList.append(KstDataObjectPtr(c));
   KST::dataObjectList.lock().writeUnlock();
   _doc->forceUpdate();
@@ -841,7 +837,7 @@ bool KstIfaceImpl::printPostScript(const QString& url) {
     app->immediatePrintActiveWindowToFile(url);
     bRetVal = true;
   }
-  
+
   return bRetVal;
 }
 
@@ -869,7 +865,7 @@ const QString& KstIfaceImpl::loadVector(const QString& file, const QString& fiel
   KstDataSourcePtr src;
   /* generate or find the kstfile */
   KST::dataSourceList.lock().writeLock();
-  KstDataSourceList::Iterator it = KST::dataSourceList.findFileName(file);
+  KstDataSourceList::Iterator it = KST::dataSourceList.findReusableFileName(file);
 
   if (it == KST::dataSourceList.end()) {
     src = KstDataSource::loadSource(file);
@@ -1015,7 +1011,7 @@ bool KstIfaceImpl::setTileMode(int mode) {
 bool KstIfaceImpl::closeWindow(const QString& windowname) {
   KstApp *app = KstApp::inst();
   KMdiChildView *win = app->findWindow(windowname);
-  
+
   if (win) {
     app->closeWindow(win, true);
     return true;
@@ -1040,7 +1036,7 @@ bool KstIfaceImpl::activateWindow(const QString& windowname) {
 bool KstIfaceImpl::positionWindow(int topleftX, int topleftY, int width, int height) {
   KstApp *app = KstApp::inst();
   KMdiChildView *win = app->activeWindow();
-  
+
   if (win) {
     if (topleftX >= 0 && topleftY >= 0 && width >= 0 && height >= 0){
       QRect rectangle(topleftX, topleftY, width, height);
@@ -1063,7 +1059,7 @@ bool KstIfaceImpl::maximizePlot(const QString& plotName) {
       Kst2DPlotList::Iterator plot_iter = plotlist.findTag(plotName);
       if (plot_iter != plotlist.end()) {
         app->deleteIterator(iter);
-        (*plot_iter)->zoom(true);
+        (*plot_iter)->setMaximized(true);
         return true;
       }
     }
@@ -1084,7 +1080,7 @@ bool KstIfaceImpl::unmaximizePlot(const QString& plotName) {
       Kst2DPlotList::Iterator plot_iter = plotlist.findTag(plotName);
       if (plot_iter != plotlist.end()) {
         app->deleteIterator(iter);
-        (*plot_iter)->zoom(false);
+        (*plot_iter)->setMaximized(false);
         return true;
       }
     }
@@ -1116,10 +1112,10 @@ bool KstIfaceImpl::toggleMaximizePlot(const QString& plotName) {
 }
 
 bool KstIfaceImpl::setPlotAxes(const QString& plotName,
-  			   int XLower,
-  			   int XUpper,
-			   int YLower,
-			   int YUpper) {
+    int XLower,
+    int XUpper,
+    int YLower,
+    int YUpper) {
   //find the plot
   KstApp *app = KstApp::inst();
   KMdiIterator<KMdiChildView*> *iter = app->createIterator();
@@ -1143,12 +1139,12 @@ bool KstIfaceImpl::setPlotAxes(const QString& plotName,
 }
 
 QString KstIfaceImpl::createHistogram(const QString& name,
-			       const QString& vector,
-             double min,
-			       double max,
-			       int numBins,
-			       int normalizationType,
-             const QColor& color) {
+    const QString& vector,
+    double min,
+    double max,
+    int numBins,
+    int normalizationType,
+    const QColor& color) {
 
   //get the vector
   KST::vectorList.lock().readLock();
@@ -1156,7 +1152,7 @@ QString KstIfaceImpl::createHistogram(const QString& name,
   KST::vectorList.lock().readUnlock();
 
   if (iter == KST::vectorList.end()) {
-    return QString();
+    return QString::null;
   }
 
   //set the normalization type
@@ -1213,14 +1209,14 @@ QString KstIfaceImpl::createHistogram(const QString& name,
 }
 
 QString KstIfaceImpl::createPowerSpectrum(const QString & name,
-				   const QString& vector,
-  				 bool appodize,
-				   bool removeMean,
-				   int fftLength,
-				   const QString& rateUnits,
-				   double sampleRate,
-				   const QString& vectorUnits,
-           const QColor& color) {
+    const QString& vector,
+    bool appodize,
+    bool removeMean,
+    int fftLength,
+    const QString& rateUnits,
+    double sampleRate,
+    const QString& vectorUnits,
+    const QColor& color) {
   //get the vector
   KST::vectorList.lock().readLock();
   KstVectorList::Iterator iter = KST::vectorList.findTag(vector);
@@ -1251,7 +1247,7 @@ QString KstIfaceImpl::createPowerSpectrum(const QString & name,
 
   KstPSDPtr powerspectrum = new KstPSD(pstag, *iter, sampleRate, true, fftLength,
                                appodize, removeMean, vectorUnits, rateUnits);
-  
+
   // FIXME: should we be plotting this?  Otherwise, remove color
   //KstVCurvePtr vc = new KstVCurve(pstag + "-curve", *iter, powerspectrum->vY(), 0L, 0L, color.isValid() ? color : QColor("darkBlue"));
 
@@ -1268,10 +1264,10 @@ QString KstIfaceImpl::createPowerSpectrum(const QString & name,
 }
 
 QString KstIfaceImpl::createEvent(const QString& name,
-   			   const QString& expression,
-  			   const QString& description,
-			   int debugLogType,
-			   const QString& email) {
+    const QString& expression,
+    const QString& description,
+    int debugLogType,
+    const QString& email) {
 
   //suggest a name if not supplied
   QString evtag_end;
@@ -1318,7 +1314,7 @@ QString KstIfaceImpl::createEvent(const QString& name,
   else {
     event->setLogKstDebug(false);
   }
-  
+
   if (email.isEmpty()) {
     event->setLogEMail(false);
   } else {
@@ -1487,7 +1483,7 @@ QString KstIfaceImpl::createMatrix(const QString& name, const QString& vector, d
   KST::vectorList.lock().readUnlock();
 
   if (iter == KST::vectorList.end()) {
-    return QString();
+    return QString::null;
   }
 
   //suggest a name if not supplied
@@ -1506,20 +1502,20 @@ QString KstIfaceImpl::createMatrix(const QString& name, const QString& vector, d
 
   //basic check to see if parameters are consistent with vector
   (*iter)->readLock();
-  if (!useMaxX && (*iter)->sampleCount() < (int)(xNumSteps * yNumSteps)) {
+  if (!useMaxX && (*iter)->length() < int(xNumSteps * yNumSteps)) {
     (*iter)->readUnlock();
-    return QString();
+    return QString::null;
   }
 
   //can't let yNumSteps be less than 1, or will divide by 0
   if (yNumSteps < 1) {
     (*iter)->readUnlock();
-    return QString();
+    return QString::null;
   }
 
   if (xStepSize <= 0 || yStepSize <= 0) {
     (*iter)->readUnlock();
-    return QString();
+    return QString::null;
   }
   (*iter)->readUnlock();
 
@@ -1589,13 +1585,10 @@ QString KstIfaceImpl::createImage(const QString &name,
     if (lowerZ > upperZ) {
       return QString::null;
     }
-    KPalette* pal = new KPalette(paletteName);
-    if (!pal) {
-      return QString::null;
-    }
     if (numContours < 1) {
       return QString::null;
     }
+    KPalette* pal = new KPalette(paletteName);
     matrix->readLock();
     image = new KstImage(imgtag, matrix, lowerZ, upperZ, false, pal,
                          numContours, contourColor.isValid() ? contourColor : QColor("darkBlue"), 0);
@@ -1635,4 +1628,86 @@ bool KstIfaceImpl::addPlotMarker(const QString &plotName, double markerValue) {
   app->deleteIterator(iter);
   return false;
 }
+
+
+bool KstIfaceImpl::changeDataFile(const QString& fileName) {
+  QStringList vl;
+  KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
+  for (KstRVectorList::ConstIterator i = rvl.begin(); i != rvl.end(); ++i) {
+    vl += (*i)->tagName();
+  }
+  return changeDataFiles(vl, fileName);
+}
+
+
+bool KstIfaceImpl::changeDataFile(const QString& vector, const QString& fileName) {
+  return changeDataFile(vector, fileName, true);
+}
+
+
+bool KstIfaceImpl::changeDataFile(const QString& vector, const QString& fileName, bool update) {
+  KST::vectorList.lock().readLock();
+  KstRVectorPtr rvp = kst_cast<KstRVector>(*KST::vectorList.findTag(vector));
+  KST::vectorList.lock().readUnlock();
+  if (!rvp) {
+    return false;
+  }
+
+  KST::dataSourceList.lock().writeLock();
+  KstDataSourceList::Iterator it = KST::dataSourceList.findReusableFileName(fileName);
+  KstDataSourcePtr file;
+  QString invalidSources;
+  int invalid = 0;
+
+  if (it == KST::dataSourceList.end()) {
+    file = KstDataSource::loadSource(fileName);
+    if (!file || !file->isValid() || file->isEmpty()) {
+      KST::dataSourceList.lock().writeUnlock();
+      return false;
+    }
+    KST::dataSourceList.append(file);
+  } else {
+    file = *it;
+  }
+
+  KST::dataSourceList.lock().writeUnlock();
+
+  rvp->writeLock();
+  file->writeLock();
+
+  if (!file->isValidField(vector)) {
+    file->writeUnlock();
+    rvp->writeUnlock();
+    return false;
+  }
+
+  rvp->changeFile(file);
+  
+  file->writeUnlock();
+  bool rc = rvp->isValid();
+  rvp->writeUnlock();
+
+  if (update) {
+    KstApp::inst()->forceUpdate();
+  }
+
+  return rc;
+}
+
+
+bool KstIfaceImpl::changeDataFiles(const QStringList& vectors, const QString& fileName) {
+  bool rc = true;
+  bool any = false;
+  for (QStringList::ConstIterator i = vectors.begin(); i != vectors.end(); ++i) {
+    bool thisrc = changeDataFile(*i, fileName, false);
+    any = thisrc || any;
+    rc = thisrc && rc;
+  }
+
+  KstApp::inst()->forceUpdate();
+
+  return rc;
+}
+
+
 // vim: ts=2 sw=2 et
