@@ -22,6 +22,8 @@
 #include <qpainter.h>
 
 #include "kstplotbase.h"
+#include "kstsharedptr.h"
+#include "rwlock.h"
 
 enum KstJustifyType { CxBy, CxTy, CxCy, LxBy, LxTy, LxCy, RxBy, RxTy, RxCy };
 
@@ -29,14 +31,18 @@ enum KstJustifyType { CxBy, CxTy, CxCy, LxBy, LxTy, LxCy, RxBy, RxTy, RxCy };
  *@author cbn
  */
 
-class KstLabel {
+class KstLabel : public KstShared, public KstRWLock {
   public:
     KstLabel();
     KstLabel(const QString& in_text, KstJustifyType in_j = CxBy,
         float in_rotation = 0.0, float in_X = 0.0,
         float in_Y = 0.0, bool is_sample = false);
+    KstLabel(const KstLabel* label);
     ~KstLabel();
 
+    void setDirty(bool dirty = true);
+    bool dirty() const;
+
     void setText(const QString& text);
     void setRotation(float new_rotation);
     void setRelPosition(float new_x, float new_y);
@@ -66,7 +72,8 @@ class KstLabel {
     void draw(QPainter& p, int px, int py, bool justify = true, bool doDraw = true);
 
     /** Interpret special characters, default = true */
-    void setInterpreted(bool interpreted) { _interpret = interpreted; }
+    void setInterpreted(bool interpreted);
+    bool interpreted() const;
 
     void setSize(int size);
     void setFontName(const QString& fontName);
@@ -75,9 +82,10 @@ class KstLabel {
 
     void save(QTextStream& ts, const QString& indent = QString::null,
               bool save_pos=true);
-    void read(QDomElement& e);
+    void read(const QDomElement& e);
 
     void setDoScalarReplacement(bool in_do);
+    bool doScalarReplacement() const;
 
     inline double x() const { return _x; }
     inline double y() const { return _y; }
@@ -89,17 +97,16 @@ class KstLabel {
     /** Computes the font size, given the size of the painter, and Size */
     int fontSize(QPainter& p);
 
-    bool _interpret;
     float Rotation;
     QString Text;
     KstJustifyType _justification;
 
     QString SymbolFontName;
     QString FontName;
-    bool _usePlotColor;
     QColor _color;
     int Size;
     float Width; /* set by draw */
+    int LineSpacing;
 
     /** relative position of label: stored for use by the parent class: since
       the geometry of the parent may have scaled and un-scaled regions, KstLabel
@@ -111,8 +118,11 @@ class KstLabel {
     KstScalarList ScalarsUsed;
     KstStringList StringsUsed;
 
-    bool doScalarReplacement;
-    bool IsSample;
+    bool _interpret : 1;
+    bool _usePlotColor : 1;
+    bool _doScalarReplacement : 1;
+    bool IsSample : 1;
+    bool _dirty : 1;
 };
 
 
@@ -132,7 +142,8 @@ class KstLabelDrag : public QDragObject {
     QPoint _hs;
 };
 
-typedef QPtrList<KstLabel> KstLabelList;
+typedef KstSharedPtr<KstLabel> KstLabelPtr;
+typedef QValueList<KstLabelPtr> KstLabelList;
 
 #endif
 // vim: ts=2 sw=2 et
