@@ -16,10 +16,10 @@
  *                                                                         *
  ***************************************************************************/
 
+#include "dialoglauncher.h"
 #include "kstdatacollection.h"
 #include "kstdebug.h"
 #include "kstmatrix.h"
-#include "kstmatrixdialog_i.h"
 
 #include <qstylesheet.h>
 
@@ -30,11 +30,12 @@
 KstMatrix::KstMatrix(const QString &in_tag, KstVectorPtr in_Z, uint nX, uint nY, double minX, double minY, double stepX, double stepY, bool useMaxX) : KstDataObject() {
   _zVector = in_Z;
   commonConstructor(in_tag, nX, nY, minX, minY, stepX, stepY, useMaxX);
-  update();
+  setDirty();
 }
 
 
-KstMatrix::KstMatrix(QDomElement &e) {
+KstMatrix::KstMatrix(const QDomElement &e)
+: KstDataObject(e) {
   QString in_tag, zVectorName, in_nX, in_nY, in_minX, in_minY, in_stepX, in_stepY;
   bool in_useMaxX = false;
 
@@ -78,6 +79,7 @@ KstMatrix::KstMatrix(QDomElement &e) {
                     in_useMaxX);
 }
 
+
 bool KstMatrix::loadInputs() {
   QValueList<QPair<QString,QString> >::Iterator i;
   KST::vectorList.lock().readLock();
@@ -93,12 +95,11 @@ bool KstMatrix::loadInputs() {
   }
   KST::vectorList.lock().readUnlock();
 
-  _zVector->readLock();
-  update(); // FIXME: this update happens in the wrong thread
-  _zVector->readUnlock();
+  setDirty();
   return true;
 }
 
+
 void KstMatrix::commonConstructor(const QString &in_tag, uint nX, uint nY, double minX, double minY, double stepX, double stepY, bool useMaxX) {
   //assuming that nX*nY <= length of in_Z. Should have been checked by dialog.
   //If for some reason not, update() will reduce _nX
@@ -117,6 +118,7 @@ void KstMatrix::commonConstructor(const QString &in_tag, uint nX, uint nY, doubl
 KstMatrix::~KstMatrix() {
 }
 
+
 //return the value at the specified matrix position
 //return false if the position is out of bounds,
 //otherwise uses a close position
@@ -134,7 +136,19 @@ bool KstMatrix::value(double x, double y, double& z) {
   return true;
 }
 
-
+//return the value at the specified matrix position
+//return false if the position is out of bounds
+bool KstMatrix::valueRaw(int x, int y, double& z) {
+  int index =  x * _nY + y;
+  if (x >= _nX || x < 0 || y >= _nY || y < 0) {
+    return false;
+  }
+  if (index >= NS || index < 0 ) {
+    return false;
+  }
+  z = _zVector->value(index);
+  return true;
+}
 
 //change the matrix parameters
 void KstMatrix::changeParameters(const QString &in_tag, KstVectorPtr in_Z, int nX, int nY, double minX, double minY, double stepX, double stepY, bool useMaxX) {
@@ -150,9 +164,10 @@ void KstMatrix::changeParameters(const QString &in_tag, KstVectorPtr in_Z, int n
   _stepY = stepY;
   _useMaxX = useMaxX;
   setTagName(in_tag);
-  update();
+  setDirty();
 }
 
+
 void KstMatrix::save(QTextStream &ts, const QString& indent) {
   QString l2 = indent + "  ";
   ts << indent << "<matrix>" << endl;
@@ -168,39 +183,57 @@ void KstMatrix::save(QTextStream &ts, const QString& indent) {
   ts << indent << "</matrix>" << endl;
 }
 
+
 KstObject::UpdateType KstMatrix::update(int update_counter) {
-  if (KstObject::checkUpdateCounter(update_counter))
-    return NO_CHANGE;
+  bool force = dirty();
+  setDirty(false);
 
-  if (_useMaxX || (_nX*_nY > _zVector->sampleCount())) {
-    _nX = (uint)floor(_zVector->sampleCount() / _nY);
+  if (KstObject::checkUpdateCounter(update_counter) && !force) {
+    return lastUpdateResult();
   }
 
-  NS = _nX * _nY;
-
-  //find max and min within the grid
-  if (NS > 0) {
-    _minZ = _zVector->value(0);
-    _maxZ = _zVector->value(0);
-  } else {
-    _minZ = 0;  //no grid, use some arbitrary value
-    _maxZ = 0;
-  }
-  for (int i = 0; i < NS; i++) {
-    if (_minZ > _zVector->value(i)) {
-      _minZ = _zVector->value(i);
+  bool updated = UPDATE == _zVector->update(update_counter);
+  if (updated || force) {
+    if (_useMaxX || _nX*_nY > _zVector->length()) {
+      _nX = (uint)floor(_zVector->length() / _nY);
     }
-    if (_maxZ < _zVector->value(i)) {
-      _maxZ = _zVector->value(i);
+
+    NS = _nX * _nY;
+
+    //find max and min within the grid
+    if (NS > 0) {
+      _minZ = _zVector->value(0);
+      _maxZ = _zVector->value(0);
+    } else {
+      _minZ = 0;  //no grid, use some arbitrary value
+      _maxZ = 0;
     }
+    for (int i = 0; i < NS; i++) {
+      if (_minZ > _zVector->value(i)) {
+        _minZ = _zVector->value(i);
+      }
+      if (_maxZ < _zVector->value(i)) {
+        _maxZ = _zVector->value(i);
+      }
+    }
+    return setLastUpdateResult(UPDATE);
   }
-  return UPDATE;
+  return setLastUpdateResult(NO_CHANGE);
+}
+
+
+KstVectorPtr KstMatrix::vector() const {
+  return _zVector;
+}
+
+
+QString KstMatrix::propertyString() const {
+  return i18n("Using vector %1").arg(_zVector->tagName());
 }
 
-QString KstMatrix::propertyString() const { return i18n("Using vector %1").arg(_zVector->tagName());}
 
 void KstMatrix::_showDialog() {
-  KstMatrixDialogI::globalInstance()->show_Edit(tagName());
+  KstDialogs::showMatrixDialog(tagName());
 }
 
 // vim: ts=2 sw=2 et
