@@ -18,51 +18,65 @@
 #ifndef KSTOBJECT_H
 #define KSTOBJECT_H
 
-#include "kstsharedptr.h"
-
-#include "rwlock.h"
-
 #include <qmutex.h>
 #include <qobject.h>
 #include <qstring.h>
 #include <qstringlist.h>
 
+#include "kst_export.h"
+#include "kstsharedptr.h"
+#include "rwlock.h"
+
 class KstObjectPrivate;
 
-// BINARY COMPATIBILITY IS NOT YET GUARANTEED
-class KstObject : public KstShared, public QObject, public KstRWLock {
-public:
-  KstObject();
-  virtual ~KstObject();
+class KST_EXPORT KstObject : public KstShared, public QObject, public KstRWLock {
+  public:
+    KstObject();
+    virtual ~KstObject();
 
-  typedef enum {NO_CHANGE=0, UPDATE} UpdateType;
+    enum UpdateType { NO_CHANGE = 0, UPDATE };
 
-  virtual UpdateType update(int updateCounter = -1) = 0;
-  virtual const QString& tagName() const;
-  virtual void setTagName(const QString& newTag);
+    virtual UpdateType update(int updateCounter = -1) = 0;
+    virtual const QString& tagName() const;
+    virtual void setTagName(const QString& newTag);
 
-  virtual QString tagLabel() const;
-  // Returns count - 2 to account for "this" and the list pointer
-  virtual int getUsage() const;
+    virtual QString tagLabel() const;
+    // Returns count - 2 to account for "this" and the list pointer
+    virtual int getUsage() const;
 
-  // Returns true if update has already been done
-  virtual bool checkUpdateCounter(int update_counter);
+    // Returns true if update has already been done
+    virtual bool checkUpdateCounter(int update_counter);
 
-  int operator==(const QString&) const;
+    int operator==(const QString&) const;
 
-protected:
-  virtual void virtual_hook(int id, void *data);
-  int _lastUpdateCounter;
+    virtual bool deleteDependents();
 
-private:
-  QString _tag;
-  KstObjectPrivate *d;
+    // @since 1.1.0
+    void setDirty();
+    // @since 1.1.0
+    bool dirty() const;
 
-  // NOTE: In order to preserve binary compatibility with plugins, you must
-  //       not add, remove, or change member variables or virtual functions.
-  //       You must also not remove or change non-virtual functions.  You can
-  //       add new variables to the KstObjectPrivate pointer if you need
-  //       them.  You can simulate virtual functions with the virtual_hook().
+  protected:
+    virtual void virtual_hook(int id, void *data);
+    int _lastUpdateCounter;
+
+    // @since 1.1.0
+    UpdateType setLastUpdateResult(UpdateType result);
+    // @since 1.1.0
+    UpdateType lastUpdateResult() const;
+
+    // @since 1.1.0
+    void setDirty(bool dirty);
+
+  private:
+    QString _tag;
+    KstObjectPrivate *d;
+
+    // NOTE: In order to preserve binary compatibility with plugins, you must
+    //       not add, remove, or change member variables or virtual functions.
+    //       You must also not remove or change non-virtual functions.  You can
+    //       add new variables to the KstObjectPrivate pointer if you need
+    //       them.  You can simulate virtual functions with the virtual_hook().
 };
 
 typedef KstSharedPtr<KstObject> KstObjectPtr;
@@ -78,14 +92,22 @@ class KstObjectList : public QValueList<T> {
     virtual ~KstObjectList() { }
 
     KstObjectList& operator=(const KstObjectList& l) {
-      // FIXME: investigate, should we just deep copy?
       this->QValueList<T>::operator=(l);
       return *this;
     }
 
     virtual QStringList tagNames() {
       QStringList rc;
-      for (typename QValueList<T>::Iterator it = QValueList<T>::begin(); it != QValueList<T>::end(); ++it) {
+      for (typename QValueList<T>::ConstIterator it = QValueList<T>::begin(); it != QValueList<T>::end(); ++it) {
+        rc << (*it)->tagName();
+      }
+      return rc;
+    }
+
+    // @since 1.1.0
+    QStringList tagNames() const {
+      QStringList rc;
+      for (typename QValueList<T>::ConstIterator it = QValueList<T>::begin(); it != QValueList<T>::end(); ++it) {
         rc << (*it)->tagName();
       }
       return rc;
@@ -120,8 +142,8 @@ class KstObjectList : public QValueList<T> {
       return -1;
     }
 
-    virtual typename QValueList<T>::iterator removeTag(const QString& x) {
-      typename QValueList<T>::iterator it = findTag(x);
+    virtual typename QValueList<T>::Iterator removeTag(const QString& x) {
+      typename QValueList<T>::Iterator it = findTag(x);
       if (it != QValueList<T>::end()) {
         return QValueList<T>::remove(it);
       }
@@ -179,10 +201,19 @@ class KstObjectMap : public QMap<QString,T> {
   public:
     KstObjectMap() : QMap<QString,T>() {}
     virtual ~KstObjectMap() {}
-    
+
     virtual QStringList tagNames() {
       QStringList rc;
-      for (typename QMap<QString,T>::Iterator it = QMap<QString,T>::begin(); it != QMap<QString,T>::end(); ++it) {
+      for (typename QMap<QString,T>::ConstIterator it = QMap<QString,T>::begin(); it != QMap<QString,T>::end(); ++it) {
+        rc << it.data()->tagName();
+      }
+      return rc;
+    }
+
+    // @since 1.1.0
+    QStringList tagNames() const {
+      QStringList rc;
+      for (typename QMap<QString,T>::ConstIterator it = QMap<QString,T>::begin(); it != QMap<QString,T>::end(); ++it) {
         rc << it.data()->tagName();
       }
       return rc;
@@ -220,5 +251,10 @@ These are wrong.  We should not assume that key(x) == x->tagName().
 */
 };
 
+template <typename T, typename U>
+inline KstSharedPtr<T> kst_cast(KstSharedPtr<U> object) {
+  return dynamic_cast<T*>(object.data());
+}
+
 #endif
 // vim: ts=2 sw=2 et
