@@ -23,13 +23,13 @@ class KstPlotGroup : public KstMetaPlot {
   Q_OBJECT
   public:
     KstPlotGroup();
-    KstPlotGroup(QDomElement& e);
+    KstPlotGroup(const QDomElement& e);
+    KstPlotGroup(const KstPlotGroup& plotGroup);
     virtual ~KstPlotGroup();
 
-    virtual UpdateType update();
     virtual void save(QTextStream& ts, const QString& indent = QString::null);
     virtual void saveTag(QTextStream& ts, const QString& indent = QString::null);
-    
+
     virtual bool removeChild(KstViewObjectPtr obj, bool recursive = false);
     virtual bool popupMenu(KPopupMenu *menu, const QPoint& pos, KstViewObjectPtr topLevelParent);
     virtual bool layoutPopupMenu(KPopupMenu *menu, const QPoint& pos, KstViewObjectPtr topParent);
@@ -38,6 +38,8 @@ class KstPlotGroup : public KstMetaPlot {
 
   public slots:
     virtual void paint(KstPaintType type, QPainter& p);
+    virtual void copyObject();
+    virtual void copyObjectQuietly(KstViewObject& parent, const QString& name = QString::null) const;
     void flatten();
 
   protected:
