@@ -429,7 +429,7 @@ KstObject::UpdateType KstRVector::update(int update_counter) {
 KstObject::UpdateType KstRVector::doUpdate(bool force) {
   int i, k, shift, n_read=0;
   int ave_nread;
-  int new_f0, new_nf, tmp_fn;
+  int new_f0, new_nf;
 
   checkIntegrity();
 
@@ -459,16 +459,18 @@ KstObject::UpdateType KstRVector::doUpdate(bool force) {
       new_nf = fc - new_f0;
     }
     if (new_nf <= 0) {
-      new_nf = new_f0 = 0;
+      // Tried to read starting past the end.
+      new_f0 = 0;
+      new_nf = 1;
     }
   }
 
   if (DoSkip) {
-      // change new_f0 and new_nf so they both lie on skip boundaries
-      if (new_f0 != 0) {
-        new_f0 = (((new_f0-1)/Skip)+1)*Skip;
-      }
-      new_nf = (new_nf/Skip)*Skip;
+    // change new_f0 and new_nf so they both lie on skip boundaries
+    if (new_f0 != 0) {
+      new_f0 = ((new_f0-1)/Skip+1)*Skip;
+    }
+    new_nf = (new_nf/Skip)*Skip;
   }
 
   if (NF == new_nf && F0 == new_f0 && !force) {
@@ -489,6 +491,7 @@ KstObject::UpdateType KstRVector::doUpdate(bool force) {
       _numSamples = (NF-1)*SPF+1;
     }
 
+    // FIXME: use memmove()
     for (i = 0; i < _numSamples; i++) {
       _v[i] = _v[i+shift];
     }
@@ -563,6 +566,7 @@ KstObject::UpdateType KstRVector::doUpdate(bool force) {
       bool rc = resize((new_nf - 1)*SPF + 1);
       if (!rc) {
         // FIXME: handle failed resize
+        abort();
       }
     }
 
@@ -572,10 +576,17 @@ KstObject::UpdateType KstRVector::doUpdate(bool force) {
 
     // read the new data from file
     if (_file->samplesPerFrame(_field) > 1) {
+      assert(new_nf - NF - 1 > 0 || new_nf - NF - 1 == -1);
+      assert(new_f0 + NF >= 0);
+      assert(new_f0 + new_nf - 1 >= 0);
       n_read = _file->readField(_v+NF*SPF, _field, new_f0 + NF, new_nf - NF - 1);
-      n_read += _file->readField(_v+(new_nf-1)*SPF, _field, new_f0 + new_nf-1, -1);
+      n_read += _file->readField(_v+(new_nf-1)*SPF, _field, new_f0 + new_nf - 1, -1);
     } else {
-      n_read = _file->readField(_v+NF*SPF, _field, new_f0 + NF, new_nf - NF);
+      //kdDebug() << "Reading into _v=" << (void*)_v << " which has size " << _size << " and starting at offset " << NF*SPF << " for s=" << new_f0 + NF << " and n=" << new_nf - NF << endl;
+      assert(new_f0 + NF >= 0);
+      if (new_nf - NF > 0 || new_nf - NF == -1) {
+        n_read = _file->readField(_v+NF*SPF, _field, new_f0 + NF, new_nf - NF);
+      }
     }
   }
 
