@@ -24,12 +24,12 @@
 static int iAnonymousScalarCounter = 0;
 
 /** Create the base scalar */
-KstScalar::KstScalar(const QString& in_tag, double val, bool orphan, bool displayable, bool doLock)
-: KstObject(), _value(val), _orphan(orphan), _displayable(displayable) {
+KstScalar::KstScalar(const QString& in_tag, double val, bool orphan, bool displayable, bool doLock, bool editable)
+: KstObject(), _value(val), _orphan(orphan), _displayable(displayable), _editable(editable) {
   QString _tag = in_tag;
   if (_tag.isEmpty()) {
     QString nt = i18n("Anonymous Scalar %1");
-    
+
     do {
       _tag = nt.arg(iAnonymousScalarCounter++);
     } while (KST::vectorTagNameNotUniqueInternal(_tag));
@@ -51,11 +51,11 @@ KstScalar::KstScalar(const QString& in_tag, double val, bool orphan, bool displa
 }
 
 
-KstScalar::KstScalar(QDomElement& e)
-: KstObject(), _orphan(false), _displayable(true) {
+KstScalar::KstScalar(const QDomElement& e)
+: KstObject(), _orphan(false), _displayable(true), _editable(false) {
   QDomNode n = e.firstChild();
   bool ok;
-  
+
   while(!n.isNull()) {
     QDomElement e = n.toElement();
     if(!e.isNull()) {
@@ -65,6 +65,8 @@ KstScalar::KstScalar(QDomElement& e)
         _orphan = true;
       } else if (e.tagName() == "value") {
         setValue(e.text().toDouble());
+      } else if (e.tagName() == "editable") {
+        _editable = true;
       }
     }
     n = n.nextSibling();
@@ -83,8 +85,19 @@ KstScalar::~KstScalar() {
 
 
 KstObject::UpdateType KstScalar::update(int updateCounter) {
-  Q_UNUSED(updateCounter)
-  return NO_CHANGE;
+  bool force = dirty();
+  setDirty(false);
+
+  if (KstObject::checkUpdateCounter(updateCounter) && !force) {
+    return lastUpdateResult();
+  }
+
+  double v = value();
+  if (_provider) {
+    _provider->update(updateCounter);
+  }
+  
+  return setLastUpdateResult(v == value() ? NO_CHANGE : UPDATE);
 }
 
 
@@ -93,12 +106,16 @@ void KstScalar::save(QTextStream &ts, const QString& indent) {
   if (_orphan) {
     ts << indent << "<orphan/>" << endl;
   }
+  if (_editable) {
+    ts << indent << "<editable/>" << endl;
+  }
   ts << indent << "<value>" << value() << "</value>" << endl;
 }
 
 
 KstScalar& KstScalar::operator=(double v) {
   _value = v;
+  emit trigger();
   return *this;
 }
 
@@ -113,9 +130,55 @@ bool KstScalar::isGlobal() const {
 
 
 void KstScalar::setValue(double inV) {
+  setDirty();
   _value = inV;
   emit trigger();
 }
 
+
+QString KstScalar::label() const {
+  return QString::number(_value);
+}
+
+
+double KstScalar::value() const {
+  return _value;
+}
+
+
+bool KstScalar::orphan() const {
+  return _orphan;
+}
+
+
+void KstScalar::setOrphan(bool orphan) {
+  _orphan = orphan;
+}
+
+
+bool KstScalar::displayable() const {
+  return _displayable;
+}
+
+
+void KstScalar::setDisplayable(bool displayable) {
+  _displayable = displayable;
+}
+
+
+bool KstScalar::editable() const {
+  return _editable;
+}
+
+
+void KstScalar::setEditable(bool editable) {
+  _editable = editable;
+}
+
+
+void KstScalar::setProvider(KstObject *obj) {
+  _provider = obj;
+}
+
 #include "kstscalar.moc"
 // vim: et ts=2 sw=2
