@@ -19,44 +19,56 @@
 #define KSTSCALAR_H
 
 #include <qdom.h>
+#include <qguardedptr.h>
 #include "kstobject.h"
 
+#include "kst_export.h"
+
 /** The base class for all scalars. */
 
-class KstScalar : public KstObject {
+class KST_EXPORT KstScalar : public KstObject {
   Q_OBJECT
   Q_PROPERTY(bool orphan READ orphan WRITE setOrphan)
+  Q_PROPERTY(double value READ value WRITE setValue)
+  Q_PROPERTY(bool displayable READ displayable WRITE setDisplayable)
   public:
-    KstScalar(const QString& in_tag = QString::null, double val = 0.0, bool orphan = false, bool displayable = true, bool bLock = true);
-    KstScalar(QDomElement& e);
+    KstScalar(const QString& in_tag = QString::null, double val = 0.0, 
+              bool orphan = false, bool displayable = true, 
+              bool bLock = true, bool editable = false);
+    KstScalar(const QDomElement& e);
     virtual ~KstScalar();
 
     /* return a string representation of the scalar */
-    const QString label() const { return QString::number(_value); }
+    QString label() const;
 
-    /** Save vector information */
+    /** Save scalar information */
     virtual void save(QTextStream &ts, const QString& indent = QString::null);
 
-    /** Update the vector.  Return true if there was new data. */
+    /** Update the scalar.  Return true if there was new data. */
     virtual UpdateType update(int updateCounter = -1);
 
     KstScalar& operator=(double v);
 
     bool isGlobal() const;
 
+    // Must not be a KstObjectPtr!
+    virtual void setProvider(KstObject *obj);
+
   public slots:
-    /* return the value of the scalar */
-    double value() const { return _value; }
+    double value() const;
 
     /** Set the value of the scalar - ignored for some types */
     void setValue(double inV);
 
-    bool orphan() const { return _orphan; }
-    void setOrphan(bool orphan) { _orphan = orphan; }
-    
-    bool displayable() const { return _displayable; }
-    void setDisplayable(bool displayable) { _displayable = displayable; }
+    bool orphan() const;
+    void setOrphan(bool orphan);
 
+    bool displayable() const;
+    void setDisplayable(bool displayable);
+
+    bool editable() const;
+    void setEditable(bool editable);
+  
   signals:
     void trigger();
 
@@ -64,6 +76,11 @@ class KstScalar : public KstObject {
     double _value;
     bool _orphan;
     bool _displayable;
+    bool _editable;
+    /** Possibly null.  Be careful, this is non-standard usage of a KstShared.
+     * The purpose of this is to trigger hierarchical updates properly.
+     */
+    QGuardedPtr<KstObject> _provider;
 };
 
 typedef KstSharedPtr<KstScalar> KstScalarPtr;
