@@ -16,11 +16,12 @@
  ***************************************************************************/
 
 #include <assert.h>
+#include <stdlib.h>
 
 // include files for Qt
 
 // include files for KDE
-#include <kdebug.h>
+#include "ksdebug.h"
 #include <klocale.h>
 
 // application specific includes
@@ -28,8 +29,11 @@
 #include "kstdoc.h"
 #include "kstplotgroup.h"
 #include "kstsettings.h"
+#include "ksttimers.h"
 #include "ksttoplevelview.h"
 
+#define STICKY_THRESHOLD 10
+
 KstTopLevelView::KstTopLevelView(QWidget *parent, const char *name, WFlags w)
 : KstViewObject("KstTopLevelView"), _w(new KstViewWidget(this, parent, name, w)) {
   _onGrid = true;
@@ -38,7 +42,7 @@ KstTopLevelView::KstTopLevelView(QWidget *parent, const char *name, WFlags w)
 }
 
 
-KstTopLevelView::KstTopLevelView(QDomElement& e, QWidget *parent, const char *name, WFlags w) : KstViewObject(e), _w(new KstViewWidget(this, parent, name, w)) {
+KstTopLevelView::KstTopLevelView(const QDomElement& e, QWidget *parent, const char *name, WFlags w) : KstViewObject(e), _w(new KstViewWidget(this, parent, name, w)) {
   commonConstructor();
   loadChildren(e);
 }
@@ -49,6 +53,7 @@ void KstTopLevelView::commonConstructor() {
   setViewMode(_mode = DisplayMode);
   _pressDirection = -1;
   _moveOffset = QPoint(-1, -1);
+  _moveOffsetSticky = QPoint(0, 0);
   _backgroundColor = _w->backgroundColor();
   _mouseGrabbed = false;
 }
@@ -58,11 +63,6 @@ KstTopLevelView::~KstTopLevelView() {
 }
 
 
-KstObject::UpdateType KstTopLevelView::update() {
-  return NO_CHANGE;
-}
-
-
 void KstTopLevelView::save(QTextStream& ts, const QString& indent) {
   bool temp_onGrid;
   if (_maximized) {
@@ -80,14 +80,29 @@ void KstTopLevelView::save(QTextStream& ts, const QString& indent) {
 
 
 void KstTopLevelView::paint(KstPaintType type, QPainter& p) {
-  if (type != P_PRINT && type != P_EXPORT) {
-    updateAlignment();
-  } else {
+#ifdef BENCHMARK
+  QTime t;
+  t.start();
+#endif
+  if (type == P_PRINT || type == P_EXPORT) {
+    updateAlignment(type, p);
     p.fillRect(geometry(), QBrush(KstSettings::globalSettings()->backgroundColor, SolidPattern));
+  } else {
+    updateAlignment(type, p);
   }
+#ifdef BENCHMARK
+  int x = t.elapsed();
+  kstdDebug() << " -> Toplevelview internally took " << x << "ms" << endl;
+  kstdDebug() << "      type was " << (int)type << endl;
+  t.start();
+#endif
   QRegion clipRegion(geometry());
   p.setClipRegion(clipRegion);
   KstViewObject::paint(type, p);
+#ifdef BENCHMARK
+  x = t.elapsed();
+  kstdDebug() << " -> Parent class took " << x << "ms" << endl;
+#endif
 }
 
 
@@ -95,16 +110,18 @@ void KstTopLevelView::resized(const QSize& size) {
   KstViewObject::resize(size);
 }
 
-void KstTopLevelView::updateAlignment() {
+
+void KstTopLevelView::updateAlignment(KstPaintType type, QPainter& p) {
   QRect plotRegion;
 
   KST::alignment.reset();  
   for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
-    (*i)->internalAlignment(plotRegion);
+    (*i)->internalAlignment(type, p, plotRegion);
     KST::alignment.setPosition((*i)->geometry(), plotRegion);
   }
 }
 
+
 void KstTopLevelView::resize(const QSize& size) {
   _w->resize(size);
   KstViewObject::resize(size);
@@ -113,16 +130,16 @@ void KstTopLevelView::resize(const QSize& size) {
 
 void KstTopLevelView::paint(KstPaintType type) {
   QPainter p;
+  QRegion r(_w->geometry());
+
   p.begin(_w);
   p.setViewXForm(true);
-  QRegion r(_w->geometry());
   for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
-    r = r.subtract(QRegion((*i)->geometry()));
+    r -= QRegion((*i)->geometry());
   }
   p.setClipRegion(r);
   _lastClipRegion = r;
-  QBrush brush(_backgroundColor);
-  p.fillRect(geometry(), brush);
+  p.fillRect(geometry(), QBrush(_backgroundColor));
   p.setClipping(false);
   paint(type, p);
   p.end();
@@ -133,6 +150,7 @@ void KstTopLevelView::clearFocus() {
   if (_focusOn) {
     _pressDirection = -1;
     _moveOffset = QPoint(-1, -1);
+    _moveOffsetSticky = QPoint(0, 0);
     _w->unsetCursor();
     _focusOn = false;
     //recursively<bool>(&KstViewObject::setFocus, false);
@@ -179,12 +197,13 @@ void KstTopLevelView::updateFocus(const QPoint& pos) {
 
 
 void KstTopLevelView::setViewMode(ViewMode v) {
+  KstApp::inst()->slotUpdateDataMsg(QString::null);
   if (_mode == LayoutMode && v != LayoutMode) {
     recursively<bool>(&KstViewObject::setSelected, false);
     clearFocus();
     paint(P_PAINT);
   } else if (_mode == DisplayMode && v != DisplayMode) {
-    recursively<bool>(&KstViewObject::zoom, false);
+    recursively<bool>(&KstViewObject::setMaximized, false);
   }
 
   _mode = v;
@@ -255,7 +274,7 @@ void KstTopLevelView::restartMove() {
 
 
 bool KstTopLevelView::handlePress(const QPoint& pos, bool shift) {
-  //kdDebug() << "HANDLE PRESS" << endl;
+  //kstdDebug() << "HANDLE PRESS" << endl;
   _pressDirection = -1;
 
   if (_mode != LayoutMode) {
@@ -265,6 +284,7 @@ bool KstTopLevelView::handlePress(const QPoint& pos, bool shift) {
 
   _prevBand = QRect(-1, -1, 0, 0);
   _moveOffset = QPoint(-1, -1);
+  _moveOffsetSticky = QPoint(0, 0);
 
   if (!_focusOn) {
     _pressTarget = 0L;
@@ -294,6 +314,7 @@ bool KstTopLevelView::handlePress(const QPoint& pos, bool shift) {
     _pressTarget = 0L;
     _pressDirection = -1;
     _moveOffset = QPoint(-1, -1);
+    _moveOffsetSticky = QPoint(0, 0);
     updateFocus(pos);
     paint(P_PAINT);
 
@@ -320,100 +341,88 @@ bool KstTopLevelView::handlePress(const QPoint& pos, bool shift) {
 }
 
 
-static QRect newSize(const QRect& oldSize, int direction, const QPoint& pos) {
-  QRect g = oldSize;
-  QPoint save, tmp;
-  switch (direction) {
+QRect KstTopLevelView::newSize(const QRect& oldSize, int direction, const QPoint& pos) {
+  QRect r = oldSize;
+  QPoint save;
+  
+  switch (direction & (UP|DOWN)) {
     case UP:
-      save = g.bottomRight();
-      if (g.bottom() < pos.y()) {
-        g.moveTop(g.bottom());
-      } else {
-        g.moveTop(pos.y());
-      }
-      g.setBottomRight(save);
+      r.setTop(pos.y());
       break;
     case DOWN:
-      save = g.topLeft();
-      if (g.top() > pos.y()) {
-        g.moveBottom(g.top());
-      } else {
-        g.moveBottom(pos.y());
-      }
-      g.setTopLeft(save);
-      break;
-    case LEFT:
-      save = g.bottomRight();
-      if (g.right() < pos.x()) {
-        g.moveLeft(g.right());
-      } else {
-        g.moveLeft(pos.x());
-      }
-      g.setBottomRight(save);
-      break;
-    case RIGHT:
-      save = g.topLeft();
-      if (g.left() > pos.x()) {
-        g.moveRight(g.left());
-      } else {
-        g.moveRight(pos.x());
-      }
-      g.setTopLeft(save);
-      break;
-    case UP|LEFT:
-      save = g.bottomRight();
-      tmp = pos;
-      if (g.right() < tmp.x()) {
-        tmp.setX(g.right());
-      }
-      if (g.bottom() < tmp.y()) {
-        tmp.setY(g.bottom());
-      }
-      g.moveTopLeft(tmp);
-      g.setBottomRight(save);
-      break;
-    case DOWN|RIGHT:
-      save = g.topLeft();
-      tmp = pos;
-      if (g.left() > tmp.x()) {
-        tmp.setX(g.left());
-      }
-      if (g.top() > tmp.y()) {
-        tmp.setY(g.top());
-      }
-      g.moveBottomRight(tmp);
-      g.setTopLeft(save);
-      break;
-    case UP|RIGHT:
-      save = g.bottomLeft();
-      tmp = pos;
-      if (g.left() > tmp.x()) {
-        tmp.setX(g.left());
-      }
-      if (g.bottom() < tmp.y()) {
-        tmp.setY(g.bottom());
-      }
-      g.moveTopRight(tmp);
-      g.setBottomLeft(save);
-      break;
-    case DOWN|LEFT:
-      save = g.topRight();
-      tmp = pos;
-      if (g.right() < tmp.x()) {
-        tmp.setX(g.right());
-      }
-      if (g.top() > tmp.y()) {
-        tmp.setY(g.top());
-      }
-      g.moveBottomLeft(tmp);
-      g.setTopRight(save);
+      r.setBottom(pos.y());
       break;
     default:
-      //abort();
       break;
   }
+  
+  switch (direction & (LEFT|RIGHT)) {  
+    case LEFT:
+      r.setLeft(pos.x());
+      break;
+    case RIGHT:
+      r.setRight(pos.x());
+      break;
+    default:
+      break;
+  }
+  
+  {
+    int iXMin = STICKY_THRESHOLD;
+    int iYMin = STICKY_THRESHOLD;
 
-  return g;
+    // check for "sticky" borders
+    for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
+      if (_pressTarget != *i) {           
+        if (direction & LEFT) {
+          if (labs(r.left() - (*i)->geometry().left()) < labs(iXMin)) {
+            iXMin = r.left() - (*i)->geometry().left();
+          } else if (labs(r.left() - (*i)->geometry().right()) < labs(iXMin)) {
+            iXMin = r.left() - (*i)->geometry().right();              
+          }
+        } else if (direction & RIGHT) {
+          if (labs(r.right() - (*i)->geometry().left()) < labs(iXMin)) {
+            iXMin = r.right() - (*i)->geometry().left();
+          } else if (labs(r.right() - (*i)->geometry().right()) < labs(iXMin)) {
+            iXMin = r.right() - (*i)->geometry().right();
+          }
+        }                 
+        
+        if (direction & UP) {
+          if (labs(r.top() - (*i)->geometry().top()) < labs(iYMin)) {
+            iYMin = r.top() - (*i)->geometry().top();
+          } else if (labs(r.top() - (*i)->geometry().bottom()) < labs(iYMin)) {
+            iYMin = r.top() - (*i)->geometry().bottom();              
+          }
+        } else if (direction & DOWN) {
+          if (labs(r.bottom() - (*i)->geometry().top()) < labs(iYMin)) {
+            iYMin = r.bottom() - (*i)->geometry().top();
+          } else if (labs(r.bottom() - (*i)->geometry().bottom()) < labs(iYMin)) {
+            iYMin = r.bottom() - (*i)->geometry().bottom();
+          }
+        }
+      }
+    }
+  
+    if (labs(iXMin) < STICKY_THRESHOLD) {
+      if (direction & LEFT) {
+        r.setLeft(r.left() - iXMin);
+      } else if (direction & RIGHT) {
+        r.setRight(r.right() - iXMin);
+      }
+    }
+    if (labs(iYMin) < STICKY_THRESHOLD) {
+      if (direction & UP) {
+        r.setTop(r.top() - iYMin);
+      } else if (direction & DOWN) {
+        r.setBottom(r.bottom() - iYMin);
+      }
+    }    
+  }
+  
+  r = r.normalize();
+  
+  return r;
 }
 
 
@@ -453,21 +462,68 @@ void KstTopLevelView::pressMove(const QPoint& pos, bool shift) {
 
   if (_pressTarget) {
     if (_pressDirection == 0) {
+      // moving a plot.
       QRect old = _prevBand;
+      QPoint topLeft;
       QRect r;
+      
       r = _pressTarget->geometry();
       if (!_selectionList.isEmpty()) {
         for (KstViewObjectList::Iterator i = _selectionList.begin(); i != _selectionList.end(); ++i) {
           r = r.unite((*i)->geometry());
         }
+      }      
+      topLeft = pos - _moveOffset - _pressTarget->geometry().topLeft() + r.topLeft();
+      r.moveTopLeft(topLeft);
+      _moveOffsetSticky = QPoint(0, 0);
+      
+      {
+        int iXMin = STICKY_THRESHOLD;
+        int iYMin = STICKY_THRESHOLD;
+      
+        // check for "sticky" borders
+        for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
+          if (_selectionList.find(*i) == _selectionList.end() && _pressTarget != *i) {           
+            if (labs(r.left() - (*i)->geometry().left()) < labs(iXMin)) {
+              iXMin = r.left() - (*i)->geometry().left();
+            } else if (labs(r.left() - (*i)->geometry().right()) < labs(iXMin)) {
+              iXMin = r.left() - (*i)->geometry().right();              
+            } else if (labs(r.right() - (*i)->geometry().left()) < labs(iXMin)) {
+              iXMin = r.right() - (*i)->geometry().left();
+            } else if (labs(r.right() - (*i)->geometry().right()) < labs(iXMin)) {
+              iXMin = r.right() - (*i)->geometry().right();
+            }                 
+            
+            if (labs(r.top() - (*i)->geometry().top()) < labs(iYMin)) {
+              iYMin = r.top() - (*i)->geometry().top();
+            } else if (labs(r.top() - (*i)->geometry().bottom()) < labs(iYMin)) {
+              iYMin = r.top() - (*i)->geometry().bottom();              
+            } else if (labs(r.bottom() - (*i)->geometry().top()) < labs(iYMin)) {
+              iYMin = r.bottom() - (*i)->geometry().top();
+            } else if (labs(r.bottom() - (*i)->geometry().bottom()) < labs(iYMin)) {
+              iYMin = r.bottom() - (*i)->geometry().bottom();
+            }
+          }
+        }
+      
+        if (labs(iXMin) < STICKY_THRESHOLD) {
+          _moveOffsetSticky.setX(iXMin);
+          topLeft.setX(topLeft.x() - iXMin);
+        }
+        if (labs(iYMin) < STICKY_THRESHOLD) {
+          _moveOffsetSticky.setY(iYMin);
+          topLeft.setY(topLeft.y() - iYMin);
+        }
+        r.moveTopLeft(topLeft);      
       }
-      r.moveTopLeft(pos - _moveOffset - _pressTarget->geometry().topLeft() + r.topLeft());
+      
       if (!_geom.contains(r, true)) {
         slideInto(_geom, r);
       }
       _prevBand = r;
       if (_prevBand != old) {
         QPainter p;
+        
         p.begin(_w);
         p.setRasterOp(Qt::NotROP);
         p.drawWinFocusRect(old);
@@ -475,19 +531,28 @@ void KstTopLevelView::pressMove(const QPoint& pos, bool shift) {
         p.end();
       }
     } else {
+      // resizing a plot.
       QRect old = _prevBand;
-      QRect r = newSize(_pressTarget->geometry(), _pressDirection, pos);
-      _prevBand = r.normalize().intersect(_geom);
+      
+      _prevBand = newSize(_pressTarget->geometry(), _pressDirection, pos);
+      _prevBand = _prevBand.intersect(_geom);
       if (_prevBand != old) {
         QPainter p;
+        
         p.begin(_w);
-        p.setRasterOp(Qt::NotROP);
-        p.drawWinFocusRect(old);
+        p.setRasterOp(Qt::XorROP);
+        if (old.topLeft() != QPoint(-1, -1)) {
+          p.drawWinFocusRect(old);
+        } else {     
+          p.drawWinFocusRect(_pressTarget->geometry());
+        }
         p.drawWinFocusRect(_prevBand);
         p.end();
       }
     }
+    KstApp::inst()->slotUpdateDataMsg(i18n("(x0,y0)-(x1,y1)", "(%1,%2)-(%3,%4)").arg(_prevBand.topLeft().x()).arg(_prevBand.topLeft().y()).arg(_prevBand.bottomRight().x()).arg(_prevBand.bottomRight().y()));
   } else {
+    // selecting plots.
     QRect old = _prevBand;
     QRect r;
     r.setTopLeft(_moveOffset);
@@ -501,16 +566,19 @@ void KstTopLevelView::pressMove(const QPoint& pos, bool shift) {
       p.drawWinFocusRect(_prevBand);
       p.end();
     }
+    KstApp::inst()->slotUpdateDataMsg(QString::null);
   }
 }
 
 
-void KstTopLevelView::releasePress(const QPoint& pos, bool shift) {
+void KstTopLevelView::releasePress(const QPoint& pos, bool shift) { 
   if (_mode != LayoutMode) {
     _pressTarget = 0L;
     return;
   }
 
+  KstApp::inst()->slotUpdateDataMsg(QString::null);
+
   if (_pressDirection == -1 && _pressTarget) { // menu released
     _pressTarget = 0L;
     return;
@@ -528,7 +596,8 @@ void KstTopLevelView::releasePress(const QPoint& pos, bool shift) {
         }
       }
       QPoint objOffset = _pressTarget->geometry().topLeft() - obj.topLeft();
-      obj.moveTopLeft(pos - _moveOffset - _pressTarget->geometry().topLeft() + obj.topLeft());
+      obj.moveTopLeft(pos - _moveOffset - _moveOffsetSticky - 
+                      _pressTarget->geometry().topLeft() + obj.topLeft());
       if (!_geom.contains(obj, true)) {
         slideInto(_geom, obj);
       }
@@ -541,13 +610,15 @@ void KstTopLevelView::releasePress(const QPoint& pos, bool shift) {
       _onGrid = false;
     } else {
       QRect r = newSize(_pressTarget->geometry(), _pressDirection, pos).intersect(_geom);
+        
       _pressTarget->move(r.topLeft());
       _pressTarget->resize(r.size());
       _onGrid = false;
     }
-    _pressTarget->setFocus(true);
+    _pressTarget->setFocus(true);   
   } else { // rubber band
     QPainter p;
+    
     p.begin(_w);
     p.setRasterOp(Qt::NotROP);
     p.drawWinFocusRect(_prevBand);
@@ -567,7 +638,10 @@ void KstTopLevelView::releasePress(const QPoint& pos, bool shift) {
   _pressTarget = 0L;
   _pressDirection = -1;
   _moveOffset = QPoint(-1, -1);
+  _moveOffsetSticky = QPoint(0, 0);
+  
   updateFocus(pos);
+  
   paint(P_PAINT);
 }
 
@@ -630,7 +704,7 @@ bool KstTopLevelView::popupMenu(KPopupMenu *menu, const QPoint& pos) {
       subMenu->insertItem(i18n("Top"), this, SLOT(alignTop()));
       subMenu->insertItem(i18n("Bottom"), this, SLOT(alignBottom()));
       menu->insertItem(i18n("Align"), subMenu);
-      
+
       subMenu = new KPopupMenu(menu);
       subMenu->insertItem(i18n("Horizontally"), this, SLOT(packHorizontally()));
       subMenu->insertItem(i18n("Vertically"), this, SLOT(packVertically()));
@@ -650,12 +724,10 @@ bool KstTopLevelView::popupMenu(KPopupMenu *menu, const QPoint& pos) {
     rc = true;
   }
 
-  //
   // don't release the _pressTarget reference here as we are not yet done with
   //  it, since several of the menu handlers require it in order to function...
   // _pressTarget = 0L; // release reference
-  //
-  
+
   return rc;
 }
 
@@ -835,7 +907,7 @@ void KstTopLevelView::packHorizontally() {
   int iLeft = 0;
   int iRight = 0;
   int iCount = 0;
-  
+
   for (KstViewObjectList::Iterator i = _selectionList.begin(); i != _selectionList.end(); ++i) {
     if (i == _selectionList.begin()) {
       iLeft = (*i)->geometry().x();
@@ -859,7 +931,7 @@ void KstTopLevelView::packHorizontally() {
     int iPosMin = 0;
     int iPos = iLeft;
     int iWidth = (iRight - iLeft) / iCount;
-    
+
     while (!selectionList.empty()) {
       for (KstViewObjectList::Iterator i = selectionList.begin(); i != selectionList.end(); ++i) {
         if (i == selectionList.begin()) {
