@@ -28,6 +28,7 @@ class KstVCurve: public KstBaseCurve {
   public:
     KstVCurve(const QString &in_tag, KstVectorPtr in_X, KstVectorPtr in_Y,
         KstVectorPtr in_EX, KstVectorPtr in_EY,
+        KstVectorPtr in_EXMinus, KstVectorPtr in_EYMinus,
         const QColor &in_color);
     KstVCurve(QDomElement &e);
     virtual ~KstVCurve();
@@ -37,20 +38,30 @@ class KstVCurve: public KstBaseCurve {
 
     virtual bool hasXError() const;
     virtual bool hasYError() const;
+    virtual bool hasXMinusError() const;
+    virtual bool hasYMinusError() const;
 
     virtual void point(int i, double &x1, double &y1);
     virtual void getEXPoint(int i, double &x1, double &y1, double &ex1);
     virtual void getEYPoint(int i, double &x1, double &y1, double &ey1);
+    virtual void getEXMinusPoint(int i, double &x1, double &y1, double &ex1);
+    virtual void getEYMinusPoint(int i, double &x1, double &y1, double &ey1);
+    virtual void getEXPoints(int i, double &x, double &y, double &ex, double &exminus);
+    virtual void getEYPoints(int i, double &x, double &y, double &ey, double &eyminus);
 
     QString xVTag() const;
     QString yVTag() const;
     QString xETag() const;
     QString yETag() const;
+    QString xEMinusTag() const;
+    QString yEMinusTag() const;
 
     void setXVector(KstVectorPtr new_vx);
     void setYVector(KstVectorPtr new_vy);
     void setXError(KstVectorPtr new_ex);
     void setYError(KstVectorPtr new_ey);
+    void setXMinusError(KstVectorPtr new_ex);
+    void setYMinusError(KstVectorPtr new_ey);
 
     /** Save curve information */
     void save(QTextStream &ts, const QString& indent = QString::null);
@@ -76,6 +87,13 @@ class KstVCurve: public KstBaseCurve {
 
     virtual bool uses(KstObjectPtr p) const;
 
+    KstVectorPtr xVector() const;
+    KstVectorPtr yVector() const;
+    KstVectorPtr xErrorVector() const;
+    KstVectorPtr yErrorVector() const;
+    KstVectorPtr xMinusErrorVector() const;
+    KstVectorPtr yMinusErrorVector() const;
+
   private:
     inline void commonConstructor(const QString &in_tag,
         const QColor &in_color);
@@ -84,6 +102,8 @@ class KstVCurve: public KstBaseCurve {
     KstVectorPtr VY;
     KstVectorPtr EX;
     KstVectorPtr EY;
+    KstVectorPtr EXMinus;
+    KstVectorPtr EYMinus;
 
     double _ns_maxx;
     double _ns_minx;
