@@ -19,21 +19,14 @@
 #include <qlabel.h>
 #include <qlayout.h>
 #include <qlistbox.h>
-#include <qmessagebox.h>
 #include <qpushbutton.h>
-#include <qstring.h>
 #include <qtable.h>
 #include <qtimer.h>
-#include <qwidget.h>
 
 #include <kdebug.h>
 #include <klocale.h>
 
-#include "vectorselector.h"
-#include "kstfittable.h"
 #include "kstplugin.h"
-#include "kstrvector.h"
-#include "kstscalar.h"
 #include "kstdatacollection.h"
 #include "kstviewfitsdialog_i.h"
 
@@ -42,7 +35,7 @@ KstViewFitsDialogI::KstViewFitsDialogI(QWidget* parent,
                                              bool modal,
                                              WFlags fl) 
 : KstViewFitsDialog(parent, name, modal, fl) {
-  
+
   tableFits = new KstFitTable(this, "tableFits");
   tableFits->setNumRows(0);
   tableFits->setNumCols(1);
@@ -50,10 +43,10 @@ KstViewFitsDialogI::KstViewFitsDialogI(QWidget* parent,
   tableFits->setSorting(false);
   tableFits->setSelectionMode(QTable::Single);
   layout2->addWidget(tableFits, 2, 0);
-    
+
   connect(Cancel, SIGNAL(clicked()), this, SLOT(close()));
   connect(_comboBoxFits, SIGNAL(activated(const QString&)), this, SLOT(fitChanged(const QString&)));
-    
+
   tableFits->setReadOnly(true);
 }
 
@@ -102,7 +95,7 @@ void KstViewFitsDialogI::showViewFitsDialog(const QString& fit) {
 void KstViewFitsDialogI::showViewFitsDialog() {
   updateViewFitsDialog();
   updateDefaults(0);
-  
+
   show();
   raise();
 }
@@ -178,7 +171,7 @@ void KstViewFitsDialogI::fitChanged(const QString& strFit) {
       plugin->readUnlock();
     }
   }
-  
+
   tableFits->update();
 }
 
