@@ -20,17 +20,20 @@
 #include <stdlib.h>
 
 // include files for Qt
+#include <qdeepcopy.h>
 #include <qstylesheet.h>
 
 // include files for KDE
 #include <kdatastream.h>
-#include <kdebug.h>
+#include "ksdebug.h"
 
 // application specific includes
 #include "kst.h"
 #include "kstdoc.h"
+#include "kstobject.h"
 #include "kstplotgroup.h"
 #include "kstsettings.h"
+#include "ksttimers.h"
 #include "kstviewobject.h"
 #include "kstviewobjectfactory.h"
 #include "kstviewwindow.h"
@@ -41,24 +44,49 @@ KstViewObject::KstViewObject(const QString& type) : KstObject(), _geom(0, 0, 1,
   _layoutActions = 0;
   _maximized = false;
   updateAspect();
-  _onGrid = true;
+  _onGrid = false;
   _columns = 0;
   _focus = false;
   _selected = false;
   _foregroundColor = KstSettings::globalSettings()->foregroundColor;
   _backgroundColor = KstSettings::globalSettings()->backgroundColor;
+  setMinimumSize(QSize(1, 1));
 }
 
 
-KstViewObject::KstViewObject(QDomElement& e) : KstObject() {
+KstViewObject::KstViewObject(const QDomElement& e) : KstObject() {
   _foregroundColor = KstSettings::globalSettings()->foregroundColor;
   _backgroundColor = KstSettings::globalSettings()->backgroundColor;
   _parent = 0L;
+  setMinimumSize(QSize(1, 1));
   load(e);
 }
 
 
-void KstViewObject::load(QDomElement& e) {
+KstViewObject::KstViewObject(const KstViewObject& viewObject) : KstObject() {
+  _parent = 0L;
+  _foregroundColor = viewObject._foregroundColor;
+  _backgroundColor = viewObject._backgroundColor;
+  _aspect = viewObject._aspect;
+  _standardActions = viewObject._standardActions;
+  _layoutActions = viewObject._layoutActions;
+  _maximized = false;
+  _onGrid = viewObject._onGrid;
+  _columns = viewObject._columns;
+  _focus = false;
+  _selected = false;
+  _geom = viewObject._geom;
+  setMinimumSize(QSize(1, 1));
+
+  setContentsRect(viewObject.contentsRect());
+
+  for (KstViewObjectList::ConstIterator i = viewObject._children.begin(); i != viewObject._children.end(); ++i) {
+    (*i)->copyObjectQuietly(*this);
+  }
+}
+
+
+void KstViewObject::load(const QDomElement& e) {
   _children.clear();
   _standardActions = 0;
   _layoutActions = 0;
@@ -96,7 +124,7 @@ KstViewObject::~KstViewObject() {
 }
 
 
-void KstViewObject::loadChildren(QDomElement& e) {
+void KstViewObject::loadChildren(const QDomElement& e) {
   QDomNode n = e.firstChild();
 
   while (!n.isNull()) {
@@ -123,8 +151,21 @@ void KstViewObject::loadChildren(QDomElement& e) {
 }
 
 
-KstObject::UpdateType KstViewObject::update(int) {
-  return NO_CHANGE;
+KstObject::UpdateType KstViewObject::update(int counter) {
+  if (checkUpdateCounter(counter)) {
+    return lastUpdateResult();
+  }
+
+  KstObject::UpdateType rc = NO_CHANGE;
+  for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
+    if (rc == NO_CHANGE) {
+      rc = (*i)->update(counter);
+    } else {
+      (*i)->update(counter);
+    }
+  }
+
+  return setLastUpdateResult(rc);
 }
 
 
@@ -168,34 +209,13 @@ void KstViewObject::saveTag(QTextStream& ts, const QString& indent) {
   }
 }
 
-void KstViewObject::readLock() const {
-  abort();
-}
-
-
-void KstViewObject::readUnlock() const {
-  abort();
-}
-
-
-void KstViewObject::writeLock() const {
-  abort();
-}
-
-
-void KstViewObject::writeUnlock() const {
-  abort();
-}
-
 
 void KstViewObject::paint(KstPaintType type, QPainter& p) {
   bool hadClipping = p.hasClipping();
   QRegion oldRegion = p.clipRegion();
   bool maximized = false;
 
-  //
   // handle the case where we have maximized plots
-  //
   for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
     if ((*i)->_maximized) {
       (*i)->_lastClipRegion = (*i)->geometry();
@@ -207,8 +227,10 @@ void KstViewObject::paint(KstPaintType type, QPainter& p) {
 
   if (!maximized) {
     for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
-      // Compute the clip reg - this can't be stored if we want to keep the code
-      // generic, but it's slow, so it's a bit of a conundrum
+#ifdef BENCHMARK
+      QTime t;
+      t.start();
+#endif
       QRegion clipRegion = oldRegion;
       KstViewObjectList::Iterator j = i;
       for (++j; j != _children.end(); ++j) {
@@ -217,12 +239,16 @@ void KstViewObject::paint(KstPaintType type, QPainter& p) {
       (*i)->_lastClipRegion = clipRegion;
       p.setClipRegion(clipRegion);
       (*i)->paint(type, p);
+#ifdef BENCHMARK
+      int x = t.elapsed();
+      kstdDebug() << "   -> object " << (*i)->tagName() << " took " << x << "ms" << endl;
+#endif
       if ((*i)->_maximized) {
         break;
       }
     }
   }
-  
+
   p.setClipRegion(oldRegion);
 
   if (_focus) {
@@ -273,6 +299,13 @@ void KstViewObject::drawSelectRect(QPainter& p) {
 void KstViewObject::appendChild(KstViewObjectPtr obj, bool keepAspect) {
   obj->_parent = this;
   _children.append(obj);
+
+  for (KstViewObjectList::Iterator i = children().begin(); i != children().end(); ++i) {
+    if ((*i)->maximized()) {
+      (*i)->setMaximized(false);
+    }
+  }
+
   if (keepAspect) {
     obj->updateFromAspect();
   } else {
@@ -284,6 +317,13 @@ void KstViewObject::appendChild(KstViewObjectPtr obj, bool keepAspect) {
 void KstViewObject::prependChild(KstViewObjectPtr obj, bool keepAspect) {
   obj->_parent = this;
   _children.prepend(obj);
+
+  for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
+    if ((*i)->maximized()) {
+      (*i)->setMaximized(false);
+    }
+  }
+
   if (keepAspect) {
     obj->updateFromAspect();
   } else {
@@ -313,6 +353,13 @@ void KstViewObject::insertChildAfter(const KstViewObjectPtr after, KstViewObject
     _children.prepend(obj);
   }
   obj->_parent = this;
+
+  for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
+    if ((*i)->maximized()) {
+      (*i)->setMaximized(false);
+    }
+  }
+
   if (keepAspect) {
     obj->updateFromAspect();
   } else {
@@ -340,9 +387,8 @@ KstViewObjectList& KstViewObject::children() {
 
 
 void KstViewObject::resize(const QSize& size) {
-  assert(size.width() > 0);
-  assert(size.height() > 0);
-  _geom.setSize(size);
+  setMinimumSize(minimumSize().expandedTo(QSize(_children.count(), _children.count())));
+  _geom.setSize(size.expandedTo(_minimumSize));
   updateAspect();
   for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
     (*i)->parentResized();
@@ -381,7 +427,10 @@ QSize KstViewObject::size() const {
 }
 
 
-void KstViewObject::internalAlignment(QRect& plotRegion) {
+void KstViewObject::internalAlignment(KstPaintType type, QPainter& p, QRect& plotRegion) {
+  Q_UNUSED(type)
+  Q_UNUSED(p)
+
   plotRegion.setLeft(0);
   plotRegion.setRight(0);
   plotRegion.setTop(0);
@@ -459,6 +508,8 @@ void KstViewObject::cleanup(int cols) {
     return;
   }
 
+  // FIXME: don't allow regrid to a number of columns that will result in
+  //        >= height() plots in a column
   if (!_onGrid) {
     if (cols <= 0) {
       cols = int(sqrt(_children.count()));
@@ -473,6 +524,7 @@ void KstViewObject::cleanup(int cols) {
     }
   }
 
+  double minDistance = 0.0;
   int pos = 0;
   int x = 0;
   int y = 0;
@@ -480,23 +532,41 @@ void KstViewObject::cleanup(int cols) {
   int lastRow = _children.count() / _columns;
   int h = _geom.height() / (lastRow + (_children.count() % _columns > 0 ? 1 : 0));
 
+  KstViewObjectList childrenCopy = QDeepCopy<KstViewObjectList>(_children);
+
   //kdDebug() << "cleanup with w=" << w << " and h=" << h << endl;
   //kdDebug() << "columns=" << _columns  << endl;
-  for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
-    QSize sz(w, h);
+  for (unsigned i = 0; i < _children.count(); ++i) {
+    KstViewObjectList::Iterator nearest = childrenCopy.end();
     QPoint pt(x, y);
-    // Adjust the last column to be sure that we don't spill over
+    QSize sz(w, h);
+
+    // adjust the last column to be sure that we don't spill over
     if (pos % _columns == _columns - 1) {
       sz.setWidth(_geom.width() - x);
     }
 
-    // Adjust the last row to be sure that we don't spill over
+    // adjust the last row to be sure that we don't spill over
     if ((pos + 1) / _columns > lastRow) {
       sz.setHeight(_geom.height() - y);
     }
 
-    (*i)->move(pt);
-    (*i)->resize(sz);
+    for (KstViewObjectList::Iterator it = childrenCopy.begin(); it != childrenCopy.end(); ++it) {
+      // find plot closest to the desired position, based on top-left corner...
+      double distance = double((x - (*it)->geometry().x()) * (x - (*it)->geometry().x())) + 
+                        double((y - (*it)->geometry().y()) * (y - (*it)->geometry().y()));
+      if (it == childrenCopy.begin() || distance < minDistance) {
+        minDistance = distance;
+        nearest = it;
+      }
+    }
+
+    if (nearest != childrenCopy.end()) {
+      KstViewObjectPtr vop = *nearest;
+      vop->move(pt);
+      vop->resize(sz);
+      childrenCopy.remove(vop);
+    }
 
     if (++pos % _columns == 0) {
       x = 0;
@@ -524,7 +594,7 @@ void KstViewObject::parentMoved(const QPoint& offset) {
 void KstViewObject::move(const QPoint& pos) {
   QPoint offset = pos - _geom.topLeft();
 
-  if( !offset.isNull() ) {
+  if (!offset.isNull()) {
     _geom.moveTopLeft(pos);
     updateAspectPos( );
     for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
@@ -616,19 +686,30 @@ QRect KstViewObject::contentsRect() const {
 }
 
 
-void KstViewObject::setContentsRect(QRect& rect) {
+void KstViewObject::setContentsRect(const QRect& rect) {
   _geom = rect;
 }
 
 
+QString KstViewObject::menuTitle() const {
+  return tagName();
+}
+
+
 bool KstViewObject::popupMenu(KPopupMenu *menu, const QPoint& pos, KstViewObjectPtr topParent) {
   Q_UNUSED(pos)
   Q_UNUSED(topParent)
   bool rc = false;
   int id;
+  QString menuTitle = this->menuTitle();
 
-  if (!tagName().isEmpty()) {
-    menu->insertTitle(tagName());
+  if (!menuTitle.isEmpty()) {
+    menu->insertTitle(menuTitle);
+  }
+
+  if (_standardActions & Edit) {
+    menu->insertItem(i18n("&Edit..."), this, SLOT(edit()));
+    rc = true;
   }
 
   if (_standardActions & Delete) {
@@ -636,8 +717,8 @@ bool KstViewObject::popupMenu(KPopupMenu *menu, const QPoint& pos, KstViewObject
     rc = true;
   }
 
-  if (_standardActions & Edit) {
-    menu->insertItem(i18n("&Edit..."), this, SLOT(edit()));
+  if (_standardActions & Copy) {
+    menu->insertItem(i18n("&Copy"), this, SLOT(copyObject()));
     rc = true;
   }
 
@@ -678,6 +759,11 @@ bool KstViewObject::layoutPopupMenu(KPopupMenu *menu, const QPoint& pos, KstView
     rc = true;
   }
 
+  if (_layoutActions & Copy) {
+    menu->insertItem(i18n("&Copy"), this, SLOT(copyObject()));
+    rc = true;
+  }
+
   if (_layoutActions & Raise) {
     menu->insertItem(i18n("&Raise"), this, SLOT(raise()));
     rc = true;
@@ -737,6 +823,39 @@ bool KstViewObject::layoutPopupMenu(KPopupMenu *menu, const QPoint& pos, KstView
     rc = true;
   }
 
+  if (_layoutActions & CopyTo) {
+    int i = 0;
+    bool hasEntry = false;
+
+    KPopupMenu *submenu = new KPopupMenu(menu);
+
+    id = menu->insertItem(i18n("&Copy To"), submenu);
+
+    KMdiIterator<KMdiChildView*> *it = KstApp::inst()->createIterator();
+    while (it->currentItem()) {
+      KstViewWindow *c = dynamic_cast<KstViewWindow*>(it->currentItem());
+      KstTopLevelViewPtr tlv = kst_cast<KstTopLevelView>(topParent);
+
+      if (c) {
+        hasEntry = true;
+        if (tlv && tlv == c->view()) {
+          submenu->insertItem(i18n("%1 (here)").arg(it->currentItem()->caption()), i);
+        } else {
+          submenu->insertItem(it->currentItem()->caption(), i);
+        }
+        submenu->connectItem(i, this, SLOT(copyTo(int)));
+        _copyToMap[i] = it->currentItem()->caption();
+        i++;
+      }
+      it->next();
+    }
+    KstApp::inst()->deleteIterator(it);
+
+    menu->setItemEnabled(id, hasEntry);
+
+    rc = true;
+  }
+    
   return rc;
 }
 
@@ -757,6 +876,16 @@ void KstViewObject::deleteObject() {
 }
 
 
+void KstViewObject::copyObject() {
+}
+
+
+void KstViewObject::copyObjectQuietly(KstViewObject &parent, const QString& name) const {
+  Q_UNUSED(parent)
+  Q_UNUSED(name)
+}
+
+
 void KstViewObject::raiseToTop() {
   // FIXME: eliminate _parent!!
   KstApp::inst()->document()->setModified();
@@ -856,14 +985,37 @@ void KstViewObject::moveTo(int id) {
 }
 
 
+void KstViewObject::copyTo(int id) {
+  KstApp::inst()->document()->setModified();
+  QString windowName = _copyToMap[id];
+  KMdiChildView *mdiChild;
+  KstViewWindow *window;
+
+  if (!windowName.isEmpty()) {
+    mdiChild = KstApp::inst()->findWindow(windowName);
+    if (mdiChild) {
+      window = dynamic_cast<KstViewWindow*>(mdiChild);
+      if (window) {
+        copyObjectQuietly(*(window->view().data()));
+      }
+    }
+  }
+}
+
+
 void KstViewObject::updateFromAspect() {
   // FIXME: eliminate _parent!!
+  setMinimumSize(minimumSize().expandedTo(QSize(_children.count(), _children.count())));
   if (_parent) {
     _geom.setLeft(_parent->geometry().left() + int(_aspect.x * _parent->geometry().width()));
     _geom.setTop(_parent->geometry().top() + int(_aspect.y * _parent->geometry().height()));
-    _geom.setRight(_parent->geometry().left() + int((_aspect.x + _aspect.w) * _parent->geometry().width()));
-    _geom.setBottom(_parent->geometry().top() + int((_aspect.y + _aspect.h) * _parent->geometry().height()));
+    _geom.setRight(_parent->geometry().left() + int((_aspect.x + _aspect.w) * _parent->geometry().width()) - 1);
+    _geom.setBottom(_parent->geometry().top() + int((_aspect.y + _aspect.h) * _parent->geometry().height()) - 1);
   }
+  if (_geom.width() >= _minimumSize.width() || _geom.height() >= _minimumSize.height()) {
+    _geom.setSize(_geom.size().expandedTo(_minimumSize));
+  }
+  assert(_geom.left() >= 0 && _geom.top() >= 0);
 }
 
 
@@ -912,8 +1064,13 @@ void KstViewObject::setSelected(bool selected) {
 }
 
 
-void KstViewObject::zoom(bool zoom) {
-  if (_maximized != zoom) {
+bool KstViewObject::maximized() const {
+  return _maximized;
+}
+
+
+void KstViewObject::setMaximized(bool maximized) {
+  if (_maximized != maximized) {
     zoomToggle();
   }
 }
@@ -969,7 +1126,11 @@ void KstViewObject::detach() {
 
 void KstViewObject::rename() {
   bool ok = false;
+#if KDE_VERSION >= KDE_MAKE_VERSION(3,3,0)
+  QString newName = KInputDialog::getText(i18n("Kst"), i18n("Enter a new name for %1:").arg(tagName()), tagName(), &ok);
+#else
   QString newName = KLineEditDlg::getText(i18n("Enter a new name for %1:").arg(tagName()), tagName(), &ok, 0L);
+#endif
   if (ok) {
     setTagName(newName);
   }
@@ -1016,7 +1177,7 @@ void KstViewObject::readBinary(QDataStream& str) {
   QString tagName;
   str >> tagName;
   setTagName(tagName);
-  kdDebug() << "Decoding " << tagName << " from drag." << endl;
+  kstdDebug() << "Decoding " << tagName << " from drag." << endl;
   // FIXME: rename objects if they cause a namespace conflict
   str >> _geom >> _backgroundColor >> _foregroundColor;
   str >> _standardActions >> _layoutActions >> _aspect;
@@ -1071,6 +1232,12 @@ void KstViewObject::mousePressEvent(QWidget *view, QMouseEvent *e) {
 }
 
 
+void KstViewObject::mouseDoubleClickEvent(QWidget *view, QMouseEvent *e) {
+  Q_UNUSED(view)
+  Q_UNUSED(e)
+}
+
+
 void KstViewObject::mouseReleaseEvent(QWidget *view, QMouseEvent *e) {
   Q_UNUSED(view)
   Q_UNUSED(e)
@@ -1129,15 +1296,27 @@ void KstViewObject::unselectAll() {
 
 bool KstViewObject::contains(KstViewObjectPtr child) {
   for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
-    if ((*i).data() == child.data()) {
-      return true;
-    }
-    if ((*i)->contains(child)) {
+    if ((*i).data() == child.data() || (*i)->contains(child)) {
       return true;
     }
   }
   return false;
 }
 
+
+const QString& KstViewObject::type() const {
+  return _type;
+}
+
+
+void KstViewObject::setMinimumSize(const QSize& sz) {
+  _minimumSize = sz.expandedTo(QSize(3, 3)); // 3,3 is the absolute minimum
+}
+
+
+const QSize& KstViewObject::minimumSize() const {
+  return _minimumSize;
+}
+
 #include "kstviewobject.moc"
 // vim: ts=2 sw=2 et
