@@ -38,6 +38,7 @@ class KstViewWidget : public QWidget {
 
     void paint();
 
+    void nextUpdateDo(KstPaintType updateType);
 
     KstViewObjectPtr findChildFor(const QPoint& pos);
 
@@ -46,6 +47,7 @@ class KstViewWidget : public QWidget {
     virtual void leaveEvent(QEvent *e);
     virtual void mouseMoveEvent(QMouseEvent *e);
     virtual void mousePressEvent(QMouseEvent *e);
+    virtual void mouseDoubleClickEvent(QMouseEvent *e);
     virtual void mouseReleaseEvent(QMouseEvent *e);
     virtual void contextMenuEvent(QContextMenuEvent *e);
     virtual void paintEvent(QPaintEvent *e);
@@ -63,10 +65,12 @@ class KstViewWidget : public QWidget {
 
   private:
     KstTopLevelViewPtr _view;
+    KstViewObject *_vo_datamode;
     bool _dropEnabled : 1;
     bool _dragEnabled : 1;
     KPopupMenu *_menu;
     QDragObject *_drag;
+    KstPaintType _nextUpdate;
 };
 
 #endif
