@@ -35,7 +35,7 @@
 #include <kconfig.h>
 #include <klocale.h>
 
-static const char* const not_close_xpm[]={
+static const char* const not_close_xpm_other[]={
 "5 5 2 1",
 "# c black",
 ". c None",
@@ -184,7 +184,7 @@ void KMdiDockContainer::insertWidget (KDockWidget *dwdg, QPixmap pixmap, const Q
       QToolTip::add( btn, i18n("Switch between overlap and side by side mode", "Overlap") );
 
       btn->setToggleButton( true );
-      btn->setPixmap( const_cast< const char** >(not_close_xpm) );
+      btn->setPixmap( const_cast< const char** >(not_close_xpm_other) );
       hdr->addButton(btn);
       m_overlapButtons.insert(w,btn);
       btn->setOn(!isOverlapMode());
