@@ -33,7 +33,11 @@ void PluginManager::install()
     }
 
     QString tmpFile;
+#if KDE_VERSION >= KDE_MAKE_VERSION(3,3,0)
+    if (!KIO::NetAccess::download(xmlfile, tmpFile, this)) {
+#else
     if (!KIO::NetAccess::download(xmlfile, tmpFile)) {
+#endif
 	KMessageBox::error(this, i18n("Unable to access file %1.").arg(xmlfile.prettyURL()), i18n("KST Plugin Loader"));
 	return;
     }
@@ -56,7 +60,11 @@ void PluginManager::install()
     tmpSoFile.replace(QRegExp(".xml$"), ".so");
     sofile.setPath(tmpSoFile);
 
+#if KDE_VERSION >= KDE_MAKE_VERSION(3,3,0)
+    if (!KIO::NetAccess::dircopy(sofile, pathURL, this)) {
+#else
     if (!KIO::NetAccess::dircopy(sofile, pathURL)) {
+#endif
 	KIO::NetAccess::removeTempFile(tmpFile);
 	KMessageBox::error(this, i18n("Unable to copy plugin file %1 to %2.").arg(sofile.prettyURL()).arg(pathURL.prettyURL()), i18n("KST Plugin Loader"));
 	return;
@@ -66,7 +74,11 @@ void PluginManager::install()
     tmpFileURL.setPath(tmpFile);
     pathURL.setFileName(xmlfile.fileName());
 
+#if KDE_VERSION >= KDE_MAKE_VERSION(3,3,0)
+    if (!KIO::NetAccess::dircopy(tmpFileURL, pathURL, this)) {
+#else
     if (!KIO::NetAccess::dircopy(tmpFileURL, pathURL)) {
+#endif
 	KMessageBox::error(this, i18n("Internal temporary file %1 could not be copied to plugin directory %2.").arg(tmpFile).arg(path), i18n("KST Plugin Loader"));
     }
 
@@ -104,6 +116,7 @@ void PluginManager::rescan()
 {
     PluginCollection::self()->rescan();
     reloadList();
+    emit rescanned();
 }
 
 
