@@ -39,6 +39,7 @@ KstStdinSource::KstStdinSource(KConfig *cfg)
 : KstDataSource(cfg, "stdin", "stdin") {
   _file = new KTempFile;
   _filename = _file->name();
+  // Unfortunately we have to update here.  stdin is a special case.
   update();
   _src = KstDataSource::loadSource(_filename, "ASCII");
   if (_src && _src->isValid()) {
@@ -56,6 +57,10 @@ KstStdinSource::~KstStdinSource() {
 
 
 KstObject::UpdateType KstStdinSource::update(int u) {
+  if (KstObject::checkUpdateCounter(u)) {
+    return lastUpdateResult();
+  }
+
   fd_set rfds;
   struct timeval tv;
   int retval;
@@ -67,7 +72,7 @@ KstObject::UpdateType KstStdinSource::update(int u) {
   FILE *fp = _file->fstream();
 
   if (!fp) {
-    return KstObject::NO_CHANGE;
+    return setLastUpdateResult(KstObject::NO_CHANGE);
   }
 
   do {
@@ -93,11 +98,10 @@ KstObject::UpdateType KstStdinSource::update(int u) {
 
   fflush(fp);
 
-  if (got_some) {
-    return _src ? _src->update(u) : KstObject::NO_CHANGE;
-  } else {
-    return KstObject::NO_CHANGE;
+  if (got_some && _src) {
+    return setLastUpdateResult(_src->update(u));
   }
+  return setLastUpdateResult(KstObject::NO_CHANGE);
 }
 
 
