@@ -22,12 +22,13 @@
 
 class ThreadEvent : public QEvent {
   public:
-    enum ThreadEventType { Unknown = 0, UpdateDataDialogs, UpdateAllDialogs, Repaint, Done };
+    enum ThreadEventType { Unknown = 0, UpdateDataDialogs, UpdateAllDialogs, Repaint, Done, NoUpdate };
 
     ThreadEvent(ThreadEventType et) : QEvent(QEvent::Type(KstEventTypeThread)), _eventType(et) {}
     virtual ~ThreadEvent() {}
 
     ThreadEventType _eventType;
+    QValueList<KstBaseCurve*> _curves; // HACK: for temporary use in update reworking
 };
 
 
