@@ -14,6 +14,7 @@ double calculate_matrix_entry( double dX, int iPos ) {
   return dY;
 }
 
+extern "C" int parameterName(int iIndex, char** pName);
 extern "C" int kstfit_polynomial_unweighted(
   const double *const inArrays[], 
   const int inArrayLens[],
@@ -40,3 +41,12 @@ int kstfit_polynomial_unweighted(
   
   return iRetVal;
 }
+
+int parameterName(int iIndex, char** pName) {
+  char name[20];
+
+  sprintf( name, "x^%d", iIndex );
+  *pName = strdup( name );
+
+  return 1;
+}
